<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief admin wiki
 *
 * This controller adds admin functionalities for the admin panel
 */

class admin_wiki extends wiki
{
  function langExist($code)
  {
    $code = $this->con->escapeString($code);

    $query = <<<SQL
      SELECT id
      FROM  {prefix}lang
      WHERE code='$code'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    return !empty($result);
  }

  function addLang($code, $description, $flag)
  {
    $code        = $this->con->escapeString($code);
    $description = $this->con->escapeString($description);
    $flag        = $this->con->escapeString($flag);

    $query =<<<SQL
      INSERT INTO {prefix}lang
      (code, description, flag)
      VALUES('$code', '$description', '$flag');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  function delLang($id)
  {
    $id = $this->con->escapeString($id);

    $allLangs     = $this->getLangs();
    $masterLangId = lang::getIdFromCode($allLangs, wc_default_lang);

    // in table content, delete all contents that are set on this lang
    $query = <<<SQL
      SELECT id FROM {prefix}content
      WHERE lang_fk='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $tids = $this->con->select($query);
    foreach ($tids as $tid)
    {
      $nid = $tid[0];
      $this->deleteContentById($nid);
    }

    // in table user, remove all occurences of lang to remove by default lang
    $query =<<<SQL
      UPDATE {prefix}user
      SET lang_fk='$masterLangId'
      WHERE lang_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $ok = $this->con->execute($query);

    // finally, delete the lang
    $query = <<<SQL
      DELETE FROM {prefix}lang
      WHERE id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  function modifyLang($id, $code, $description, $flag)
  {
    $id          = $this->con->escapeString($id);
    $code        = $this->con->escapeString($code);
    $description = $this->con->escapeString($description);
    $flag        = $this->con->escapeString($flag);

    $query = <<<SQL
      UPDATE {prefix}lang
      SET code='$code', description='$description', flag='$flag'
      WHERE id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  function fixParentId($content_id, $new_parent_content_fk)
  {
    $content_id            = $this->con->escapeString($content_id);
    $new_parent_content_fk = $this->con->escapeString($new_parent_content_fk);

    $query = <<<SQL
      UPDATE {prefix}content_hierarchy
      SET parent_content_fk='$new_parent_content_fk'
      WHERE content_fk='$content_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  function addAnonymous($lang_fk)
  {
    $lang_fk    = $this->con->escapeString($lang_fk);
    $user_value = user_value;
    $time = time();

    $query =<<<SQL
      INSERT INTO {prefix}user
      (login, firstname, lastname, passwd, email, admin, lang_fk, timestamp)
      VALUES('anonymous', 'anonymous', 'anonymous', '', 'anonymous@anonymous.com', '$user_value', '$lang_fk', '$time');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  function addAdministrator($login, $fname, $lname, $password, $email, $lang_fk, $lang_ui)
  {
    $login       = $this->con->escapeString($login);
    $fname       = $this->con->escapeString($fname);
    $lname       = $this->con->escapeString($lname);
    $password    = $this->con->escapeString($password);
    $email       = $this->con->escapeString($email);
    $lang_fk     = $this->con->escapeString($lang_fk);
    $lang_ui     = $this->con->escapeString($lang_ui);
    $admin_value = admin_value;
    $time = time();

    $query =<<<SQL
      INSERT INTO {prefix}user
      (login, firstname, lastname, passwd, email, admin, lang_fk, lang_ui, timestamp)
      VALUES('$login', '$fname', '$lname', MD5('$password'), '$email', '$admin_value', '$lang_fk', '$lang_ui', '$time');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  function getAllUsersRights()
  {
    $anonymous_id = user_anonymous_id;

    // get rights
    $query = <<<SQL
      SELECT id, login, admin, temp_key
      FROM   {prefix}user
      WHERE  id!='$anonymous_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->select($query);
  }

  function updateUsersRights($usersToUpdate)
  {
    // update rights
    foreach ($usersToUpdate as $user)
    {
      $user[0] = $this->con->escapeString($user[0]);
      $user[1] = $this->con->escapeString($user[1]);

      $query = <<< SQL
        UPDATE {prefix}user
        SET    admin='$user[1]'
        WHERE  id='$user[0]'
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $this->con->execute($query);
    }
  }

  function getUserById($id)
  {
    $id = $this->con->escapeString($id);

    $query =<<<SQL
      SELECT {prefix}user.login, {prefix}user.passwd, {prefix}user.admin, {prefix}user.firstname,
             {prefix}user.lastname, {prefix}user.email, {prefix}user.lang_ui, {prefix}user.timestamp,
             {prefix}lang.code
      FROM   {prefix}user
      JOIN   {prefix}lang on {prefix}lang.id={prefix}user.lang_fk
      WHERE  {prefix}user.id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    if (empty($result))
    {
      return $this->anonymousUser();
    }
    else
    {
      // $id, $login, $password, $admin, $first_name, $last_name, $email, $lang_fk, $timestamp
      return new user($id, $result[0], $result[1], $result[2], $result[3], $result[4], $result[5], $result[8], $result[6], $result[7]);
    }
  }

  function confirmUserById($id)
  {
    $id = $this->con->escapeString($id);

    $query =<<< SQL
      UPDATE {prefix}user
      SET temp_key=NULL
      WHERE id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  function deleteUserById($id)
  {
    $id = $this->con->escapeString($id);

    // remove the user from table user_group
    $query = <<<SQL
      DELETE FROM {prefix}user_group
      WHERE user_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // remove the user from table content_user_email
    $query = <<<SQL
      DELETE FROM {prefix}content_user_email
      WHERE user_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // update contents by modifying user to anonymous
    $user_anonymous_id = user_anonymous_id;
    $query = <<<SQL
      UPDATE {prefix}content
      SET user_fk='$user_anonymous_id'
      WHERE user_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // update history by modifying user to anonymous
    $user_anonymous_id = user_anonymous_id;
    $query = <<<SQL
      UPDATE {prefix}history
      SET user_fk='$user_anonymous_id'
      WHERE user_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // update attachments by modifying user to anonymous
    $user_anonymous_id = user_anonymous_id;
    $query = <<<SQL
      UPDATE {prefix}attachments
      SET user_fk='$user_anonymous_id'
      WHERE user_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // update comments by info stored in user
    $user = $this->getUserById($id);
    $query = <<<SQL
      UPDATE {prefix}comment
      SET name='$user->login', email='$user->email', user_fk='$user_anonymous_id'
      WHERE user_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // delete user
    $query =<<< SQL
      DELETE FROM {prefix}user
      WHERE id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  function groupExist($groupName)
  {
    $groupName = $this->con->escapeString($groupName);

    $query = <<<SQL
      SELECT id
      FROM  {prefix}groups
      WHERE name='$groupName'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    return !empty($result);
  }

  function addGroup($groupName)
  {
    $groupName = $this->con->escapeString($groupName);

    $query = <<<SQL
      INSERT INTO {prefix}groups
      (name) VALUES('$groupName');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->insertAndGetId($query);
  }

  function delGroup($id)
  {
    $id = $this->con->escapeString($id);

    // remove the group from table user_group
    $query = <<<SQL
      DELETE FROM {prefix}user_group
      WHERE group_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // remove the group from table content_group
    $query = <<<SQL
      DELETE FROM {prefix}content_group
      WHERE group_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // remove the group from table group
    $query = <<<SQL
      DELETE FROM {prefix}groups
      WHERE id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  function getGroup($id)
  {
    $id = $this->con->escapeString($id);

    // the outer join means we want result from the left part of the join (table group)
    // even if there is no result in the right part (table user_group)
    // in that case, user_group values will appear as NULL
    $query = <<<SQL
      SELECT {prefix}groups.id, {prefix}groups.name, {prefix}user.id, {prefix}user.login
      FROM   {prefix}groups
      LEFT JOIN {prefix}user_group ON {prefix}groups.id={prefix}user_group.group_fk
      LEFT JOIN {prefix}user       ON {prefix}user.id={prefix}user_group.user_fk
      WHERE {prefix}groups.id='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    foreach ($result as $r)
    {
      if (empty($group))
      {
        $group = new group($r[0], $r[1]);
      }

      if ($r[2] != 0)
      {
        $group->addUser($r[2], $r[3]);
      }
    }

    return $group;
  }

  function addUsersToGroup($id, $userIds)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      INSERT INTO {prefix}user_group
      (group_fk, user_fk) VALUES
SQL;

    $first = true;
    foreach ($userIds as $user_id)
    {
      $user_id = $this->con->escapeString($user_id);

      if (!$first)
      {
        $query .= ',';
      }

      $query .= "('".$id."', '".$user_id."')";
      $first = false;
    }

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcClean();
  }

  function removeUsersFromGroup($id, $userIds)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      DELETE FROM {prefix}user_group
      WHERE group_fk='$id' AND user_fk IN (
SQL;

    $first = true;
    foreach ($userIds as $user_id)
    {
      $user_id = $this->con->escapeString($user_id);

      if (!$first)
      {
        $query .= ',';
      }

      $query .= $user_id;
      $first = false;
    }
    $query .= ')';

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  function metaDataExist($metaName)
  {
    $metaName = $this->con->escapeString($metaName);

    $query = <<<SQL
      SELECT id
      FROM  {prefix}metadata
      WHERE name='$metaName'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    return !empty($result);
  }

  function addMetaData($metaName)
  {
    $metaName = $this->con->escapeString($metaName);

    $query = <<<SQL
      INSERT INTO {prefix}metadata
      (name) VALUES('$metaName');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->insertAndGetId($query);
  }

  function addMetaDataWithType($metaName, $type)
  {
    $metaName = $this->con->escapeString($metaName);
    $type = $this->con->escapeString($type);

    $query = <<<SQL
      INSERT INTO {prefix}metadata
      (name, type) VALUES('$metaName', '$type');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->insertAndGetId($query);
  }

  function updateMetadatas($metaToUpdate)
  {
    // update metadatas
    foreach ($metaToUpdate as $id=>$type)
    {
      $id   = $this->con->escapeString($id);
      $type = $this->con->escapeString($type);

      $query = <<< SQL
        UPDATE {prefix}metadata
        SET    type='$type'
        WHERE  id='$id'
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $this->con->execute($query);

      // remove the metadata from table metadata_content
      $query = <<<SQL
        DELETE FROM {prefix}metadata_content
        WHERE metadata_fk='$id'
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $this->con->execute($query);
    }
  }

  function delMetaData($id)
  {
    $id = $this->con->escapeString($id);

    // remove the metadata from table metadata_content
    $query = <<<SQL
      DELETE FROM {prefix}metadata_content
      WHERE metadata_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // remove the group from table metadata
    $query = <<<SQL
      DELETE FROM {prefix}metadata
      WHERE id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  function getMetaDataById($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      SELECT id, name, type
      FROM {prefix}metadata
      WHERE id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $r = $this->con->select1($query);

    return new metadata($r[0], $r[1], $r[2]);
  }

  function getMetaDataInfoByMetadataId($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      SELECT {prefix}metadata_content.id, {prefix}metadata_content.value,
             {prefix}content.title, {prefix}lang.code
      FROM   {prefix}metadata_content
      JOIN   {prefix}content on {prefix}content.id={prefix}metadata_content.content_fk
      JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
      WHERE  {prefix}metadata_content.metadata_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->select($query);
  }

  function existMetadataClosedValue($id, $value)
  {
    $id    = $this->con->escapeString($id);
    $value = $this->con->escapeString($value);

    $query = <<<SQL
      SELECT id
      FROM {prefix}metadata_value
      WHERE metadata_fk='$id'
      AND   value='$value'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    return !empty($result);
  }

  function addMetadataClosedValue($id, $value)
  {
    $id    = $this->con->escapeString($id);
    $value = $this->con->escapeString($value);

    $query = <<<SQL
      INSERT INTO {prefix}metadata_value
      (metadata_fk, value) VALUES('$id', '$value');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcCleanAll();

    return $result;
  }

  function removeClosedValuesFromMetadata($valueIds)
  {
    // create list of ids suitable for sql query
    $listIds = '';

    $first = true;
    foreach ($valueIds as $id)
    {
      $id = $this->con->escapeString($id);

      if (!$first)
      {
        $listIds .= ',';
      }

      $listIds .= $id;
      $first = false;
    }

    // metadata_content
    $query = <<<SQL
      DELETE FROM {prefix}metadata_content
      WHERE metadata_value_fk IN ($listIds)
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // metadata_value
    $query = <<<SQL
      DELETE FROM {prefix}metadata_value
      WHERE id IN ($listIds)
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcCleanAll();
  }

  function getMetaDataClosedInfoByMetadataId($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      SELECT {prefix}metadata_content.id, {prefix}metadata_value.value,
             {prefix}content.title, {prefix}lang.code
      FROM   {prefix}metadata_content
      JOIN   {prefix}metadata_value on {prefix}metadata_value.id={prefix}metadata_content.metadata_value_fk
      JOIN   {prefix}content on {prefix}content.id={prefix}metadata_content.content_fk
      JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
      WHERE  {prefix}metadata_content.metadata_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->select($query);
  }

  function getBannedIps()
  {
    $query = <<<SQL
      SELECT id, ip
      FROM   {prefix}ban
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->select($query);
  }

  function delBan($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      DELETE
      FROM   {prefix}ban
      WHERE  id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  function addbans($ipsToBan)
  {
    // try to see which ips are already bans
    $query = <<<SQL
      SELECT ip
      FROM   {prefix}ban
      WHERE  ip in (
SQL;

    $first = true;
    foreach ($ipsToBan as $ip)
    {
      if (!$first)
      {
        $query .= ',';
      }
      $query .= "'".$ip."'";
      $first = false;
    }
    $query .= ')';

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    // transform result
    $alreadyBans = array();
    foreach ($result as $r)
    {
      $alreadyBans[] = $r[0];
    }

    // remove all ips already bans
    foreach ($ipsToBan as $i=>$ip)
    {
      if (in_array($ip, $alreadyBans))
      {
        unset($ipsToBan[$i]);
      }
    }

    if (sizeof($ipsToBan) != 0)
    {
      // and insert all new ips
      $query = <<<SQL
      INSERT INTO {prefix}ban
      (ip)
SQL;

      $first = true;
      foreach ($ipsToBan as $ip)
      {
        if (!$first)
        {
          $query .= ',';
        }
        $query .= "VALUES ('".$ip."')";
        $first = false;
      }

      $query = str_replace('{prefix}', $this->prefix, $query);
      return $this->con->execute($query);
    }

    return false;
  }

  function recComputeAndSetNestedSetModel($adjacency, $id, $count)
  {
    $left  = $count;
    $right = $left + 1;

    if (!empty($adjacency[$id]))
    {
      $childs = $adjacency[$id];
      foreach ($childs as $child)
      {
        $right  = $this->recComputeAndSetNestedSetModel($adjacency, $child, $right) + 1;
      }
    }

    $query = <<< SQL
      UPDATE {prefix}content_hierarchy
      SET left_n='$left', right_n='$right'
      WHERE content_fk='$id'
SQL;
    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    return $right;
  }

  function computeNestedSetModel()
  {
    $this->con->execute('START TRANSACTION');

    $query = <<<SQL
      SELECT content_fk, parent_content_fk FROM {prefix}content_hierarchy
      ORDER BY content_fk ASC
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    // create a list parent_id => list(child_id)
    $root_id = 0;
    $adjacency = array();
    foreach ($result as $r)
    {
      $id = $r[0];
      $p  = $r[1];

      if (empty($adjacency[$p]))
      {
        $adjacency[$p] = array();
      }

      if ($id != $p /*not root node*/)
      {
        $adjacency[$p][] = $id;
      }
      else
      {
        $root_id = $id;
      }
    }

    // now recursively compute and set right_n and left_n
    $this->recComputeAndSetNestedSetModel($adjacency, $root_id, 1);
    $this->con->execute('COMMIT');
  }

  function createQueryRemoveUnknownId($table_target, $fk_name, $table_src, $pk_name)
  {
    // example with
    // $table_target = content_group
    // $fk_name      = group_fk
    // $table_src    = groups
    // $pk_name      = id

    // DELETE {prefix}content_group FROM {prefix}content_group
    // LEFT JOIN {prefix}groups on {prefix}groups.id={prefix}content_group.group_fk
    // WHERE ISNULL({prefix}groups.id)

    return
    'DELETE {prefix}'.$table_target.' FROM {prefix}'.$table_target.' '.
    'LEFT JOIN {prefix}'.$table_src.' '.
    'ON {prefix}'.$table_src.'.'.$pk_name.'={prefix}'.$table_target.'.'.$fk_name.' '.
    'WHERE ISNULL({prefix}'.$table_src.'.'.$pk_name.')';
  }

  function removeUnknownId($table_target, $fk_name, $table_src, $pk_name)
  {
    $query = $this->createQueryRemoveUnknownId($table_target, $fk_name, $table_src, $pk_name);

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  function createQueryReplaceUnknownId($table_target, $fk_name, $table_src, $pk_name, $newId)
  {
    // example with
    // $table_target = comment
    // $fk_name      = user_fk
    // $table_src    = user
    // $pk_name      = id

    // UPDATE {prefix}comment
    // LEFT JOIN {prefix}user ON {prefix}user.id={prefix}comment.user_fk
    // SET {prefix}comment.user_fk='$newId'
    // WHERE ISNULL({prefix}user.id)

    return
    'UPDATE {prefix}'.$table_target.' '.
    'LEFT JOIN {prefix}'.$table_src.' '.
    'ON {prefix}'.$table_src.'.'.$pk_name.'={prefix}'.$table_target.'.'.$fk_name.' '.
    'SET {prefix}'.$table_target.'.'.$fk_name.'='.$newId.' '.
    'WHERE ISNULL({prefix}'.$table_src.'.'.$pk_name.')';
  }

  function replaceUnknownId($table_target, $fk_name, $table_src, $pk_name, $newId)
  {
    $query = $this->createQueryReplaceUnknownId($table_target, $fk_name, $table_src, $pk_name, $newId);

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  function canonize()
  {
    $ok = true;

    $allLangs     = $this->getLangs();
    $masterLangId = lang::getIdFromCode($allLangs, wc_default_lang);

    // LANG *******************************************************************
    if ($ok)
    {
      // update unknown lang_fk to default lang in content
      $ok = $this->replaceUnknownId('content', 'lang_fk', 'lang', 'id', $masterLangId);
    }

    if ($ok)
    {
      // update unknown lang_fk to default lang in user
      $ok = $this->replaceUnknownId('user', 'lang_fk', 'lang', 'id', $masterLangId);
    }

    // USERS ******************************************************************
    if ($ok)
    {
      // remove unknown user_fk from user_group table
      $ok = $this->removeUnknownId('user_group', 'user_fk', 'user', 'id');
    }

    if ($ok)
    {
      // remove unknown user_fk from content_user_email table
      $ok = $this->removeUnknownId('content_user_email', 'user_fk', 'user', 'id');
    }

    $user_anonymous_id = user_anonymous_id;
    if ($ok)
    {
      // remove unknown user_fk from content table, replacing it by anonymous_id
      $ok = $this->replaceUnknownId('content', 'user_fk', 'user', 'id', $user_anonymous_id);
    }

    if ($ok)
    {
      // remove unknown user_fk from history table, replacing it by anonymous_id
      $ok = $this->replaceUnknownId('history', 'user_fk', 'user', 'id', $user_anonymous_id);
    }

    if ($ok)
    {
      // remove unknown user_fk from comment table, replacing it by anonymous_id
      $ok = $this->replaceUnknownId('comment', 'user_fk', 'user', 'id', $user_anonymous_id);
    }

    if ($ok)
    {
      // remove unknown user_fk from attachments table, replacing it by anonymous_id
      $ok = $this->replaceUnknownId('attachments', 'user_fk', 'user', 'id', $user_anonymous_id);
    }

    // GROUPS *****************************************************************
    if ($ok)
    {
      // remove unknown group_fk from content_group table
      $ok = $this->removeUnknownId('content_group', 'group_fk', 'groups', 'id');
    }

    if ($ok)
    {
      // remove unknown group_fk from user_group table
      $ok = $this->removeUnknownId('user_group', 'group_fk', 'groups', 'id');
    }

    // CONTENT ****************************************************************
    if ($ok)
    {
      // remove unknown content_fk from history
      $ok = $this->removeUnknownId('history', 'content_fk', 'content', 'id');
    }

    if ($ok)
    {
      // remove unknown content_fk from comment
      $ok = $this->removeUnknownId('comment', 'content_fk', 'content', 'id');
    }

    if ($ok)
    {
      // remove unknown content_fk from content_group
      $ok = $this->removeUnknownId('content_group', 'content_fk', 'content', 'id');
    }

    if ($ok)
    {
      // remove unknown content_fk from content_hierarchy
      $ok = $this->removeUnknownId('content_hierarchy', 'content_fk', 'content', 'id');
    }

    if ($ok)
    {
      // remove unknown content_fk from content_lang
      $ok = $this->removeUnknownId('content_lang', 'content_fk', 'content', 'id');
    }

    if ($ok)
    {
      // remove unknown content_fk from content_user_email
      $ok = $this->removeUnknownId('content_user_email', 'content_fk', 'content', 'id');
    }

    if ($ok)
    {
      // remove unknown content_fk from metadata_content
      $ok = $this->removeUnknownId('metadata_content', 'content_fk', 'content', 'id');
    }

    if ($ok)
    {
      // remove unknown content_fk from attachments
      $ok = $this->removeUnknownId('attachments', 'content_fk', 'content', 'id');
    }

    // HIERARCHY/TRANSLATIONS *************************************************
    if ($ok)
    {
      // remove content_hierarchy relationship for translated nodes
      $query =<<<SQL
        DELETE     {prefix}content_hierarchy
        FROM       {prefix}content_hierarchy
        INNER JOIN {prefix}content_lang
        WHERE      {prefix}content_hierarchy.content_fk={prefix}content_lang.content_fk
        AND        {prefix}content_lang.content_fk!={prefix}content_lang.related_content_fk
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $ok = $this->con->execute($query);
    }

    if ($ok)
    {
      // change lang for translated nodes that are master nodes but do not have the master lang
      $query =<<<SQL
        UPDATE     {prefix}content
        LEFT JOIN  {prefix}content_lang ON {prefix}content_lang.content_fk={prefix}content.id
        SET        {prefix}content.lang_fk='$masterLangId'
        WHERE      {prefix}content_lang.content_fk={prefix}content_lang.related_content_fk
        AND        {prefix}content.lang_fk!='$masterLangId'
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $ok = $this->con->execute($query);
    }

    if ($ok)
    {
      // change parent for nodes that have a parent which is not a master node
      $query =<<<SQL
        UPDATE    {prefix}content_hierarchy
        LEFT JOIN {prefix}content_lang ON {prefix}content_lang.content_fk={prefix}content_hierarchy.parent_content_fk
        SET       {prefix}content_hierarchy.parent_content_fk={prefix}content_lang.related_content_fk
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $ok = $this->con->execute($query);
    }

    // METADATA ***************************************************************
    if ($ok)
    {
      // remove unknown metadata_fk from metadata_content table
      $ok = $this->removeUnknownId('metadata_content', 'metadata_fk', 'metadata', 'id');
    }

    if ($ok)
    {
      // remove unknown metadata_fk from metadata_value table
      $ok = $this->removeUnknownId('metadata_value', 'metadata_fk', 'metadata', 'id');
    }

    if ($ok)
    {
      // remove unknown metadata_value_fk from metadata_content table
      $ok = $this->removeUnknownId('metadata_content', 'metadata_value_fk', 'metadata_value', 'id');
    }

    return $ok;
  }
}
?>