<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

$adminBoxHandle['wiki_pref']                      = 'wikiPreferencesDisplay';

$adminActionHandle['wiki_pref']                   = 'onWikiPreferencesSave';
$adminActionHandle['wiki_pref_generate_htaccess'] = 'onWikiPreferencesGenerateHtAccess';
$adminActionHandle['wiki_pref_delete_htaccess']   = 'onWikiPreferencesDeleteHtAccess';
$adminActionHandle['wiki_pref_clean_cache']       = 'onWikiPreferencesCleanCache';

function getThemes()
{
  $themes = array();

  $themeBaseDir = '../themes/';
  $h = opendir($themeBaseDir);
  while (($file = readdir($h)) !== false)
  {
    if (is_dir($themeBaseDir.$file) && file_exists($themeBaseDir.$file.'/template.php'))
    {
      $themes[] = $file;
    }
  }
  closedir($h);

  return $themes;
}

function wikiPreferencesDisplay()
{
  global $wiki;

  $themes = getThemes();
  $wikiLangs  = $wiki->getLangs();
  $url_format = array('get', 'path_info', 'query_string', 'path_info_mod_rewrite');
  $feed_format = array('atom', 'rss_1', 'rss_2');

  $langs = array();
  foreach ($wikiLangs as $wikiLang)
  {
    $langs[] = $wikiLang->code;
  }

  $table_renderers = array('table', 'big_div');

  define('WICLEAR_INI_TEXT',     1);
  define('WICLEAR_INI_PASSWORD', 2);
  define('WICLEAR_INI_BOOL',     3);
  define('WICLEAR_INI_MULTIPLE', 4);
  define('WICLEAR_INI_INTEGER',  5);

  // fields      : map<config_entry=>config_desc>
  // config_desc : list<type, description, active(optional, def=true), possible_values(optional, def=none)>
  $fields = array(
    'wc_db_host'                       => array(WICLEAR_INI_TEXT,     tr('Sql host')),
    'wc_db_login'                      => array(WICLEAR_INI_TEXT,     tr('Login')),
    'wc_db_password'                   => array(WICLEAR_INI_PASSWORD, tr('Password')),
    'wc_db_database'                   => array(WICLEAR_INI_TEXT,     tr('Database')),
    'wc_db_prefix'                     => array(WICLEAR_INI_TEXT,     tr('Prefix')),
    'wc_default_lang'                  => array(WICLEAR_INI_TEXT,     tr('Default lang'), false),
    'wc_wiki_name'                     => array(WICLEAR_INI_TEXT,     tr('Wiki name')),
    'wc_mail'                          => array(WICLEAR_INI_TEXT,     tr('Reply-to email address')),
    'wc_charset'                       => array(WICLEAR_INI_TEXT,     tr('Charset'), false),
    'wc_wiki_accept_anonymous_content' => array(WICLEAR_INI_BOOL,     tr('Accept anonymous content')),
    'wc_wiki_accept_anonymous_comment' => array(WICLEAR_INI_BOOL,     tr('Accept anonymous comment')),
    'wc_wiki_open_registration'        => array(WICLEAR_INI_BOOL,     tr('Open registration')),
    'wc_login_timeout'                 => array(WICLEAR_INI_INTEGER,  tr('Login timeout'), true),
    'wc_wiki_force_preview'            => array(WICLEAR_INI_BOOL,     tr('Force preview')),
    'wc_wiki_allow_html'               => array(WICLEAR_INI_BOOL,     tr('Allow html')),
    'wc_show_toc'                      => array(WICLEAR_INI_BOOL,     tr('Show table of content')),
    'wc_edit_by_section'               => array(WICLEAR_INI_BOOL,     tr('Allow edition by section')),
    'wc_show_lang_with_title'          => array(WICLEAR_INI_BOOL,     tr('Show lang with title')),
    'wc_table_rendering'               => array(WICLEAR_INI_MULTIPLE, tr('Table rendering'), true, $table_renderers),
    'wc_max_url_title'                 => array(WICLEAR_INI_INTEGER,  tr('Max URL characters in title')),
    'wc_theme'                         => array(WICLEAR_INI_MULTIPLE, tr('Theme'), true, $themes),
    'wc_url_format'                    => array(WICLEAR_INI_MULTIPLE, tr('Url format'), true, $url_format),
    'wc_feed_format'                   => array(WICLEAR_INI_MULTIPLE, tr('Feed format'), true, $feed_format),
    'wc_path'                          => array(WICLEAR_INI_TEXT,     tr('Path on server')),
    'wc_app_path'                      => array(WICLEAR_INI_TEXT,     tr('Application path')),
    'wc_mod_rewrite_app_path'          => array(WICLEAR_INI_TEXT,     tr('Application path when using mod_rewrite')),
    'wc_image_path'                    => array(WICLEAR_INI_TEXT,     tr('Image relative path')),
    'wc_icon_path'                     => array(WICLEAR_INI_TEXT,     tr('Icon relative path')),
    'wc_icon_unknown'                  => array(WICLEAR_INI_INTEGER,  tr('Icon for unknown files'), false),
    'wc_icon_broken'                   => array(WICLEAR_INI_TEXT,     tr('Icon for broken files'), false),
    'wc_icon_pdf'                      => array(WICLEAR_INI_TEXT,     tr('Icon for pdf files'), false),
    'wc_icon_music'                    => array(WICLEAR_INI_TEXT,     tr('Icon for music files'), false),
    'wc_icon_video'                    => array(WICLEAR_INI_TEXT,     tr('Icon for video files'), false),
    'wc_icon_folder'                   => array(WICLEAR_INI_TEXT,     tr('Icon for directories'), false),
    'wc_activate_cache'                => array(WICLEAR_INI_BOOL,     tr('Activate html cache')),
    'wc_cache_path'                    => array(WICLEAR_INI_TEXT,     tr('Cache path'))
  );

  // complete our description with the one from ini
  global $ini;

  foreach ($ini->config->fields as $field=>$value)
  {
    if (!isset($fields[$field]))
    {
      // woops, we're missing this one, add it with a default type so that it is automatically handled
      $fields[$field] = array(WICLEAR_INI_TEXT, $field);
    }
  }

  $form = new form(createAdminURL(array('mode'=>'wiki_pref')), 'post');
  $form->addFieldset();

  if (existVarSession('error'))
  {
    $error = varSession('error');
    unsetVarSession('error');
    $form->addHtml('<h1>'.$error.'</h1>');
  }

  $form->addLegend(tr('Preferences'));
  foreach ($fields as $config_name=>$field_desc)
  {
    $config_type  = $field_desc[0];
    $label        = $field_desc[1];
    $config_value = constant($config_name);

    if (!isset($field_desc[2]) || $field_desc[2]==true)
    {
      $form->addHtml('<p class="field">');
      $form->addLabel($label, 'f'.$config_name);

      // field is active
      switch ($config_type)
      {
        case WICLEAR_INI_TEXT:
        case WICLEAR_INI_INTEGER:
          $form->addInput($config_name, $label, $config_value, 'f'.$config_name);
        break;

        case WICLEAR_INI_PASSWORD:
          $form->addPassword($config_name, $config_value, 'f'.$config_name);
        break;

        case WICLEAR_INI_BOOL:
          $form->addCheckBox($config_name, $config_value=='true');
        break;

        case WICLEAR_INI_MULTIPLE:
          if (is_null($field_desc[3]))
          {
            $values = array();
          }
          else
          {
            $values = $field_desc[3];
          }

          $form->addDropDown($values, $config_value, $config_name, 'f'.$config_name);
        break;
      }

      $form->addHtml('</p>');
    }
    else
    {
      // field is inactive : add it as an hidden field
      $form->addHidden($config_name, $config_value);
    }
  }
  $form->addHtml(
  '<p class="field">'
  );
  $form->addHidden('check', '1');
  $form->addHtml(
  '</p>'.
  '<p class="field">'
  );
  $form->addButton('SavePref', tr('Save preferences'), 'action');
  $form->addHtml(
  '</p>'.
  '<p class="field"/>'
  );

  $form2 = new form(createAdminURL(array('mode'=>'wiki_pref_generate_htaccess')), 'post');
  $form2->addFieldset();
  $form2->addLegend(tr('Generate mod_rewrite .htaccess'));
  $form2->addHtml(
  '<p class="field">'
  );
  $form2->addHidden('check', '1');
  $form2->addHtml(
  '</p>'.
  '<p class="field">'
  );
  $form2->addButton('Generate', tr('Generate .htaccess'), 'action');
  $form2->addHtml(
  '</p>'.
  '<p class="field"/>'
  );

  $form3 = new form(createAdminURL(array('mode'=>'wiki_pref_delete_htaccess')), 'post');
  $form3->addFieldset();
  $form3->addLegend(tr('Delete mod_rewrite .htaccess'));
  $form3->addHtml(
  '<p class="field">'
  );
  $form3->addHidden('check', '1');
  $form3->addHtml(
  '</p>'.
  '<p class="field">'
  );
  $form3->addButton('Delete', tr('Delete .htaccess'), 'action');
  $form3->addHtml(
  '</p>'.
  '<p class="field"/>'
  );

  $form4 = new form(createAdminURL(array('mode'=>'wiki_pref_clean_cache')), 'post');
  $form4->addFieldset();
  $form4->addLegend(tr('Clean cache'));
  $form4->addHtml(
  '<p class="field">'
  );
  $form4->addHidden('check', '1');
  $form4->addHtml(
  '</p>'.
  '<p class="field">'
  );
  $form4->addButton('Clean', tr('Clean cache'), 'action');
  $form4->addHtml(
  '</p>'.
  '<p class="field"/>'
  );

  return $form->toHtml().$form2->toHtml().$form3->toHtml().$form4->toHtml();
}

function onWikiPreferencesSave()
{
  if (!isset($_POST['check']))
  {
    return;
  }

  $new_ini = new ini('');
  $new_ini->file = '../conf/config.ini';
  foreach ($new_ini->config->fields as $field=>$default_value)
  {
    if ($default_value == 'true' || $default_value == 'false')
    {
      // boolean value, rendered as a checkbox : the value exists or not
      $value = !isset($_POST[$field]) ? 'false' : 'true';
    }
    else
    {
      $value = '';
      if (isset($_POST[$field]))
      {
        $value = varPost($field);
      }
    }

    if ($value != $default_value)
    {
      $new_ini->addConstant($field, $value);
    }
  }

  if (!$new_ini->forceSaveFile())
  {
    setVarSession('error', tr('Could not save preferences : are rights OK ?'));
  }
  else
  {
    unsetVarSession('error');
  }

  global $cacheLite;
  $cacheLite->wcCleanAll();

  session_write_close();
  header('Location: '.createHeaderAdminURL(array('mode'=>'wiki_pref')));
}

function generateHtAccessFileName()
{
  $htaccessFileName = wc_inc_path.wc_mod_rewrite_app_path;
  if (!preg_match('!.*/$!', $htaccessFileName))
  {
    // no terminal '/' : add one
    $htaccessFileName .= '/';
  }
  $htaccessFileName .= '.htaccess';

  return $htaccessFileName;
}

function onWikiPreferencesGenerateHtAccess()
{
  if (!isset($_POST['check']))
  {
    return;
  }

  $htaccess =<<<HTACCESS
RewriteEngine On

# Redirect old QueryString url to our new PathInfo
RewriteCond %{QUERY_STRING} ^(.+)$
RewriteRule ^$ {wc_path}%{QUERY_STRING}? [R=permanent,L]

# Prevent Urls beginning with
# -*.php (index.php, feed.php, comment_feed.php, textasimage.php, view_thumb.php)
# -themes
# -images
# -wc_icons
# -tools
# -admin
# to be internally redirected
RewriteRule ^[_0-9a-zA-Z]*.php - [L]
RewriteRule ^themes/ - [L]
RewriteRule ^images/ - [L]
RewriteRule ^wc_icons/ - [L]
RewriteRule ^tools/ - [L]
RewriteRule ^admin/ - [L]

# And rewrite all other urls to PathInfo format
RewriteRule ^(.+)$ {wc_mod_rewrite_path}$1 [L]

HTACCESS;

  // replace {wc_path}
  $wc_path = wc_path;
  if (!preg_match('!.*/$!', $wc_path))
  {
    // no terminal '/' : add one
    $wc_path .= '/';
  }
  $htaccess = str_replace('{wc_path}', $wc_path, $htaccess);

  // replace {wc_mod_rewrite_path}
  $wc_mod_rewrite_path = $wc_path.wc_app_path;
  if (!preg_match('!.*/$!', $wc_mod_rewrite_path))
  {
    // no terminal '/' : add one
    $wc_mod_rewrite_path .= '/';
  }
  $htaccess = str_replace('{wc_mod_rewrite_path}', $wc_mod_rewrite_path, $htaccess);

  // and save the file
  $htaccessFileName = generateHtAccessFileName();

  $fp = @fopen($htaccessFileName, 'w+');
  if ($fp)
  {
    @fwrite($fp, $htaccess);
    @fclose($fp);
  }

  header('Location: '.createHeaderAdminURL(array('mode'=>'wiki_pref')));
}

function onWikiPreferencesDeleteHtAccess()
{
  if (!isset($_POST['check']))
  {
    return;
  }

  $htaccessFileName = generateHtAccessFileName();
  @unlink($htaccessFileName);

  header('Location: '.createHeaderAdminURL(array('mode'=>'wiki_pref')));
}

function onWikiPreferencesCleanCache()
{
  if (!isset($_POST['check']))
  {
    return;
  }

  global $cacheLite;
  $cacheLite->wcCleanAll();

  header('Location: '.createHeaderAdminURL(array('mode'=>'wiki_pref')));
}
?>
