<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

class WiclearWikiRendererConfig extends WikiRendererConfig
{
  var $inlinetags = array(
      'strong'   => array('__','__',      null,null),
      'em'       => array('\'\'','\'\'',  null,null),
      //'code'     => array('@@','@@',      null,null),
      'q'        => array('^^','^^',      array('lang','cite'),null),
      'cite'     => array('{{','}}',      array('title'),null),
      'acronym'  => array('??','??',      array('title'),null),
      'link'     => array('[',']',        array('href','hreflang','title'), 'wiclearwikibuildlink', false),
      'image'    => array('((','))',      array('src','alt','align','longdesc', 'width', 'height'),'wiclearwikibuildimage', false, true),
      'anchor'   => array('~~','~~',      array('name', 'title'),'wiclearwikibuildanchor'),
      'icons'    => array('(|','|)',      array('name'), 'wiclearwikibuildicon'),
      'redirect' => array('&','&',        array('href'), 'wiclearwikiredirect')
   );

   var $simpletags = array('%%%'=>'<br />');

   var $checkWikiWord = true;
   var $checkWikiWordFunction = 'wiclearWikiWordFunction';

   var $bloctags = array(
     'wc_title'=>true,
     'list'=>true,
     'pre'=>true,
     'hr'=>true,
     'blockquote'=>true,
     'definition'=>true,
     'table'=>true,
     'gallerie'=>true,
     'p'=>true
   );

   var $showImage = true;

   //Steve - change starting level of heading to h2 so that it follows on from the title which is h1
  var $minHeaderLevel=2;

  var $doGenerateToc   = false;
  var $doEditBySection = false;

  var $titles = array(); // array<array<level, title, id> >

  function WiclearWikiRendererConfig()
  {
    //$this->simpletags[':-)'] = '<img src="'.createIconURL('laugh.png').'" alt=":-)" />';

    $this->simpletags[':-D'] = '<img src="'.createIconURL('emoticons/evilgrin.png'). '" alt=":-D" />';
    $this->simpletags[':D']  = '<img src="'.createIconURL('emoticons/grin.png').     '" alt=":D" />';
    $this->simpletags[':)']  = '<img src="'.createIconURL('emoticons/happy.png').    '" alt=":)" />';
    $this->simpletags[':-)'] = '<img src="'.createIconURL('emoticons/smile.png').    '" alt=":-)" />';
    $this->simpletags[':-o'] = '<img src="'.createIconURL('emoticons/surprised.png').'" alt=":-o" />';
    $this->simpletags[':-0'] = '<img src="'.createIconURL('emoticons/surprised.png').'" alt=":-0" />';
    $this->simpletags[':-P'] = '<img src="'.createIconURL('emoticons/tongue.png').   '" alt=":-P" />';
    $this->simpletags[':-p'] = '<img src="'.createIconURL('emoticons/tongue.png').   '" alt=":-P" />';
    $this->simpletags[':-('] = '<img src="'.createIconURL('emoticons/unhappy.png').  '" alt=":-(" />';
    $this->simpletags[':(']  = '<img src="'.createIconURL('emoticons/unhappy.png').  '" alt=":(" />';
    $this->simpletags['^_^'] = '<img src="'.createIconURL('emoticons/waii.png').     '" alt="^_^" />';
    $this->simpletags[';-)'] = '<img src="'.createIconURL('emoticons/wink.png').     '" alt=";-)" />';
    $this->simpletags[',-)'] = '<img src="'.createIconURL('emoticons/wink.png').     '" alt=",-)" />';
  }

  function reset()
  {
    $this->titles = array();
  }

  function addTitle($level, $title, $line)
  {
    $id = sizeof($this->titles);
    $title = strip_tags($title);
    $this->titles[] = array($level, $title, $id, $line);
    return $id;
  }

  function generateTOC()
  {
    $html = '';
    // Check to see if there are any titles!
    if (count($this->titles) > 0)
    {
      // sdw - modified to display as ordered list
      $html = "\n <h2>".tr('Contents').'</h2>';
      $html .= '<ul class="toc">' ;

      foreach ($this->titles as $levelTitleId)
      {
        $level = $levelTitleId[0];
        $title = $levelTitleId[1];
        $id    = $levelTitleId[2];
        $html .= '<li class="toclevel'.$level.'"><a href="#toc_'.$id.'"  title="Go to section - '.$title.'">'.$title.'</a></li>'."\n";
      }
      $html .= '</ul>'."\n";
    }

    return $html;
  }
}

/**
 * handle wiclear title
 */
class WRB_wc_title extends WRB_title
{
  function _renderInlineTag($string)
  {
    $html = $rendered = parent::_renderInlineTag($string);

    global $wikiRenderer;
    // keep track of titles
    $hx = $this->getRenderedLevel();
    $id = $wikiRenderer->config->addTitle($hx, $html, $wikiRenderer->num);

    if ($wikiRenderer->config->doGenerateToc)
    {
      $html = '<a name="toc_'.$id.'" id="toc_'.$id.'"></a>'.$html;
    }
    if ($wikiRenderer->config->doEditBySection)
    {
      global $node;
      if (!empty($node))
      {
        $url = createURL($node->title, array('mode'=>'edit', 'lang'=>$node->lang->code, 'edit_section'=>$id));
        $html .= ' <a class="section_edit" href="'.$url.'" title="'.tr('Edit section').' - '.htmlspecialchars($rendered).'">'.tr('Edit').'</a> ';
      }
    }

    return $html;
  }
}

/**
 * handle galleries
 */
class WRB_gallerie extends WikiRendererBloc
{
  var $type='gallery';
  var $regexp='/%%(.*)%%/';
  var $_closeNow=true;

  function getRenderedLine()
  {
    $contents = explode('|', $this->_detectMatch[1]);
    $cnt = count($contents);
    if ($cnt > 4)
    {
      $cnt = 4;
    }

    $height = -1;
    $width  = -1;
    $title  = '';

    switch($cnt)
    {
    case 4:
      $height = $contents[3];
    case 3:
      $width  = $contents[2];
    case 2:
      $title  = $contents[1];
    case 1:
    default:
      $dir    = $contents[0];
    }

    $html = '';
    if (!empty($title))
    {
      $html = '<h1>'.$title.'</h1>';
    }

    if ($width == -1) $width = 50;
    if ($height == -1) $height = 50;

    global $wikiRenderer;
    if ($wikiRenderer->config->showImage == true)
/*  does not work in PHP4... Strange
    if ($this->engine->config->showImage == true)*/
    {
      $html .= displayAttachmentBrowser($dir, $width, $height, false);
    }
    else
    {
      $html .= '<p>'.tr('Gallery content').'</p>';
    }

    return $html;
  }
}

function wiclearWikiWordFunction($words)
{
  $result = array();

  // iterate over all words
  foreach ($words as $w)
  {
    // by default, the result of '$w' is '$r'
    $r = $w;

    // count number of upper characters
    $nbUpper = 0;
    for ($i = 0; $i < strlen($w); ++$i)
    {
      if (ctype_upper($w[$i]))
      {
        ++$nbUpper;
        if ($nbUpper == 2)
        {
          // 2 found, not necessary to go ahead
          break;
        }
      }
    }

    if ($nbUpper == 2)
    {
      // WikiWord : build a link
      $r = '<a href="'.createURL($w).'">'.$w.'</a>';
    }

    $result[] = $r;
  }

  return $result;
}

function protocolToClass($url)
{
  if (strpos($url, 'mailto:') > -1 )
  {
    return 'mailto';
  }
  else
  {
    $urlExp = explode(':/', $url);

    if (count($urlExp) > 1)
    {
      // get protocol bit i.e. http, https, rtsp
      $prot = strtolower(trim($urlExp[0]));

      // protocol types 
      $urlTypes = array(
        'http'  => 'httplnk',
        'https' => 'httpslnk',
        'rtsp'  => 'streamlnk',
        'mms'   => 'streamlnk',
        'ftp'   => 'ftplnk'
      );

      //check to see if protocol is in utrlTypes array
      $class = isset($urlTypes[$prot]) ? $urlTypes[$prot] : 'offsitelnk';

      return $class;
    }
  }
}

function wiclearwikibuildlink($contents, $attr)
{
  global $user_lang;
  global $user;

  $cnt = count($contents);
  $attribut = '';
  $url_options = array();
  if (!empty($user) && $user_lang != $user->lang)
  {
    $url_options['lang'] = $user_lang;
  }

  if ($cnt > 1)
  {
    $contents[1] = trim($contents[1]);

    if ($cnt > count($attr))
    {
      $cnt = count($attr)+1;
    }
    if (strpos($contents[1], 'javascript:') !== false) // for security reason
    {
      $contents[1] = '#';
    }
    if (strpos($contents[1], ':/') === false && strpos($contents[1], 'mailto:') === false)
    {
      $contents[1] = createURL($contents[1], $url_options);
    }
    else
    {
      // embeds offsite url description in class
      $attribut .= ' rel="nofollow" class ="offsite '.protocolToClass($contents[1]).'"';
    }

    for($i = 1; $i < $cnt; $i++)
    {
      $attribut .= ' '.$attr[$i-1].'="'.$contents[$i].'"';
    }
  }
  else
  {
    if (!isset($contents[0]))
    {
      $contents[0] = '#';
    }

    if (strpos($contents[0], 'javascript:') !== false) // for security reason
    {
      $contents[0] = '#';
    }

    $attribut = ' href="'.createURL(trim($contents[0]), $url_options).'"';
    if (utf8_helper::strlen($contents[0]) > wc_max_url_title)
    {
      $contents[0] = utf8_helper::substr($contents[0], 0, wc_max_url_title).'(..)';
    }
    $contents[0] = formatHtml($contents[0]);
  }

  return '<a'.$attribut.'>'.$contents[0].'</a>';
}

function wiclearwikibuildanchor($contents, $attr)
{
  $cnt = count($contents);
  if ($cnt == 0)
  {
    return;
  }

  $title = '';
  if ($cnt > 1)
  {
    $title = $contents[1];
  }

  $anchor = $contents[0];
  if (!preg_match('/[a-zA-Z][a-zA-Z0-9_:\.-]*/', $anchor))
  {
    // invalid id !
    // anyway, try to make a conformant id by adding a front letter to it
    $anchor = 'a'.$anchor;
  }

  return '<a id="'.$anchor.'" name="'.$anchor.'">'.$title.'</a>';
}

function wiclearwikibuildimage($contents, $attr)
{
  $cnt = count($contents);
  $attribut = '';
  if ($cnt > 6)
  {
    $cnt = 6;
  }

  $height = -1;
  $width  = -1;
  $alt = tr('Image (hidden)');

  switch($cnt)
  {
  case 6:
    $height = $contents[5];
  case 5:
    $width = $contents[4];
  case 4:
    $attribut.=' longdesc="'.rawurlencode($contents[3]).'"';
  case 3:
    if($contents[2] == 'l' || $contents[2] == 'L' || $contents[2] == 'g' || $contents[2] == 'G')
    {
      $attribut .= ' style="float:left;"';
    }
    elseif
    ($contents[2] == 'r' || $contents[2] == 'R' || $contents[2] == 'd' || $contents[2] == 'D')
    {
      $attribut .= ' style="float:right;"';
    }
  case 2:
    $alt = $contents[1];
    $attribut .= ' alt="'.$alt.'"';
  case 1:
    if (strpos($contents[0], ':/') === false)
    {
      $attribut .= ' src="'.createImageURL($contents[0], $width, $height).'"';
    }
    else
    {
      $attribut .= ' src="'.$contents[0].'"';
      // let's ignore those attributes when image is a remote url
      $width  = -1;
      $height = -1;
    }
    if($cnt == 1)
    {
      $attribut.=' alt=""';
    }
  break;
  default:
    $attribut = '';
    $alt      = '';
  }

  global $wikiRenderer;
  if ($wikiRenderer->config->showImage == true)
  {
    $html = '<img'.$attribut.' />';

    if ($width != -1 || $height != -1)
    {
      $html = '<a href="'.createImageURL($contents[0]).'">'.$html.'</a>';
    }
  }
  else
  {
    $html = $alt;
  }

  return $html;
}

/* Steve create icon based on name */
function wiclearwikibuildicon($name = '')
{
  global $wikiRenderer;

  if (!isset($name[0])) 
  {
    return '';
  }
  $icon = $name[0];

  if ($icon != '' && $icon != null)
  {
    if ($wikiRenderer->config->showImage == true)
    {
      $html = '<img class="icon" src="'.createIconUrl('icons/'.$icon.'.png').'" alt="'.$icon.'" title="'.$icon.'" />';
      return $html;
    }
    else
    {
      return ' '.$icon.' ';
    }
  }
  else
  {
    return '';
  }
}

function wiclearwikiredirect($contents, $attr)
{
    global $mode;
    global $user_lang;
    global $user;
    global $title;

    $contents[0] = trim($contents[0]);
    if (strpos($contents[0], 'javascript:') !== false)
    { // for security reason
        $contents[0] = '#';
    }

    $headUrl = $url = $contents[0];
    if (strpos($contents[0], ':/') === false && strpos($contents[0], 'mailto:') === false)
    {
        $url_options = array();
        if (!empty($user) && $user_lang != $user->lang)
        {
            $url_options['lang'] = $user_lang;
        }
        $url_options['from'] = $title;

        $url     = createURL($contents[0], $url_options);
        $headUrl = createHeaderUrl($contents[0], $url_options);
    }

    $redirect = varGet('redirect');
    if ($mode != 'edit' && $mode != 'add' && $redirect != 'no')
    {
        header("LOCATION: ".$headUrl);
    }
    else
    {
        return '<p>'.tr('REDIRECT TO').': '.$url.'</p>';
    }
}

?>
