<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief a comment
 */
class comment
{
  var $id;        //!< database id
  var $title;     //!< title of comment
  var $content;   //!< the comment
  var $timestamp; //!< date of comment
  var $name;      //!< name of user
  var $email;     //!< email of user
  var $ip;        //!< ip of user

  /**
   * \brief constructs a comment
   *
   * the comment can be constructed either from a logged user or an anonymous user.
   * Depending of user_id, the comment will get its information either from anon_name and anon_email,
   * or from firstname, lastname and email
   *
   * \param id database id
   * \param title title of comment
   * \param content the comment
   * \param timestamp date of the comment
   * \param anon_name name of anonymous user
   * \param anon_email email of anonymous user
   * \param user_id database id of user if the comment was done by a logged user
   * \param firstname first name of logged user
   * \param lastname last name of logged user
   * \param email email of logged user
   * \param ip ip of user
   */
  function comment($id, $title, $content, $timestamp, $anon_name, $anon_email, $user_id, $firstname, $lastname, $email, $ip)
  {
    $this->id        = $id;
    $this->title     = $title;
    $this->content   = $content;
    $this->timestamp = $timestamp;

    if ($user_id == user_anonymous_id)
    {
      // anonymous user
      $this->name    = $anon_name;
      $this->email   = $anon_email;
    }
    else
    {
      $this->name    = $firstname.' '.$lastname;
      $this->email   = $email;
    }
    $this->ip        = $ip;
  }
}
?>