<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief a feed element
 */
class FeedNode
{
  var $title;     //!< title of this feed element
  var $author;    //!< author of this feed element
  var $date;      //!< date of this feed element
  var $lang;      //!< lang of this feed element
  var $content;   //!< content of this feed element Array 'content' is payload, 'type' is format of payload i.e. 'html' or 'text'
  var $url;       //!< url where we can find this full feed

  function FeedNode($title, $author, $date, $lang, $content, $url)
  {
    $this->title   = trim($title);
    $this->author  = trim($author);
    $this->date    = $date;
    $this->lang    = $lang;
    $this->content = $content;
    $this->url     = trim($url);
  }
}

/**
 * \brief a feed
 */
class feed
{
  var $charset;     //!< charset for the feed
  var $url;         //!< url of web application
  var $title;       //!< title of web application
  var $description; //!< description of web application
  var $lang;        //!< lang of content
  var $mostRecent;  //!< date of most recent update
  var $items;       //!< items of feed made up of feednode class

  function feed()
  {
    $this->items = array();
  }

  function setCharset($value)
  {
    $this->charset = trim($value);
  }

  function setLang($value)
  {
    $this->lang = trim($value);
  }

  function setFeedUrl($value)
  {
    $this->feedUrl = trim($value);
  }

  function setSiteUrl($value)
  {
    $this->siteUrl = trim($value);
  }

  function setTitle($value)
  {
    $this->title = trim($value);
  }

  function setDescription($value)
  {
    $this->description = trim($value);
  }

  function addItem($title, $author, $date, $lang, $content, $url)
  {
    $item = new FeedNode($title, $author, $date, $lang, $content, $url);

    // Keep track of the most recent item and use as the date of the feed.
    if (($date > $this->mostRecent) || ($this->mostRecent == null))
    {
      $this->mostRecent = $date;
    }

    // Add feed node to list of nodes
    $this->items[] = $item;
  }

  function is_whitespace($char)
  {
    return $char == ' ' || $char == '\t' || $char == '\r' || $char == '\n';
  }

  function summary($content)
  {
    // cut string at 200 char
    $length = min(200, strlen($content));

    // search last word boundary
    while ($length > 0 && !$this->is_whitespace($content[$length - 1]))
    {
      --$length;
    }

    // return resulting string
    return substr($content, 0, $length).'...';
  }

  /**
   * \brief generates a feed
   *
   * this method should be considered abstract
   *
   */
  function generate()
  {
    return '';
  }
}
?>
