<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief mail helper class
 */

class mail_helper
{
  /**
   * \brief This does exactly what preg_quote() does in PHP 4-ish
   *
   * \param $str string for quote
   * \param delimiter quote delimiter
   *
   * \return encoded text if success
   */
  function phpbb_preg_quote($str, $delimiter)
  {
    $text = preg_quote($str);
    $text = str_replace($delimiter, '\\' . $delimiter, $text);
    return $text;
  }

  /**
   * \brief Encodes the given string for proper mail send
   *
   * \param str string for enconding
   * \param charset charset for encoding
   *
   * \return encoded str if sucess or original string if failure
   */
  function encode($str, $charset)
  {
    if (($charset == 'ISO-8859-1') || (!$charset))
    {
      return $str;
    }

    if (preg_match('/[^\x20-\x7E]/', $str)) 
    {
      $chunk_size = 47; /* floor((75 - strlen("=?UTF-8?B??=")) * 0.75); */
      $len = strlen($str);
      $output = '';
      while ($len > 0) 
      {
        $chunk = utf8_helper::truncate($str, $chunk_size);
        $output .= ' =?UTF-8?B?'. base64_encode($chunk) ."?=\n";
        $c = strlen($chunk);
        $str = substr($str, $c);
        $len -= $c;
      }
      return trim($output);
    }
    else
    {
      /* define start delimimter, end delimiter and spacer */
      $end = "?=";
      $start = "=?$charset?B?";
      $spacer = "$end\r\n $start";

      /* determine length of encoded text within chunks and ensure length is even */
      $length = 75 - strlen($start) - strlen($end);
      $length = floor($length / 2) * 2;

      /* encode the string and split it into chunks with spacers after each chunk */
      $str = chunk_split(base64_encode($str), $length, $spacer);

      /* remove trailing spacer and add start and end delimiters */
      $str = preg_replace('#' . mail_helper::phpbb_preg_quote($spacer, '#') . '$#', '', $str);
    }
    return $start . $str . $end;
  }

  /**
   * \brief sends an email
   *
   * \param from from address
   * \param to to address
   * \param charset charset used to encode message
   * \param subject subject of mail
   * \param message body of mail
   *
   * \return true if success, error message otherwise
   */
  function send($from, $to, $charset, $subject, $message)
  {
    /* Do a little spring cleaning
     * idea stolen from PunBB */
    $to      = trim(preg_replace('#[\n\r]+#s', '', $to));
    $subject = trim(preg_replace('#[\n\r]+#s', '', $subject));
    $from    = trim(preg_replace('#[\n\r:]+#s', '', $from));

    $headers =
    'From: '.$from."\r\n".
    'Content-Type: text/plain;'."\r\n".
    "\t".'charset='.$charset."\r\n";
    $subject = mail_helper::encode(html_entity_decode($subject), $charset);

    return @mail($to, $subject, html_entity_decode($message), $headers);
  }
}

?>
