<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief provides an abstraction to a mysql database
 *
 * This file is based on the DotClear class.mysql.php
 */
class mysql extends db_connection
{
  var $host;                         //!< host to connect to
  var $login;                        //!< login to use
  var $password;                     //!< plain text password to use
  var $database;                     //!< database to use

  var $id;                           //!< database handle

  /**
   * \brief creates object, connect to mysql database, and use database
   */
  function mysql($host, $login, $password, $database)
  {
    $this->host                         = $host;
    $this->login                        = $login;
    $this->password                     = $password;
    $this->database                     = $database;
    $this->error                        = '';
    $this->debug_mode                   = false;
    $this->no_execute                   = false;
    $this->cumulated_sql_execution_time = 0;

    $this->id = @mysql_connect($host, $login, $password);

    if (!$this->connected())
    {
      $this->setError();
    }
    else
    {
      $result = @mysql_select_db($database);
      if (!$result)
      {
        $this->id = 0;
        $this->setError();
      }
    }
  }

  function connected()
  {
    return $this->id != 0;
  }

  function escapeString($str)
  {
    if (function_exists('mysql_real_escape_string'))
    {
      return mysql_real_escape_string($str);
    }
    else
    {
      return mysql_escape_string($str);
    }
  }

  function setError()
  {
    if ($this->id)
    {
      $this->error = mysql_error($this->id);
    }
    else
    {
      $this->error = mysql_error();
    }

    if ($this->debug_mode && !empty($this->error))
    {
      echo '<pre>'.$this->error.'</pre>';
    }
  }

  function createTable($query, $charset)
  {
    $query = str_replace("\n", '', $query);
    if (!empty($charset))
    {
      $query = str_replace(');', ') ENGINE=MyISAM DEFAULT CHARSET='.$charset.';', $query);
    }

    return $this->execute($query);
  }

  function execute($query)
  {
    if (!$this->id)
    {
      return false;
    }

    if ($this->debug_mode)
    {
      echo '<pre>'.$query.'</pre>';
    }

    if ($this->no_execute)
    {
      return;
    }

    $query = trim($query); // adamantix do not like trailing spaces after final ';'
    $start = chrono::now();
    $result = mysql_query($query, $this->id);
    $end = chrono::now();
    $this->cumulated_sql_execution_time += $end - $start;

    if (!$result)
    {
      $this->setError();
      return false;
    }
    else
    {
      return true;
    }
  }

  function insertAndGetId($query)
  {
    if (!$this->id)
    {
      return false;
    }

    if ($this->debug_mode)
    {
      echo '<pre>'.$query.'</pre>';
    }

    if ($this->no_execute)
    {
      return;
    }

    $query = trim($query); // adamantix do not like trailing spaces after final ';'
    $start = chrono::now();
    $result = mysql_query($query, $this->id);
    $end = chrono::now();
    $this->cumulated_sql_execution_time += $end - $start;

    if (!$result)
    {
      $this->setError();
      return 0;
    }
    else
    {
      return mysql_insert_id($this->id);
    }
  }

  function select1($query)
  {
    if (!$this->id)
    {
      return false;
    }

    if ($this->debug_mode)
    {
      echo '<pre>'.$query.'</pre>';
    }

    if ($this->no_execute)
    {
      return;
    }

    $query = trim($query); // adamantix do not like trailing spaces after final ';'
    $start = chrono::now();
    $h = mysql_query($query);
    $end = chrono::now();
    $this->cumulated_sql_execution_time += $end - $start;

    if ($h)
    {
      $result = mysql_fetch_row($h);
      mysql_free_result($h);
      return $result;
    }

    $this->setError();

    return false;
  }

  function select($query)
  {
    if (!$this->id)
    {
      return false;
    }

    $result = array();

    if ($this->debug_mode)
    {
      echo '<pre>'.$query.'</pre>';
    }

    if ($this->no_execute)
    {
      return;
    }

    $query = trim($query); // adamantix do not like trailing spaces after final ';'
    $start = chrono::now();
    $h = mysql_query($query);
    $end = chrono::now();
    $this->cumulated_sql_execution_time += $end - $start;

    if ($h)
    {
      while (($row = mysql_fetch_row($h)) == true)
      {
        array_push($result, $row);
      }
      mysql_free_result($h);
    }
    else
    {
      $this->setError();
    }

    return $result;
  }

  function close()
  {
    if (!$this->id)
    {
      return;
    }

    mysql_close($this->id);
  }

  function tableExists($table_name)
  {
    $sql = 'show tables';
    $rows = $this->select($sql);

    foreach ($rows as $r)
    {
      if ($r[0] == $table_name)
      {
        return true;
      }
    }
    return false;
  }
}
?>