<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief generates a 1.0 feed
 *
 * when called this method will generate the appropriate header
 */
class rss_1 extends feed
{
  function headers($charset)
  {
    header('Content-Type: application/rss+xml; charset='.$charset);
    header('Content-Disposition: inline; filename=feed_rss_1.xml');
  }

  function generate()
  {
    $blob = '<?xml version="1.0" encoding="'.$this->charset.'"?>'."\n".
    '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"'."\n".
    '         xmlns="http://purl.org/rss/1.0/" '."\n".
    '         xmlns:dc="http://purl.org/dc/elements/1.1/" >'."\n".
    '<channel rdf:about="'.$this->feedUrl.'">'."\n".
    '   <title>'.$this->title.'</title>'."\n".
    '   <link>'.$this->feedUrl.'</link>'."\n".
    '   <description>'.$this->description.'</description>'."\n".
    "\n".
    '   <items>'."\n".
    '       <rdf:Seq>'."\n";

    foreach ($this->items as $item)
    {
      $blob .=
      '         <rdf:li rdf:resource="'.$item->url.'"/>'."\n";
    }

    $blob .=
    '       </rdf:Seq>'."\n".
    '   </items>'."\n".
    '</channel>'."\n";

    foreach ($this->items as $item)
    {
      $blob .=
      '<item rdf:about="'.$item->url.'">'."\n".
      '     <title>'.$item->title.'</title>'."\n".
      '     <link>'.$item->url.'</link>'."\n".
      '     <description>'.htmlspecialchars($item->content['content']).'</description>'."\n".
      '     <dc:date>'.date_helper::getISO8601Date($item->date).'</dc:date>'."\n".
      '     <dc:creator>'.$item->author.'</dc:creator>'."\n".
      '</item>'."\n";
    }

    $blob .= '</rdf:RDF>';

    return $blob;
  }
}
?>
