<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

$contentBoxHandle['attach_view']          = 'displayAttachments';
$contentBoxHandle['attach_browse']        = 'displayAttachmentBrowser';
$userActionHandle['attach']               = 'onAttachFiles';
$userActionHandle['attach_del']           = 'onAttachDelete';
$browserTitlePageHandle['attach_view']    = 'browserAttachments';
$browserTitlePageHandle['attach_browser'] = 'browserAllAttachments';

/**
 * \brief creates attachments link
 * \ingroup template_functions
 *
 * \return html link
 */
function attachmentsBox()
{
  global $node;

  if (empty($node))
  {
    return '';
  }

  return '<a href="'.createURL($node->title, array('lang'=>$node->lang->code, 'mode'=>'attach_view')).'">'.tr('Attachments').'</a>';
}

/**
 * \brief creates attachments link
 * \ingroup template_functions
 *
 * \return html link
 */
function browseAllAttachmentsBox()
{
  return '<a href="'.createURL('SpecialPage:attach_browse').'">'.tr('Browse all attachments').'</a>';
}

/**
 * \brief customize browser title when viewing attachements
 *
 * \return browser title
 */
function browserAttachments()
{
  return wc_wiki_name.' - '.tr('Attachments');
}

/**
 * \brief customize browser title when viewing all attachements
 *
 * \return browser title
 */
function browserAllAttachments()
{
  return wc_wiki_name.' - '.tr('Browse attachments');
}

/**
 * \brief display attachments
 *
 * will display attachments linked to a page and a form to add more files
 */
function displayAttachments()
{
  global $node;
  global $wiki;

  $attachments = $wiki->getAttachments($node->id);

  $html = '';
  if (sizeof($attachments) == 0)
  {
    $html = '<h1>'.tr('No attachments on this page yet').'</h1>';
  }
  else
  {
    $html = '<h1>'.tr('Current files attached to page').'</h1>';
    $html .= '<table class="attachments">';
    $html .= 
    '<tr>'.
    '<th>'.tr('Thumb').'</th>'.
    '<th>'.tr('File').'</th>'.
    '<th>'.tr('Size').'</th>'.
    '<th>'.tr('Date').'</th>'.
    '<th>'.tr('User').'</th>'.
    '<th>'.tr('Ip').'</th>';

    global $user;
    if ($user->isModerator() || $user->isAdmin())
    {
      $html .= '<th>'.tr('Delete attachments').'</th>';
      $html .= '<th>'.tr('Delete attachments and file').'</th>';
      $html .= '<th>'.tr('Ban').'</th>';
    }

    $html .=
    '</tr>';

    $even = false;
    foreach ($attachments as $attach)
    {
      $file_name       = $attach[0];
      $timestamp       = $attach[1];
      $ip              = $attach[2];
      $user_login      = $attach[3];
      $user_first_name = $attach[4];
      $user_last_name  = $attach[5];

      $file            = createImagePath($file_name);

      $thumb           = '<img src="'.createImageURL($file_name, 40, 40).'"/>';
      $url             = createImageUrl($file_name);

      if (file_exists($file))
      {
        $file_desc = '<a href="'.$url.'">'.$file_name.'</a>';
        $size      = filesize($file);
        $size      = formatSize($size);
      }
      else
      {
        $file_desc = '<a href="'.$url.'">'.$file_name.' ('.tr('Missing file').')</a>';
        $size      = 'N/A';
      }

      if ($even)
      {
        $trclass = ' class="even"';
      }
      else
      {
        $trclass = '';
      }
      $even = !$even;

      $html .=
      '<tr'.$trclass.'>'.
      '<td>'.$thumb.'</td>'.
      '<td>'.$file_desc.'</td>'.
      '<td>'.$size.'</td>'.
      '<td>'.formatDate($timestamp).'</td>'.
      '<td>'.formatName($user_login, $user_first_name, $user_last_name).'</td>'.
      '<td>'.$ip.'</td>';

      if ($user->isModerator() || $user->isAdmin())
      {
        $url = createUrl($node->title, array('lang'=>$node->lang->code, 'mode'=>'attach_del', 'file'=>$file_name));
        $html .= '<td><a href="'.$url.'">'.tr('Delete').'</a></td>';
        $url = createUrl($node->title, array('lang'=>$node->lang->code, 'mode'=>'attach_del', 'all'=>'yes', 'file'=>$file_name));
        $html .= '<td><a href="'.$url.'">'.tr('Delete').'</a></td>';
        $url = createUrl($node->title, array('lang'=>$node->lang->code, 'mode'=>'ban_add', 'ip'=>$ip));
        $html .= '<td><a href="'.$url.'">'.tr('Ban').'</a></td>';
      }

      $html .=
      '</tr>';
    }
    $html .= '</table>';
  }

  $html .= '<hr/>';

  global $user;
  if (auth::canEdit($user, $node))
  {
    $targetUrl = createUrl($node->title, array('lang'=>$node->lang->code, 'mode'=>'attach'));
    $form = new form($targetUrl, 'post', '', 'multipart/form-data');
    $form->addFieldSet();
    $form->addLegend(tr('Add attachments'));

    $form->addHtml('<p class="field">');
    $form->addLabel(tr('File'), 'ftitle');
    $form->addHidden('MAX_FILE_SIZE', '2000000'); // 2Mb max size
    $form->addFileUploadButton('upload');
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addButton('attach', tr('Attach'), 'submit');
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addHidden('check', '1');
    $form->addHtml('</p>');

    $form->addHtml('<p class="field"/>');

    $html .= $form->toHtml();
  }

  return $html;
}

/**
 * \brief display all files in images dir
 */
function displayAttachmentBrowser($dir = '', $width=100, $height=100, $showDir=true)
{
  $html = '';

  if (isset($_GET['dir']))
  {
    $dir = varGet('dir');
  }

  if (!empty($dir) && !preg_match('!.*/$!', $dir))
  {
    // no terminal '/' : add one
    $dir .= '/';
  }

  $imageBaseDir = createImagePath($dir);

  if (file_exists($imageBaseDir))
  {
    $h = opendir($imageBaseDir);
    while (($file = readdir($h)) !== false)
    {
      if ($file == '.' || $file == '..')
      {
        continue;
      }

      $full_file = $imageBaseDir.$file;

      if ($showDir && @is_dir($full_file))
      {
        // special : create a link to browse this directory
        $image_url = createIconURL(wc_icon_folder, $width, $height);
        $image_lnk = createURL('SpecialPage:attach_browse', array('dir'=>$dir.$file));

        $html .= 
        '<div class="image">'.
          '<a href="'.$image_lnk.'">'.
            '<img src="'.$image_url.'" />'.
          '</a>'.
          '<h1 class="vertical_align">'.$file.' ('.tr('directory').')</h1>'.
        '</div>';
      }
      else
      if (@is_file($full_file))
      {
        $size = filesize($full_file);
        $size = formatSize($size);

        $image_url = createImageURL($dir.$file, $width, $height);
        $image_lnk = createImageURL($dir.$file);

        $html .= 
        '<div class="image">'.
          '<a href="'.$image_lnk.'">'.
            '<img src="'.$image_url.'" />'.
          '</a>'.
          '<h1 class="vertical_align">'.$file.' ('.$size.')</h1>'.
        '</div>';
      }
    }
    closedir($h);
  }

  $html .= '<div class="spacer">&nbsp;</div>';

  return $html;
}

/**
 * \brief called back when a file has been uploaded
 *
 * will analyse form values to handle uploaded files, move it to the image folder
 * and add it to the attachments table
 */
function onAttachFiles()
{
  global $node;
  global $user;
  if (!auth::canEdit($user, $node))
  {
    return;
  }
  if ($user->isAnonymous() && auth::banned())
  {
    return;
  }

  if (!isset($_POST['check']))
  {
    return;
  }

  if (!isset($_FILES['upload']))
  {
    return;
  }

  $tmp_name = $_FILES['upload']['tmp_name'];
  if (!is_uploaded_file($tmp_name))
  {
    // file not uploaded (upload attack ?)
    return;
  }

  $file_name = $_FILES['upload']['name'];
  if (strpos($file_name, '..') !== false)
  {
    // trying to escape upload dir ? (upload attack ?)
    return;
  }

  if ($file_name == '.htaccess')
  {
    // trying to trash .htaccess to allow php exec for example ?  (upload attack ?)
    return;
  }

  $uploadFile = createImagePath($file_name);

  $error = @move_uploaded_file($tmp_name, $uploadFile);
  if ($error === true)
  {
    global $user;
    $node_id = $node->id;
    $user_id = $user->id;

    global $wiki;
    // file was correctly moved, remember it in attachments table
    $wiki->attachFileToPage($file_name, $node_id, $user_id);

    // notify users
    $message =
    tr('Attachment has been added ').' '.tr('on page').' '.$node->title."\n".
    createHeaderURL($node->title, array('lang'=>$node->lang->code))."\n";

    $email_title = '['.wc_wiki_name.']'.tr('Attachment added');

    $emails = $wiki->getEmails($node->id, array($user->id));
    foreach ($emails as $email)
    {
      $result = mail_helper::send(
        wc_mail, $email[0], wc_charset, 
        $email_title,
        $message
      );
    }
  }

  header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code, 'mode'=>'attach_view')));
}

/**
 * \brief called back when an attachment needs to be deleted
 *
 * will delete attachments
 */
function onAttachDelete()
{
  if (!isset($_GET['file']))
  {
    return;
  }

  $attach = varGet('file');

  global $user;
  global $node;
  if ($user->isModerator() || $user->isAdmin())
  {
    global $wiki;
    $wiki->deleteAttachments($attach, $node->id);

    if (isset($_GET['all']) && varGet('all') === 'yes')
    {
      unlink(createImagePath($attach)); 
    }
  }

  header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code, 'mode'=>'attach_view')));
}
?>