<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Several helper methods for template creation on feeds
 *
 * This file creates no hook.
 *
 * This page contains several helper functions for the user to create template :
 */

/**
 * \brief creates an url to recent comment feed
 * \ingroup template_functions
 *
 * \return html url
 */
function feedRecentCommentsUrl()
{
  global $app_path;

  return $app_path.'comment_feed.php?feed_type='.wc_feed_format.'&amp;full_content=1';
}

/**
 * \brief creates an html link to recent comment feed
 * \ingroup template_functions
 *
 * \return html link
 */
function feedRecentCommentsLink()
{
  return '<link rel="alternate" type="application/rss+xml" title="'.wc_wiki_name.' '.tr('comments').'(RSS 1.0)" href="'.feedRecentCommentsUrl().'"/>';
}

/**
 * \brief creates an html url to list of comment of current node
 * \ingroup template_functions
 *
 * \return html url
 */
function feedLocalCommentsUrl()
{
  global $app_path;
  global $node;

  if (empty($node))
  {
    return;
  }

  $title = rawurlencode($node->title);
  $lang  = $node->lang->code;

  if (!empty($title))
  {
    return $app_path.'comment_feed.php?feed_type='.wc_feed_format.'&amp;full_content=1&amp;title='.$title.'&amp;lang='.$lang;
  }
  else
  {
    return '';
  }
}

/**
 * \brief creates an html link to list of comment of current node
 * \ingroup template_functions
 *
 * \return html link
 */
function feedLocalCommentsLink()
{
  global $node;

  if (empty($node))
  {
    return;
  }

  $title = empty($node->alternate_title) ? $node->title : $node->alternate_title;
  $title = rawurlencode($title);

  if (!empty($title))
  {
    return '<link rel="alternate" type="application/rss+xml" title="'.$title.' '.tr('comments').' (RSS 1.0)" href="'.feedLocalCommentsUrl().'"/>';
  }
  else
  {
    return '';
  }
}

/**
 * \brief creates an html url to list of recent changes
 * \ingroup template_functions
 *
 * \return html url
 */
function feedRecentChangesUrl()
{
  global $app_path;

  return $app_path.'feed.php?feed_type='.wc_feed_format.'&amp;full_content=1';
}

/**
 * \brief creates an html link to list of recent changes
 * \ingroup template_functions
 *
 * \return html link
 */
function feedRecentChangesLink()
{
  return '<link rel="alternate" type="application/rss+xml" title="'.wc_wiki_name.' (RSS 1.0)" href="'.feedRecentChangesUrl().'"/>';
}

/**
 * \brief creates an html url to history of current node
 * \ingroup template_functions
 *
 * \return html url
 */
function feedLocalChangesUrl()
{
  global $app_path;
  global $node;

  if (empty($node))
  {
    return;
  }

  $title = rawurlencode($node->title);
  $lang  = $node->lang->code;

  if (!empty($title))
  {
    return $app_path.'feed.php?feed_type='.wc_feed_format.'&amp;full_content=1&amp;title='.$title.'&amp;lang='.$lang;
  }
  else
  {
    return '';
  }
}

/**
 * \brief creates an html link to history of current node
 * \ingroup template_functions
 *
 * \return html link
 */
function feedLocalChangesLink()
{
  global $node;

  if (empty($node))
  {
    return;
  }

  $title = empty($node->alternate_title) ? $node->title : $node->alternate_title;
  $title = rawurlencode($title);

  if (!empty($title))
  {
    return '<link rel="alternate" type="application/rss+xml" title="'.$title.' '.tr('history').' (RSS 1.0)" href="'.feedLocalChangesUrl().'"/>';
  }
  else
  {
    return '';
  }
}

/**
 * \brief return local links (history and comments of current nodes)
 * \ingroup template_functions
 *
 * \return html url
 */
function feedLocalLinks()
{
  return feedLocalChangesLink()."\n".feedLocalCommentsLink()."\n";
}

?>
