<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls the main functionality : content edition
 *
 * This file creates one hook :
 * <ul>
 *   <li>recent_change : to display recent changes to the site</li>
 * </ul>
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>recentChangesBox</li>
 * </ul>
 */

$contentBoxHandle['recent_change']       = 'recentChanges';

$browserTitlePageHandle['recent_change'] = 'browserRecentChange';

/**
 * \brief customize browser title when viewing recent changes
 *
 * \return browser title
 */
function browserRecentChange()
{
  return wc_wiki_name.' - '.tr('Recent changes');
}

/**
 * \brief display a link to recent changes
 * \ingroup template_functions
 *
 * \return html link
 */
function recentChangesBox()
{
  return '<a href="'.createURL('SpecialPage:recent_change').'">'.tr('Recent changes').'</a>';
}

/**
 * \brief display recent changes information
 *
 * \return html blob
 */
function recentChanges()
{
  global $wiki;
  global $user;

  $old_offset = 0;
  $offset     = 0;
  $limit      = 10;

  if (isset($_POST['limit']))
  {
    $limit = varPost('limit');
  }
  if (isset($_POST['offset']))
  {
    $offset = varPost('offset');
  }
  if (isset($_POST['old_offset']))
  {
    $old_offset = varPost('old_offset');
  }

  if (isset($_POST['previous']))
  {
    $offset = $old_offset;
  }

  if ($user->isModerator() || $user->isAdmin())
  {
    $titleAltDatesLang = $wiki->recentChanges($offset, $limit + 1, false, false);
  }
  else
  {
    $titleAltDatesLang = $wiki->recentChanges($offset, $limit + 1, false, false, $user->id);
  }

  $blob = '<ul>';

  $limit = min($limit, count($titleAltDatesLang));
  for ($i = 0; $i < $limit; ++$i)
  {
    $titleAltDateLang = $titleAltDatesLang[$i];

    $content_title    = $titleAltDateLang[0];
    $alt_title        = $titleAltDateLang[1];
    $date             = $titleAltDateLang[2];
    $lang             = $titleAltDateLang[3];
    $parent           = $titleAltDateLang[4];

    $display_title = $content_title;
    if (!empty($alt_title))
    {
      $display_title = $alt_title;
    }

    $blob .= sprintf(
      '<li><a href="%s">%s [%s]</a> (%s), %s</li>',
      createURL($content_title, array('lang'=>$lang)),
      formatHtml($display_title),
      $lang,
      $parent,
      formatDate($date)
    );
  }
  $blob .= '</ul>';

  global $title;
  if ($offset != 0 || sizeof($titleAltDatesLang) > $limit)
  {
    $form = new form(createURL($title, array('mode'=>'recent_change')), 'post', 'more_results');
    $form->addFieldSet();
    $form->addLegend(tr('More results'));

    $form->addHtml('<p class="field">');
    $form->addHidden('old_offset', max(0, $offset - $limit));
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addHidden('offset', $offset + $limit);
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addLabel(tr('Nb results'), 'flimit');
    $form->addInput('limit', 'Limit', $limit, 'flimit');
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    if (sizeof($titleAltDatesLang) > $limit)
    {
      // there are other results to display
      $form->addButton('next', '&gt;&gt;', 'submit');
    }
    if ($offset != 0)
    {
      $form->addButton('previous', '&lt;&lt;', 'submit');
    }
    $form->addHtml('</p>');

    $form->addHtml('<p class="field"/>');

    $blob .= $form->toHtml();
  }

  $form2 = new form(createURL($title), 'post', 'cancel');

  $form2->addHtml('<p class="field">');
  $form2->addButton('submit', tr('Get back to wiki'), 'submit');
  $form2->addHtml('</p>');

  $form2->addHtml('<p class="field"/>');

  $blob .= $form2->toHtml();

  return $blob;
}
?>