<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls user registration
 *
 * This file creates several hooks :
 * <ul>
 *   <li>register : to register a new user</li>
 *   <li>first_login : when the user logs in the first time</li>
 *   <li>confirm : when the user use the confirm link sent by email</li>
 * </ul>
 *
 * This page contains no helper functions (they are included in login.lib.php)
 */

$contentBoxHandle['register']    = 'registerBox';
$contentBoxHandle['first_login'] = 'registerFirstLogin';

$userActionHandle['register']    = 'onRegister';
$userActionHandle['confirm']     = 'onConfirm';

$browserTitlePageHandle['register']    = 'browserRegister';
$browserTitlePageHandle['first_login'] = 'browserFirstLogin';

/**
 * \brief customize browser title when registering
 *
 * \return browser title
 */
function browserRegister()
{
  return wc_wiki_name.' - '.tr('Register');
}

/**
 * \brief customize browser title when logging in the first time
 *
 * \return browser title
 */
function browserFirstLogin()
{
  return wc_wiki_name.' - '.tr('First login');
}

/**
 * \brief display a register form
 *
 * \return html blob
 */
function registerBox()
{
  if (wc_wiki_open_registration != 'true')
  {
    return;
  }

  global $title;
  global $wiki;

  $langs = $wiki->getLangs();

  $user_login        = '';
  $user_fname        = '';
  $user_lname        = '';
  $user_email        = '';
  $user_content_lang = wc_default_lang;
  $user_ui_lang      = wc_default_lang;
  $user_error        = '';

  $user_antibot = auth::generateRandomText(8);
  setVarSession('user_antibot', $user_antibot);

  if (existVarSession('user_error'))
  {
    $user_login        = varSession('user_login');
    $user_fname        = varSession('user_fname');
    $user_lname        = varSession('user_lname');
    $user_email        = varSession('user_email');
    $user_content_lang = varSession('user_content_lang');
    $user_ui_lang      = varSession('user_ui_lang');
    $user_error        = varSession('user_error');

    unsetVarSession('user_login');
    unsetVarSession('user_fname');
    unsetVarSession('user_lname');
    unsetVarSession('user_email');
    unsetVarSession('user_content_lang');
    unsetVarSession('user_ui_lang');
    unsetVarSession('user_error');
    unsetVarSession('user_password');
  }

  if ($user_error == 'OK')
  {
    // everything went good
    $blob =
    '<h1>'.tr('User successfully added').'</h1>'.
    '<p>'.tr('An email has been sent to you').'</p>'.
    '<p>'.tr('You will need to follow the link given to activate your account').'</p>';

    $form = new form(createURL($title), 'post');
    $form->addHtml('<p class="field">');
    $form->addButton('submit', tr('Get back to wiki'), 'submit');
    $form->addHtml('</p>');

    return $blob.$form->toHtml();
  }

  $blob =
  '<h1>'.tr('User creation').'</h1>';

  if (!empty($user_error))
  {
    $blob .=
    '<p>'.$user_error.'</p>';
  }

  $form = new form(createURL($title, array('mode'=>'register')), 'post', 'register');
  $form->addFieldset();
  $form->addLegend(tr('Antibot'));

  global $app_path;
  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Antibot text'));
  $form->addHtml('<img src="'.$app_path.'textasimage.php"/>');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Write antibot text here'), 'fantibot');
  $form->addInput('user_antibot', 'Antibot text', '', 'fantibot');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field"/>');

  $form->addFieldset();
  $form->addLegend(tr('Login'));

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('login'), 'flogin');
  $form->addInput('user_login', 'Login', $user_login, 'flogin');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('First name'), 'ffname');
  $form->addInput('user_fname', 'First name', $user_fname, 'ffname');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Last name'), 'flname');
  $form->addInput('user_lname', 'Last name', $user_lname, 'flname');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Password'), 'fpassword');
  $form->addPassword('user_password', '', 'fpassword');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Email'), 'femail');
  $form->addInput('user_email', 'Email', $user_email, 'femail');
  $form->addHtml('</p>');

  $langList = array();
  foreach ($langs as $l)
  {
    $langList[] = $l->code;
  }

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Prefered lang for content'), 'fcontent_lang');
  $form->addDropDown($langList, $user_content_lang, 'user_content_lang', 'fcontent_lang');
  $form->addHtml('</p>');

  $langBaseDir = wc_inc_path.'/i18n/';
  $uiLangList = lang::getUILangs($langBaseDir);

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Prefered lang for interface'), 'fui_lang');
  $form->addDropDown($uiLangList, $user_ui_lang, 'user_ui_lang', 'fui_lang');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHidden('check', '1');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addButton('submit', tr('Register'), 'submit');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field"/>');

  return $blob.$form->toHtml();
}

/**
 * \brief called when user validates form from registerBox
 */
function onRegister()
{
  global $wiki;
  global $title;

  if (wc_wiki_open_registration != 'true' || !isset($_POST['submit']))
  {
    return;
  }

  $user_login        = varPost('user_login');
  $user_fname        = varPost('user_fname');
  $user_lname        = varPost('user_lname');
  $user_password     = varPost('user_password');
  $user_email        = varPost('user_email');
  $user_content_lang = varPost('user_content_lang');
  $user_ui_lang      = varPost('user_ui_lang');
  $user_antibot      = varPost('user_antibot');

  $session_user_antibot = varSession('user_antibot');
  unsetVarSession('user_antibot');

  setVarSession('user_login',        $user_login);
  setVarSession('user_fname',        $user_fname);
  setVarSession('user_lname',        $user_lname);
  setVarSession('user_email',        $user_email);
  setVarSession('user_content_lang', $user_content_lang);
  setVarSession('user_ui_lang',      $user_ui_lang);

  if ($session_user_antibot != $user_antibot)
  {
    setVarSession('user_error', tr('Antibot validation failed'));

    session_write_close();
    header('Location: '.createHeaderURL($title, array('mode'=>'register')));
    return;
  }

  if ($wiki->userExists($user_login))
  {
    setVarSession('user_error', tr('User').' '.$user_login.' '.tr('already exists'));

    session_write_close();
    header('Location: '.createHeaderURL($title, array('mode'=>'register')));
    return;
  }

  $langs = $wiki->getLangs();
  $lang_fk = lang::getIdFromCode($langs, $user_content_lang);
  $key = $wiki->addUnconfirmedUser($user_login, $user_fname, $user_lname, $user_password, $user_email, $lang_fk, $user_ui_lang);

  if (!empty($key))
  {
    $message =
    tr('Welcome').' '.tr('on').' '.wc_wiki_name.' '.$user_fname.' '.$user_lname."\n".
    tr('to validate your inscription, you need to follow the following link').' :'."\n".
    createHeaderURL($title, array('mode'=>'confirm', 'key'=>$key))."\n\n".
    tr('See you soon');

    $result = mail_helper::send(
      wc_mail, $user_email, wc_charset, 
      tr('Your inscription on').' '.wc_wiki_name, 
      $message
    );

    if ($result)
    {
      setVarSession('user_error', 'OK'); // do not use tr() here to translate OK. OK is used as an 'error code' and is never displayed to user
    }
    else
    {
      setVarSession('user_error', tr('Could not send an email. Contact the webmaster').' : '.$result);
    }

    // admin notifications
    $admin_message = tr('User').' '.$user_fname.' '.$user_lname.' '.tr('has registered');

    $admin_emails = $wiki->getAdminEmails();
    foreach ($admin_emails as $admin_email)
    {
      mail_helper::send(
        wc_mail, $admin_email, wc_charset,
        tr('New registered user'),
        $admin_message
      );
    }

    session_write_close();
    header('Location: '.createHeaderURL($title, array('mode'=>'register')));
  }
  else
  {
    setVarSession('user_error', tr('Error while adding user. Contact the webmaster'));

    session_write_close();
    header('Location: '.createHeaderURL($title, array('mode'=>'register')));
  }
}

/**
 * \brief first login of user, called from onConfirm()
 */
function registerFirstLogin()
{
  global $user;

  if ($user->isAnonymous())
  {
    return;
  }

  return
  '<p>'.tr('Welcome').' '.$user->first_name.' '.$user->last_name.'</p>'.
  '<p>'.tr('You are now logged on').' '.wc_wiki_name.' '.tr('and can start contributing').'</p>';
}

/**
 * \brief the user clicked on the link given in email confirmation
 *
 * it is validated, and gets redirected to registerFirstLogin
 */
function onConfirm()
{
  global $wiki;
  global $title;

  if (wc_wiki_open_registration != 'true' || !isset($_GET['key']))
  {
    return;
  }
  $key = varGet('key');

  $result = $wiki->confirmUser($key);

  if (!empty($result))
  {
    setVarSession('user_id', $result[0]);
    setVarSession('user_password', $result[1]);

    session_write_close();
    header('Location: '.createHeaderURL($title, array('mode'=>'first_login')));
    return;
  }

  header('Location: '.createHeaderURL($title));
  return;
}
?>
