<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls basic full text search functionalities
 *
 * This file creates one hook :
 * <ul>
 *   <li>search : to display the result of a full text search</li>
 * </ul>
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>searchBox</li>
 * </ul>
 */

$contentBoxHandle['search']        = 'search';

$browserTitlePageHandle['search']  = 'browserSearch';

/**
 * \brief customize browser title when searching the site
 *
 * \return browser title
 */
function browserSearch()
{
  $search = '';
  if (isset($_POST['search']))
  {
    $search = varPost('search');
  }
  return wc_wiki_name.' - '.tr('Search').' '.$search;
}

/**
 * \brief display a search box
 * \ingroup template_functions
 *
 * \return html blob
 */
function searchBox($searchTerm = '')
{
  global $title;

  $form = new form(createURL($title, array('mode'=>'search')), 'post', 'search');
  $form->addFieldset();
  $form->addLegend(tr('Search'));
  $form->addHtml('<p class="field">');
  $form->addInput('search', 'Search', $searchTerm);
  $form->addButton('go', tr('Go'), 'searchbutton');
  $form->addHtml('</p>');
  $form->addHtml('<p class="field"/>');

  return $form->toHtml();
}

/**
 * \brief perform and display the result of a search
 *
 * \return html blob
 */
function search()
{
  global $wiki;

  $old_offset = 0;
  $offset     = 0;
  $limit      = 10;

  $search = varPost('search');
  if (isset($_POST['limit']))
  {
    $limit = varPost('limit');
  }
  if (isset($_POST['offset']))
  {
    $offset = varPost('offset');
  }
  if (isset($_POST['old_offset']))
  {
    $old_offset = varPost('old_offset');
  }

  if (isset($_POST['previous']))
  {
    $offset = $old_offset;
  }

  $titlesLang = $wiki->searchPages($search, $offset, $limit + 1);

  $blob = searchBox($search);

  if (empty($titlesLang))
  {
    $blob .= tr('No pages found');
  }
  else
  {
    $blob .= '<ul>';
    $i = 0; // keep track of number of item displayed
    // because we have asked $limit + 1 but we want to display only $limit
    foreach ($titlesLang as $titleLang)
    {
      //Use alternate title if not empty
      $display_title = (!empty($titleLang[1])) ? $titleLang[1] : $titleLang[0];

      $blob .= '<li><a href="'.createURL($titleLang[0], array('lang'=>$titleLang[2])).'">'.$display_title.' ['.$titleLang[2].']</a></li>';

      ++$i;
      if ($i == $limit)
      {
        break;
      }
    }
    $blob .= '</ul>';
  }

  global $title;

  if ($offset != 0 || sizeof($titlesLang) > $limit)
  {
    $form = new form(createURL($title, array('mode'=>'search')), 'post', 'more_results');
    $form->addFieldSet();
    $form->addLegend(tr('More results'));

    $form->addHtml('<p class="field">');
    $form->addHidden('search', $search);
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addHidden('old_offset', max(0, $offset - $limit));
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addHidden('offset', $offset + $limit);
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addLabel(tr('Nb results'), 'flimit');
    $form->addInput('limit', 'Limit', $limit, 'flimit');
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    if (sizeof($titlesLang) > $limit)
    {
      // there are other results to display
      $form->addButton('next', '&gt;&gt;', 'submit');
    }
    if ($offset != 0)
    {
      $form->addButton('previous', '&lt;&lt;', 'submit');
    }
    $form->addHtml('</p>');

    $form->addHtml('<p class="field"/>');

    $blob .= $form->toHtml();
  }

  $form2 = new form(createURL($title), 'post', 'cancel');

  $form2->addHtml('<p class="field">');
  $form2->addButton('submit', tr('Get back to wiki'), 'submit');
  $form2->addHtml('</p>');

  $form2->addHtml('<p class="field"/>');

  $blob .= $form2->toHtml();

  return $blob;
}
?>
