<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls user preferences
 *
 * This file creates one hook :
 * <ul>
 *   <li>user_pref : to display user preferences</li>
 * </ul>
 *
 * This page contains no helper functions (they are included in login.lib.php)
 */

$contentBoxHandle['user_pref'] = 'userPreferencesBoxWrapper';

$userActionHandle['user_pref'] = 'onChangeUserPreferencesWrapper';

$browserTitlePageHandle['user_pref'] = 'browserUserPref';

/**
 * \brief customize browser title when viewing preferences
 *
 * \return browser title
 */
function browserUserPref()
{
  return wc_wiki_name.' - '.tr('Preferences');
}

/**
 * \brief wrapper for userPreferencesBox() method when called from public site
 *
 * \returns userPreferencesBox html blob
 */
function userPreferencesBoxWrapper()
{
  global $user;
  global $title;
  $action = createURL($title, array('mode'=>'user_pref'));
  $langBaseDir = wc_inc_path.'/i18n/';

  return userPreferencesBox($user, $action, $langBaseDir, true);
}

/**
 * \brief wrapper for onChangeUserPreferences() callback when called from public site
 */
function onChangeUserPreferencesWrapper()
{
  global $user;
  global $title;
  $action = createHeaderURL($title, array('mode'=>'user_pref'));

  onChangeUserPreferences($user->id, $action, true);
}

/**
 * \brief display a user with a form to change preferences
 *
 * \param user user to display
 * \param action url to use
 * \param $langBaseDir path to i18n lang files
 * \param displayAlerts wether to display subscription
 *
 * \return html blob
 */
function userPreferencesBox($user, $action, $langBaseDir, $displayAlerts)
{
  global $wiki;
  global $title;

  if ($user->isAnonymous())
  {
    return;
  }

  $user_login   = $user->login;
  $user_fname   = $user->first_name;
  $user_lname   = $user->last_name;
  $user_email   = $user->email;
  $user_lang    = $user->lang;
  $user_ui_lang = $user->ui_lang;
  $user_id      = $user->id;

  $langs = $wiki->getLangs();

  // content langs
  $contentLangs = array();
  $user_lang_id = 1;
  foreach ($langs as $l)
  {
    $contentLangs[$l->code] = $l->id;
    if ($l->code == $user_lang)
    {
      $user_lang_id = $l->id;
    }
  }

  // ui langs
  $uiLangs = lang::getUILangs($langBaseDir);

  $form = new form($action, 'post', 'preferences');
  $form->addFieldset();
  $form->addLegend(tr('Preferences'));

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Login'), 'flogin');
  $form->addInput('user_login', 'Login', $user_login, 'flogin');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('First name'), 'ffname');
  $form->addInput('user_fname', 'First name', $user_fname, 'ffname');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Last name'), 'flname');
  $form->addInput('user_lname', 'Last name', $user_lname, 'flname');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Password'), 'fpassword');
  $form->addInput('user_password', 'Password', '', 'fpassword');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Email'), 'femail');
  $form->addInput('user_email', 'Email', $user_email, 'femail');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Lang'), 'flang');
  $form->addDropDown($contentLangs, $user_lang_id, 'user_lang', 'flang');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('UI Lang'), 'fuilang');
  $form->addDropDown($uiLangs, $user_ui_lang, 'user_ui_lang', 'fuilang');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHtml(tr('Joined').' '.formatDate($user->timestamp));
  $form->addHtml('</p>');

  if ($displayAlerts)
  {
    $form->addHtml('<p class="field">');
    $form->addHtml('<a href="'.createURL($title, array('mode'=>'watch_view')).'">'.tr('See watches').'</a>');;
    $form->addHtml('</p>');
  }

  $form->addHtml('<p class="field">');
  $form->addHidden('user_id', $user_id);
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHidden('check', '1');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addButton('submit', tr('Change Preferences'), 'submit');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field"/>');

  return $form->toHtml();
}

/**
 * \brief called from userPreferencesBox() form
 */
function onChangeUserPreferences($id, $action, $reset_session)
{
  global $wiki;
  global $title;
  global $user;

  if ($user->isAnonymous() || !isset($_POST['check']))
  {
    return;
  }

  $user_login    = varPost('user_login');
  $user_fname    = varPost('user_fname');
  $user_lname    = varPost('user_lname');
  $user_password = varPost('user_password');
  $user_email    = varPost('user_email');
  $user_lang     = varPost('user_lang');
  $user_ui_lang  = varPost('user_ui_lang');

  $wiki->updateUser($id, $user_login, $user_fname, $user_lname, $user_password, $user_email, $user_lang, $user_ui_lang);

  if (!empty($user_password) && $reset_session)
  {
    $user = $wiki->findUser($user_login, $user_password);
    // reset password in session
    setVarSession('user_password', $user->password);

    session_write_close();
  }

  header('Location: '.$action);
}
?>