<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
* \file
 * \brief this file allow some transformation when retrieving values from untrusted values
 *
 * basically, wuth magic quotes, php can escape automatically or not values.
 * as I find it painfull to browse automatically all variables and strip or unstrip them, I prefer
 * to force developers to call this method to always retrieve stripped variables.
 * It is then up to the developer to properly escape untrusted variables
 */

function &unmagic_quote_gpc(&$value)
{
  if (get_magic_quotes_gpc())
  {
    if (is_array($value))
    {
      foreach ($value as $k=>$v)
      {
        $value[$k] = stripslashes($v);
      }
    }
    else
    {
      $value = stripslashes($value);
    }
  }
  return $value;
}

/**
 * \brief get a value from a GET variable
 *
 * \param name GET variable name
 * \return stripped variable
 */
function &varGet($name)
{
  return unmagic_quote_gpc($_GET[$name]);
}

/**
 * \brief test for existence of a GET variable
 *
 * \param name GET variable name
 */
function existVarGet($name)
{
  return isset($_GET[$name]);
}

// SESSION --------------------------------------------------------------------

/**
 * \brief get a value from a POST variable
 *
 * \param name POST variable name
 * \return stripped variable
 */
function &varPost($name)
{
  return unmagic_quote_gpc($_POST[$name]);
}

/**
 * \brief test for existence of a POST variable
 *
 * \param name POST variable name
 */
function existVarPost($name)
{
  return isset($_POST[$name]);
}

// SESSION --------------------------------------------------------------------
$sessionPrefix = 'wiclear_';

/**
 * \brief get a value from a SESSION variable
 *
 * \param name SESSION variable name
 * \return stripped variable
 */
function &varSession($name)
{
  global $sessionPrefix;
  return unmagic_quote_gpc($_SESSION[$sessionPrefix.$name]);
}

/**
 * \brief test for existence of a SESSION variable
 *
 * \param name SESSION variable name
 */
function existVarSession($name)
{
  global $sessionPrefix;
  return isset($_SESSION[$sessionPrefix.$name]);
}

/**
 * \brief set a SESSION variable
 *
 * \param name SESSION variable name
 * \param value value of session variable
 */
function setVarSession($name, $value)
{
  global $sessionPrefix;
  $_SESSION[$sessionPrefix.$name] = $value;
}

/**
 * \brief unset a SESSION variable
 *
 * \param name SESSION variable name
 */
function unsetVarSession($name)
{
  global $sessionPrefix;
  unset($_SESSION[$sessionPrefix.$name]);
}

// COOKIE ---------------------------------------------------------------------
$cookiePrefix = 'wiclear_';

/**
 * \brief get a value from a COOKIE variable
 *
 * \param name COOKIE variable name
 * \return stripped variable
 */
function &varCookie($name)
{
  global $cookiePrefix;
  return unmagic_quote_gpc($_COOKIE[$cookiePrefix.$name]);
}

/**
 * \brief test for existence of a COOKIE variable
 *
 * \param name COOKIE variable name
 */
function existVarCookie($name)
{
  global $cookiePrefix;
  return isset($_COOKIE[$cookiePrefix.$name]);
}

/**
 * \brief set a COOKIE variable
 *
 * \param name COOKIE variable name
 * \param value value of COOKIE variable
 * \param expire optional parameter
 */
function setVarCookie($name, $value, $expire = 0)
{
  global $cookiePrefix;
  setcookie($cookiePrefix.$name, $value, $expire, '/');
}
?>