<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Display the list of page referencing current page
 *
 * This file creates one hook :
 * <ul>
 *   <li>what_links_here : to display the result of the search</li>
 * </ul>
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>whatLinksHereBox</li>
 * </ul>
 */

$contentBoxHandle['what_links_here'] = 'whatLinksHere';

$browserTitlePageHandle['what_links_here'] = 'browserWhatLinksHere';

/**
 * \brief customize browser title when viewing links pointing here
 *
 * \return browser title
 */
function browserWhatLinksHere()
{
  return wc_wiki_name.' - '.tr('What links here');
}

/**
 * \brief display a link to the WhatLinksHere feature
 * \ingroup template_functions
 *
 * \brief return link
 */
function whatLinksHereBox()
{
  global $title;
  global $mode;

  if (!empty($mode))
  {
    return '';
  }
  else
  {
    return '<a href="'.createURL($title, array('mode'=>'what_links_here')).'">'.tr('What links here').'</a>';
  }
}

/**
 * \brief display what links here
 *
 * \return html blob
 */
function whatLinksHere()
{
  global $wiki;
  global $title;

  $old_offset = 0;
  $offset     = 0;
  $limit      = 10;

  if (isset($_POST['limit']))
  {
    $limit = varPost('limit');
  }
  if (isset($_POST['offset']))
  {
    $offset = varPost('offset');
  }
  if (isset($_POST['old_offset']))
  {
    $old_offset = varPost('old_offset');
  }

  if (isset($_POST['previous']))
  {
    $offset = $old_offset;
  }

  $titlesLang = $wiki->whatLinksHere($title, $offset, $limit + 1);

  if (empty($titlesLang))
  {
    $blob = tr('No pages found');
  }
  else
  {
    $blob = '<ul>';
    $i = 0; // keep track of number of item displayed
      // because we have asked $limit + 1 but we want to display only $limit
    foreach ($titlesLang as $titleLang)
    {
      $blob .= '<li><a href="'.createURL($titleLang[0], array("lang"=>$titleLang[1])).'">'.$titleLang[0].' ['.$titleLang[1].']</a></li>';

      ++$i;
      if ($i == $limit)
      {
        break;
      }
    }
    $blob .= '</ul>';
  }

  if ($offset != 0 || sizeof($titlesLang) > $limit)
  {
    $form = new form(createURL($title, array('mode'=>'what_links_here')), 'post', 'more_results');
    $form->addFieldSet();
    $form->addLegend(tr('More results'));

    $form->addHtml('<p class="field">');
    $form->addHidden('old_offset', max(0, $offset - $limit));
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addHidden('offset', $offset + $limit);
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addLabel(tr('Nb results'), 'flimit');
    $form->addInput('limit', 'Limit', $limit, 'flimit');
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    if (sizeof($titlesLang) > $limit)
    {
      // there are other results to display
      $form->addButton('next', '&gt;&gt;', 'submit');
    }
    if ($offset != 0)
    {
      $form->addButton('previous', '&lt;&lt;', 'submit');
    }
    $form->addHtml('</p>');

    $form->addHtml('<p class="field"/>');

    $blob .= $form->toHtml();
  }

  $form2 = new form(createURL($title), 'post', 'cancel');

  $form2->addHtml('<p class="field">');
  $form2->addButton('submit', tr('Get back to wiki'), 'submit');
  $form2->addHtml('</p>');

  $form2->addHtml('<p class="field"/>');

  $blob .= $form2->toHtml();

  return $blob;
}

?>