<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief This page setup the world (global variable), import plugins and finally handle user action

 * The world consists of several objects :
 * <ul>
 *   <li>The configuration : ini</li>
 *   <li>The connection to the database : con</li>
 *   <li>The controller : wiki</li>
 *   <li>The wiki renderer : wikiRenderer</li>
 *   <li>The current user : user</li>
 * </ul>
 * other global variables are created :
 * <ul>
 *   <li>the title : title</li>
 *   <li>the mode : mode</li>
 *   <li>the user lang for the ui : ui_lang</li>
 *   <li>the user lang for the content : user_lang</li>
 *   <li>the theme : theme</li>
 * </ul>
 * Finally, when the world is setup, the available plugins are imported and a special function :
 * handleUserAction() is called. This function is responsible for reacting on user input before any other
 * content is displayed.
 */

function dumpVariables($string)
{
  echo '<pre>'.$string.' BEGIN</pre>';
  foreach ($_POST as $k=>$v)
  {
    echo '<pre>POST['.$k.']='.$v.'</pre>';
  }
  foreach ($_FILES as $uk=>$ks)
  {
    foreach ($ks as $k=>$v)
    {
      echo '<pre>FILES['.$uk.']['.$k.']='.$v.'</pre>';
    }
  }
  foreach ($_SESSION as $k=>$v)
  {
    echo '<pre>SESSION['.$k.']='.$v.'</pre>';
  }
  foreach ($_GET as $k=>$v)
  {
    echo '<pre>GET['.$k.']='.$v.'</pre>';
  }
  foreach ($_COOKIE as $k=>$v)
  {
    echo '<pre>COOKIE['.$k.']='.$v.'</pre>';
  }
  foreach ($_SERVER as $k=>$v)
  {
    echo '<pre>SERVER['.$k.']='.$v.'</pre>';
  }
  echo '<pre>'.$string.' END</pre>';
}

//dumpVariables("start");

require wc_inc_path.'/inc/classes/chrono.class.php';
/**
 * \brief global php start time
 */
$php_start_time = chrono::now();

require wc_inc_path.'/inc/lib/constants.lib.php';
// database interface
require wc_inc_path.'/inc/classes/db_connection.class.php';
// wiki
require wc_inc_path.'/inc/classes/nested_set_model_helper.class.php';
require wc_inc_path.'/inc/classes/wiki.class.php';
// translations
require wc_inc_path.'/inc/classes/translation.class.php';
// wiki renderer
require wc_inc_path.'/inc/classes/WikiRenderer.class.php';
require wc_inc_path.'/inc/classes/WiclearWikiRenderer.conf.php';
// auth
require wc_inc_path.'/inc/classes/lang.class.php';
require wc_inc_path.'/inc/classes/user.class.php';
require wc_inc_path.'/inc/classes/group.class.php';
// helper
require wc_inc_path.'/inc/classes/mail_helper.class.php';
require wc_inc_path.'/inc/classes/date_helper.class.php';
require wc_inc_path.'/inc/classes/utf8_helper.class.php';
// current node
require wc_inc_path.'/inc/classes/node.class.php';
require wc_inc_path.'/inc/classes/comment.class.php';

/**
 * \brief a title describing the page and the eventual action going on
 */
$browser_title_page = 'Wiclear';

/**
  * \brief global wiki renderer configuration variable
  */
$wikiRendererConfig = new WiclearWikiRendererConfig();
if (wc_wiki_allow_html == 'true')
{
  $wikiRendererConfig->escapeSpecialChars = false;
}

/**
  * \brief global wiki renderer variable
  */
$wikiRenderer = new WikiRenderer($wikiRendererConfig);

if ($ini->file_exists)
{
  $browser_title_page = wc_wiki_name;

  /**
   * \brief full url to web application
   */
  $app_path = createWebAppPath();

  // db
  require wc_inc_path.'/inc/classes/mysql.class.php';
  /**
   * \brief global connection to database variable
   */
  $con = new mysql(wc_db_host, wc_db_login, wc_db_password, wc_db_database);

  if ($con->connected() == false)
  {
    $error_message = tr('Could not connect to database');

    require 'inc/classes/null_db.class.php';
    $con = new null_db();
  }

  // theme var
  /**
   * \brief theme path
   */
  $theme_path    = 'themes/';

  /**
   * \brief the theme we want to load
   */
  $theme         = wc_theme;

  if ($mode == 'print')
  {
    $template_file = '/print.php';
  }
  else
  {
    $template_file = '/template.php';
  }

  /**
   * \brief full path to theme file
   */
  $template_path = wc_inc_path.$theme_path.$theme.$template_file;

  // if the theme defined in the ini file is not defined, revert to the default one
  if (!file_exists($template_path))
  {
    $theme         = 'default';
    $template_path = wc_inc_path.$theme_path.$theme.$template_file;
  }

  /**
   * \brief full path to eventual prepend file for the theme
   */
  $template_prepend_file = wc_inc_path.$theme_path.$theme.'/prepend.inc.php';
}
else
{
  // ini file does not exist : no installation yet ?
  $p = $_SERVER['REQUEST_URI'];
  $app_path = createWebAppPath($p);

  require 'inc/classes/null_db.class.php';
  $con = new null_db();

  $error_message =
  tr('No configuration. Perhaps have you just deployed Wiclear.').'%%%'.
  tr('In that case, you now need to go to the').' [install program|'.$app_path.'install/], '.
  tr('to setup configuration and finish installation');

  $template_path = 'themes/default/template.php';
  $template_prepend_file = 'themes/default/prepend.inc.php';
}

if ($con->connected())
{
  //$con->setDebug(true);

  /**
   * \brief global wiki controller variable
   */
  $wiki = new wiki($con, wc_db_prefix);

  /**
   * \brief current connected user
   */
  $user = auth::checkRights($wiki);

  // translation
  /**
   * \brief lang of user for the interface, always set to prefered lang of user
   */
  $ui_lang = wc_default_lang;
  if (!$user->isAnonymous())
  {
    $ui_lang = $user->ui_lang;
  }

  // always translate in the lang specified in the user's preferences
  if ($ui_lang != 'en')
  {
    $translation = new translation(wc_inc_path.'/i18n', $ui_lang, 'en', '.lang');
  }

  // locale
  setlocale (LC_ALL, $ui_lang);

  $user_lang = wc_default_lang;
  // set lang to asked content lang
  if (!$user->isAnonymous())
  {
    $user_lang = $user->lang;
  }

  // now overload user's lang preference by the one specified in the URL
  if (isset($_GET['lang']))
  {
    $user_lang = varGet('lang');
  }

  /**
   * \brief if we're looking to the main page of the site or not
   */
  $mainPage = true;

  if (isset($_GET['title']) || !empty($mode))
  {
    $mainPage = false;
  }
}
else
{
  $node = new node(
    0, tr('Wiclear maintenance mode'), '',
    $error_message, '', major_value,
    '',
    0, new lang(0, wc_default_lang, 'default_lang', 'flag_default.png'), 0, 0, 0, 0,
    0, '', '', '', ''
  );
  $node->parents = array();

  $wiki = new wiki($con, '');
  $user = $wiki->anonymousUser();
  $user_lang = wc_default_lang;
  $ui_lang = wc_default_lang;
  $mainPage = true;
}

/**
 * \brief global variable. indicates if we want to show table of content on current page
 */
$showTOC = ($mode != 'print' && wc_show_toc == 'true');

/**
 * \brief global variable. indicates if we want to edit a page by section
 */
$editBySection = ($mode != 'print' && wc_edit_by_section == 'true');

/**
 * \brief global variable. indicates if we want to display the lang next to the title
 */
$showLangWithTitle = ($mode != 'print' && wc_show_lang_with_title == 'true');

/**
 * \brief global variable. This is a callback to load main page (no title, no mode, no nada)
 */
$mainPageHandler = '';

/**
 * \brief global variable to control external javascript library to include
 *
 * Each page that want to include a javascript library should add a relative link from the "tool" path.
 * This is used only in site_plan plugin for now.
 */
$javascriptLibs = array();

/**
 * \brief global variable to control main area display
 *
 * This is a map<mode=>callback>
 * If no mode is active, then current wiki page will be displayed. If a mode is active, and
 * this mode exists in the map, the corresponding callback is called.
 * For example, the login page add a callback to the function loginBox() from the mode "login".
 * The mode is read from the url either via the "mode" HTML GET variable, or by the SpecialPage syntax.
 * The callback signature is :
 * <pre>void callback()</pre>
 * Plugins can use this global array to dynamically add new functionalities
 */
$contentBoxHandle = array();

/**
 * \brief global variable to control user interaction
 *
 * This is a map<mode=>callback>
 * If no mode is active, nothing will be called. If a mode is active, and
 * this mode exists in the map, the corresponding callback is called.
 * This callback is called before anything. So you can use header() call to make redirection.
 * For example, the login page add a callback to the function onLogin() from the mode "login".
 * The mode is read from the url either via the "mode" HTML GET variable, or by the SpecialPage syntax.
 * The callback signature is :
 * <pre>void callback()</pre>
 * Plugins can use this global array to dynamically add new functionalities
 */
$userActionHandle = array();

/**
 * \brief global variable to customize title in browser
 * if no mode is active, default handle "default" is used, else handle $mode is used.
 */
$browserTitlePageHandle = array();

require wc_inc_path.'inc/classes/table.class.php';
/**
 * \brief global variable to control rendering of tabular data
 *
 * A rendered used to render tabular data
 */
$tableRenderer = null;
if (wc_table_rendering == 'table')
{
  $tableRenderer = new HtmlTableRenderer();
}
else
if (wc_table_rendering == 'big_div')
{
  $tableRenderer = new HtmlBigDivTableRenderer();
}
else
{
  // default
  $tableRenderer = new HtmlTableRenderer();
}


require wc_inc_path.'inc/classes/plugin_manager.class.php';
/**
 * \brief global variable the plugin manager
 */
$pluginManager = new PluginManager(wc_inc_path);
$pluginManager->importPlugins();

// boxes
require wc_inc_path.'/inc/lib/boxes.lib.php';
?>
