<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****
#
# This file is heavily based on index.php of the dotclear project

error_reporting(E_ALL);
@ini_set('session.use_cookies','1');
@ini_set('session.use_only_cookies','1');
@ini_set('url_rewriter.tags','');
@ini_set('session.use_trans_sid','0');
@ini_set('display_errors','on');

/**
 * \file
 * \brief this page is responsible for sending appropriate headers, and calling prepend.inc.php as well as the template
 *
 * <ul>
 *   <li>the file prepend.inc.php is first called,</li>
 *   <li>the appropriate headers are sent,</li>
 *   <li>the template choosen in the configuration is called, and</li>
 *   <li>the file append.inc.php is called</li>
 * </ul>
 */

$file_path = __FILE__;
$path      = dirname(__FILE__).'/';
// let's use a constant instead of a variable so that hacker can't use it to inject something using a require
// on an unprotected directory
define('wc_inc_path', $path);
unset($path);

// util
require wc_inc_path.'/inc/classes/util.class.php';
util::sessionWorkaround(wc_inc_path);
session_start();

// variables
require wc_inc_path.'/inc/lib/variables.lib.php';
// auth
require wc_inc_path.'/inc/classes/auth.class.php';
list($user_id, $user_password) = auth::getUserInfoFromCookieOrSession();

// conf
require wc_inc_path.'/inc/classes/WiclearIni.conf.php';
require wc_inc_path.'/inc/classes/ini.class.php';

/**
 * \brief global configuration variable
 */
$ini = new ini(wc_inc_path.'/conf/config.ini');
$ini->importConstants();

@ini_set('default_charset',wc_charset);
@ini_set('mbstring.func_overload','7');
@ini_set('mbstring.internal_encoding',wc_charset);
@ini_set('mbstring.internal_encoding',wc_charset);

// urls
require wc_inc_path.'/inc/lib/url.lib.php';
decodeUrl();

/**
 * \brief current mode of wiki, it controls the behaviour or the active plugin
 */
$mode = '';
if (isset($_GET['mode']))
{
  $mode = varGet('mode');
}

/**
 * \brief the title of the page we're looking at
 */
$title = '';
if (isset($_GET['title']))
{
  $title = varGet('title');
}

/**
 * \brief lang of user for the content, can be overriden by the lang specified in the url
 */
$user_lang = wc_default_lang;
// now overload user's lang preference by the one specified in the URL
if (isset($_GET['lang']))
{
  $user_lang = varGet('lang');
}

// cache
require wc_inc_path.'/inc/classes/PEAR/Cache_Lite.class.php';
require wc_inc_path.'/inc/classes/WiclearCache.class.php';
/**
 * \brief the cache
 */
$cacheLite = new WiclearCache($user_id, $mode, $title, $user_lang);

// Test if there is a valid cache for this id
if (($data = $cacheLite->wcGet()) != false)
{
  // cache hit
}
else
{
  ob_start();

  // world setup
  require wc_inc_path.'inc/prepend.inc.php';

  // if a prepend.inc.php exist in the theme, load it
  if (file_exists($template_prepend_file))
  {
    require $template_prepend_file;
  }
  // load global info for current page from database
  loadCurrentPage();
  // gives a chance to handle forms before outputting something
  handleUserAction();
  // creates browser title
  $browser_title_page = handleBrowserTitlePage();
  // finally load user's template to display web page
  echo '<?xml version="1.0" encoding="'.wc_charset.'"?>';
  require $template_path;

  require 'inc/append.inc.php';

  $data = ob_get_contents();
  ob_end_clean();

  global $node;
  global $mainPage;
  if ($mainPage || !empty($node))
  {
    $cacheLite->wcSave($data);
  }
}

// send HTTP headers
$accept_xml = wc_wiki_allow_html == 'false' &&
  isset($_SERVER['HTTP_ACCEPT']) &&
  strpos($_SERVER['HTTP_ACCEPT'],'application/xhtml+xml') !== false;

if ($accept_xml)
{
  header('Content-Type: application/xhtml+xml; charset='.wc_charset);
}
else
{
  header('Content-Type: text/html; charset='.wc_charset);
}

echo $data;
?>
