<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

@ini_set('display_errors','on');

function getCharsets($con)
{
  $available_charsets = array();

  $query =<<<SQL
    SHOW CHARACTER SET;
SQL;
  $result = $con->select($query);

  foreach ($result as $r)
  {
    if (!in_array($r[0], $available_charsets) && in_array($r[0], $con->knownDbCharset))
    {
      $available_charsets[] = $r[0];
    }
  }

  return $available_charsets;
}

$db_host     = '';
$db_login    = '';
$db_password = '';
$db_database = '';
$db_prefix   = 'wc_';
$db_charset  = '';
$db_err      = '';

require '../inc/lib/variables.lib.php';

// form validated : try to connect to database, check result and check available charsets
if (!empty($_POST['check']))
{
  $db_host     = varPost('host');
  $db_login    = varPost('login');
  $db_password = varPost('password');
  $db_database = varPost('database');
  $db_prefix   = varPost('prefix');
  $db_charset  = isset($_POST['charset']) ? varPost('charset') : '';

  require '../inc/classes/chrono.class.php';
  require '../inc/classes/db_connection.class.php';
  require '../inc/classes/mysql.class.php';

  $con = new mysql($db_host, $db_login, $db_password, $db_database);

  if (empty($con->error))
  {
    // connected
    if (empty($db_charset))
    {
      // no charset information : retrieve them
      $available_charsets = getCharsets($con);
    }
    else
    {
      // set by user
      $available_charsets = array();
      $available_charsets[] = $db_charset;
    }

    // no charset, or only one charset available
    if (count($available_charsets) <= 1)
    {
      if (count($available_charsets) == 1)
      {
        // only one charset available : default to it now
        $db_charset = $available_charsets[0];
      }
      else
      {
        // no charset support at all : let's default to php support only
      }

      $path      = dirname(__FILE__).'/../';
      require $path.'/inc/classes/util.class.php';
      util::sessionWorkaround($path);

      @ini_set('session.use_cookies','1');
      @ini_set('session.use_only_cookies','1');
      @ini_set('url_rewriter.tags','');
      @ini_set('session.use_trans_sid', '0');
      session_start();

      setVarSession('db_host',     $db_host);
      setVarSession('db_login',    $db_login);
      setVarSession('db_password', $db_password);
      setVarSession('db_database', $db_database);
      setVarSession('db_prefix',   $db_prefix);
      setVarSession('db_charset',  $db_charset);

      header('Location: install_tables.php');
      exit;
    }
  }
  else
  {
    // could not connect : why ?
    $db_err = $con->error;
  }
}

require 'prepend.php';
define('wc_charset', 'ISO-8859-1');
require '../inc/classes/form.class.php';

if (!empty($db_err))
{
  echo
  '<p>'.tr('Please, check your configuration : ').$db_err.'</p>';
}

echo
'<h1>'.tr('Step').' 2/4 : '.tr('Database configuration').'</h1>';

$form = new form('db_conf.php', 'post', 'framed');

$form->addHtml('<p class="field">');
$form->addLabel(tr('Sql host'), 'fsql');
$form->addInput('host', 'Host', $db_host, 'fsql');
$form->addHtml('</p>');

$form->addHtml('<p class="field">');
$form->addLabel(tr('Login'), 'flogin');
$form->addInput('login', 'Login', $db_login, 'flogin');
$form->addHtml('</p>');

$form->addHtml('<p class="field">');
$form->addLabel(tr('Password'), 'fpassword');
$form->addInput('password', 'Password', $db_password, 'fpassword');
$form->addHtml('</p>');

$form->addHtml('<p class="field">');
$form->addLabel(tr('Database'), 'fdatabase');
$form->addInput('database', 'Database', $db_database, 'fdatabase');
$form->addHtml('</p>');

$form->addHtml('<p class="field">');
$form->addLabel(tr('Prefix'), 'fprefix');
$form->addInput('prefix', 'Prefix', $db_prefix, 'fprefix');
$form->addHtml('</p>');

if (!isset($available_charsets))
{
  // displaying this form for the first time
  // nothing to do
}
else
{
  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Charset'), 'fcharset');
  $form->addDropDown($available_charsets, $db_charset, 'charset', 'fcharset');
  $form->addHtml('</p>');
}

$form->addHtml('<p class="field">');
$form->addHidden('check', '1');
$form->addHtml('</p>');

if (!isset($available_charsets))
{
  $form->addHtml('<p class="field">');
  $form->addButton('submit', tr('Lookup charsets/Create tables'), 'action');
  $form->addHtml('</p>');
}
else
{
  $form->addHtml('<p class="field">');
  $form->addButton('submit', tr('Create tables'), 'action');
  $form->addHtml('</p>');
}

echo $form->toHtml();

require 'append.php';
?>
