<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

require 'prepend.php';
require '../inc/classes/check_list.class.php';
define('wc_charset', 'ISO-8859-1');
require '../inc/classes/form.class.php';

$check = new check_list('ok', 'nok', 'warn');

$confDir  = '../conf';
$confFile = $confDir.'/config.ini';
$check->addItem(
  is_writable($confDir) && (!file_exists($confFile) || is_writable($confFile)),
  $confDir.' '.tr('is writable'),
  $confDir.' '.tr('is not writable, or file ').$confFile.' '.tr('is not writable')
);
$check->addItem(
  function_exists('mysql_connect'),
  tr('MySql is installed'),
  tr('MySql is not installed')
);
$check->addItem(
  file_exists($session_dir) && is_writable($session_dir),
  tr('session dir').' '.$session_dir.' '.tr('exists and is writable'),
  tr('session dir').' '.$session_dir.' '.tr('does not exist or is not writable : sessions will not work'),
  true
);

if (file_exists($confFile))
{
  echo
  '<h1>'.tr('Welcome to Wiclear update wizard').'</h1>';
  echo
  '<h2>'.tr('Step').' 1/3 : '.tr('Check rights and php functions').'</h2>';
}
else
{
  echo
  '<h1>'.tr('Welcome to Wiclear installation process').'</h1>';
  echo
  '<h2>'.tr('Step').' 1/4 : '.tr('Check rights and php functions').'</h2>';
}

echo $check->htmlReport();

if ($check->allOK())
{
  if ($check->hasWarning())
  {
    echo '<h1>'.tr('Warning').'</h1>';
    echo '<p>'.tr('At least one operation generated a warning.').
               tr(' This is not necessary fatal, and installation should be able to proceed.').
               tr(' But you should expect things to break or appears not to work as expected.').
               tr(' Try to fix the problem if possible.').
         '</p>';
  }

  if (file_exists($confFile))
  {
    $form = new form('update.php', 'post');
    $form->addHtml('<p class="field">');
    $form->addButton('submit', tr('Update existing configuration'), 'action');
    $form->addHtml('</p');
  }
  else
  {
    $form = new form('db_conf.php', 'post');
    $form->addHtml('<p class="field">');
    $form->addButton('submit', tr('Configure database access'), 'action');
    $form->addHtml('</p');
  }

  echo $form->toHtml();
}
else
{
  echo
  '<p>'.tr('You will not be able to install Wiclear : please check your configuration').'</p>';
}

setVarSession('session_ok', 'Session OK');

require 'append.php';
?>
