<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

require 'prepend.php';
require '../inc/classes/form.class.php';
require '../inc/classes/WiclearIni.conf.php';
require '../inc/classes/ini.class.php';
require '../inc/classes/chrono.class.php';
require '../inc/classes/db_connection.class.php';
require '../inc/classes/mysql.class.php';
require '../inc/classes/check_list.class.php';
require '../inc/classes/lang.class.php';
require '../inc/lib/constants.lib.php';
require '../inc/classes/wiki.class.php';
require '../admin/inc/classes/admin_wiki.class.php';

function scanUpdateFiles()
{
  $updatesBaseDir = 'updates/';
  if (file_exists($updatesBaseDir))
  {
    $h = opendir($updatesBaseDir);
    while (($file = readdir($h)) !== false)
    {
      $updateName = basename($file, '.php');

      if (file_exists($updatesBaseDir.$updateName.'.php'))
      {
        require $updatesBaseDir.$updateName.'.php';
      }
    }
    closedir($h);
  }
}

function chooseUpdateFrom()
{
  global $updateInfo;

  $versions = array();
  foreach ($updateInfo as $from=>$to)
  {
    $versions[] = $from;
  }

  echo '<p>'.tr('DO NOT FORGET TO BACKUP YOUR DATABASE BEFORE TRYING THIS').'</p>';

  define('wc_charset', 'ISO-8859-1');
  $form = new form('update.php', 'post');
  $form->addHtml('<p class="field">');
  $form->addDropDown($versions, '0.6', 'version_id');
  $form->addHtml('</p');

  $form->addHtml('<p class="field">');
  $form->addHidden('step', '1');
  $form->addHtml('</p');

  $form->addHtml('<p class="field">');
  $form->addButton('submit', tr('Update'), 'action');
  $form->addHtml('</p');

  echo $form->toHtml();
}

function updateFrom($version)
{
  global $updateInfo;

  $ini = new ini('../conf/config.ini');
  $ini->importConstants();

  $wiki->con = new mysql(wc_db_host, wc_db_login, wc_db_password, wc_db_database);
  $wiki = new admin_wiki($wiki->con, wc_db_prefix);

  $ok = true;

  while (array_key_exists($version, $updateInfo))
  {
    $toAndUpdateFunction = $updateInfo[$version];

    $toVersion      = $toAndUpdateFunction[0];
    $updateFunction = $toAndUpdateFunction[1];

    // invokes update function
    $ok = $updateFunction($wiki, $ini);
    $version = $toVersion;
  }

  if ($ok)
  {
    echo '<p>'.tr('Installation successfully updated').'</p>';

    $form1 = new form('../', 'post');
    $form1->addHtml('<p class="field">');
    $form1->addButton('submit', tr('Go back to wiki'), 'action');
    $form1->addHtml('</p>');

    echo $form1->toHtml();

    $form2 = new form('../admin', 'post');
    $form2->addHtml('<p class="field">');
    $form2->addButton('submit', tr('Go to admin panel'), 'action');
    $form2->addHtml('</p>');

    echo $form2->toHtml();
  }
  else
  {
    echo '<p>'.tr('Error occurred, hope you have made a backup').'</p>';
  }
}

scanUpdateFiles();

$step = 0;
if (!empty($_POST['step']))
{
  $step = varPost('step');
}

switch ($step)
{
case 0 :
  chooseUpdateFrom();
break;

case 1 :
  $version = varPost('version_id');
  updateFrom($version);
break;
}

require 'append.php';
?>
