<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

global $updateInfo;
$updateInfo['0.6'] = array('0.7', 'updateFrom0dot6To0dot7');

function updateFrom0dot6To0dot7($wiki, $ini)
{
  $check = new check_list('ok', 'nok');

  // create lang table
  $query = <<<SQL
  CREATE TABLE {prefix}lang (
    id          int(6)    NOT NULL auto_increment,
    code        char(3)   NOT NULL,
    description char(100) NOT NULL,
    flag        char(100) NOT NULL,

    PRIMARY   KEY (id)
  );
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $ok = $wiki->con->execute($query);

  if ($ok)
  {
    // populate default values
    $ok = $ok && $wiki->addLang('en', 'English', 'flag_en.png');
    $wiki->addLang('fr', 'Français', 'flag_fr.png');
  }

  $check->addItem(
    $ok,
    tr('Table lang successfully created'),
    tr('Error while creating table lang : ').$wiki->con->error
  );

  // modify user table :
  // add lang_fk and remove lang
  $query = <<<SQL
    ALTER TABLE {prefix}user
    DROP   lang,
    ADD    lang_fk int(6) NOT NULL,
    CHANGE admin admin int(1) NOT NULL
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $ok = $wiki->con->execute($query);

  if ($ok)
  {
    $query = <<<SQL
      UPDATE {prefix}user
      SET lang_fk='1';
SQL;

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $ok = $wiki->con->execute($query);
  }

  $check->addItem(
    $ok,
    tr('Table user successfully modified'),
    tr('Error while modifying table user : ').$wiki->con->error
  );

  // create table content_hierarchy
  $query = <<<SQL
  CREATE TABLE {prefix}content_hierarchy
  (
    parent_content_fk       int(6),
    content_fk              int(6),
    left_n                  int(6),
    right_n                 int(6)
  );
SQL;
  $query = str_replace('{prefix}', wc_db_prefix, $query);

  $check->addItem(
    $wiki->con->execute($query),
    tr('Table content_hierarchy successfully created'),
    tr('Error while creating table content_hierarchy : ').$wiki->con->error
  );

  // create table content_lang
  $query = <<<SQL
  CREATE TABLE {prefix}content_lang
  (
    related_content_fk      int(6),
    content_fk              int(6)
  );
SQL;
  $query = str_replace('{prefix}', wc_db_prefix, $query);

  $check->addItem(
    $wiki->con->execute($query),
    tr('Table content_lang successfully created'),
    tr('Error while creating table content_lang : ').$wiki->con->error
  );

  // modify content table :
  // add alternate_title, lang_fk, and
  // update content_hierarchy
  // remove parent_content_fk, left_n and right_n
  $query = <<<SQL
    ALTER TABLE {prefix}content
    ADD alternate_title char(100),
    ADD lang_fk int(6),
    ADD INDEX (title)
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $ok = $wiki->con->execute($query);

  if ($ok)
  {
    $query = <<<SQL
      UPDATE {prefix}content
      SET lang_fk='1';
SQL;

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $ok = $wiki->con->execute($query);
  }

  if ($ok)
  {
    $query = <<<SQL
      INSERT {prefix}content_hierarchy
      ({prefix}content_hierarchy.parent_content_fk,
       {prefix}content_hierarchy.content_fk,
       {prefix}content_hierarchy.left_n,
       {prefix}content_hierarchy.right_n
      )
      SELECT {prefix}content.parent_content_fk, {prefix}content.id, {prefix}content.left_n, {prefix}content.right_n
      FROM {prefix}content;
SQL;

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $ok = $wiki->con->execute($query);
  }

  if ($ok)
  {
    $query = <<<SQL
      ALTER TABLE {prefix}content
      DROP parent_content_fk,
      DROP left_n,
      DROP right_n
SQL;

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $ok = $wiki->con->execute($query);
  }

  if ($ok)
  {
    $query = <<<SQL
      INSERT {prefix}content_lang
      ({prefix}content_lang.related_content_fk,
       {prefix}content_lang.content_fk
      )
      SELECT {prefix}content.id, {prefix}content.id
      FROM {prefix}content;
SQL;

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $ok = $wiki->con->execute($query);
  }

  $check->addItem(
    $ok,
    tr('Table content successfully modified'),
    tr('Error while modifying table content : ').$wiki->con->error
  );

  // modify history
  // add alternate_title
  // remove parent_content_fk
  $query = <<<SQL
      ALTER TABLE {prefix}history
      ADD alternate_title char(100),
      DROP parent_content_fk
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);

  $check->addItem(
    $wiki->con->execute($query),
    tr('Table history successfully modified'),
    tr('Error while modifying table history : ').$wiki->con->error
  );

  // rename group to groups
  $query = <<<SQL
      ALTER TABLE {prefix}group
      RENAME TO {prefix}groups
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);

  $check->addItem(
    $wiki->con->execute($query),
    tr('Table groups successfully renamed'),
    tr('Error while renaming table groups : ').$wiki->con->error
  );

  // canonise
  $check->addItem(
    $wiki->canonize(),
    tr('Tables successfully canonised'),
    tr('Error while canonising tables (not fatal) : ').$wiki->con->error
  );

  // compute nested set model
  $wiki->computeNestedSetModel();

  // change config.ini file
  $ini->addConstant('wc_mail', 'webmaster@'.$_SERVER['SERVER_NAME']);
  $ini->forceSaveFile();

  // report
  echo $check->htmlReport();

  return $check->allOK();
}

?>
