<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

global $updateInfo;
$updateInfo['0.9'] = array('0.10', 'updateFrom0dot9To0dot10');

function updateFrom0dot9To0dot10($wiki, $ini)
{
  $check = new check_list('ok', 'nok');

  // create table ban
  $query = <<<SQL
  CREATE TABLE {prefix}ban 
  (
    id          int(6)    NOT NULL auto_increment,
    ip          char(15),

    PRIMARY     KEY (id),
    INDEX       (ip)
  );
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $ok = $wiki->con->execute($query);

  $check->addItem(
    $ok,
    tr('Table ban successfully created'),
    tr('Error while creating table ban : ').$wiki->con->error
  );

  // alter table history
  // add a parent_content_fk field
  $query = <<<SQL
    ALTER TABLE {prefix}history
    ADD parent_content_fk int(6)
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $ok = $wiki->con->execute($query);

  if ($ok)
  {
    // force value to current parent node id
    $query = <<<SQL
      UPDATE {prefix}history
      JOIN {prefix}content_hierarchy on {prefix}content_hierarchy.content_fk={prefix}history.content_fk
      SET {prefix}history.parent_content_fk={prefix}content_hierarchy.parent_content_fk
SQL;

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $ok = $wiki->con->execute($query);
  }

  $check->addItem(
    $ok,
    tr('Table history successfully modified'),
    tr('Error while modifying table history : ').$wiki->con->error
  );

  // alter table content_group
  // add a acl_type field
  $query = <<<SQL
    ALTER TABLE {prefix}content_group
    ADD acl_type int(2) NOT NULL
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $ok = $wiki->con->execute($query);

  if ($ok)
  {
    // force value to acl_write
    $default_value = acl_write;
    $query = <<<SQL
      UPDATE {prefix}content_group
      SET acl_type='$default_value'
SQL;

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $ok = $wiki->con->execute($query);
  }

  $check->addItem(
    $ok,
    tr('Table content_group successfully modified'),
    tr('Error while modifying table content_group : ').$wiki->con->error
  );

  // change blob types to longtext types
  $tableBlobs = array(
    array('content',          'content'),
    array('comment',          'content'),
    array('history',          'content'),
    array('metadata_content', 'value'),
    array('metadata_value',   'value')
  );

  $charset = $wiki->con->htmlCharsetToMySqlCharset(wc_charset);
  if (!empty($charset))
  {
    // check if charset is supported
    $query = <<<SQL
      SHOW CHARACTER SET LIKE '$charset%';
SQL;
    $result = $wiki->con->select($query);
    if (empty($result))
    {
      // not supported
      $charset = '';
    }
  }

  foreach ($tableBlobs as $tableBlob)
  {
    $table = $tableBlob[0];
    $field = $tableBlob[1];

    $query = <<<SQL
      ALTER TABLE {prefix}$table
      CHANGE $field $field LONGTEXT;
SQL;

    if (!empty($charset))
    {
      $query .= ' CHARACTER SET '.$charset;
    }

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $ok = $wiki->con->execute($query);

    $check->addItem(
      $ok,
      tr('Table').' '.$table.' '.tr('successfully modified'),
      tr('Error while modifying table').' '.$table.' : '.$wiki->con->error
    );
  }

  // optimizations
  $tableToIndexes = array(
    'metadata_value'     => array('metadata_fk'),
    'attachments'        => array('content_fk'),
    'comment'            => array('content_fk'),
    'content_group'      => array('content_fk'),
    'content_hierarchy'  => array('content_fk', 'parent_content_fk'),
    'content_lang'       => array('content_fk', 'related_content_fk'),
    'content'            => array('lang_fk'),
    'content_user_email' => array('user_fk'),
    'history'            => array('content_fk'),
    'user_group'         => array('group_fk'),
    'metadata_content'   => array('content_fk')
  );

  foreach ($tableToIndexes as $table=>$indexes)
  {
    $sql = <<<SQL
      ALTER TABLE {prefix}$table
SQL;
    $first = true;
    foreach ($indexes as $index)
    {
      if ($first == false)
      {
        $sql .= ',';
      }

      $sql .= <<<SQL
        ADD INDEX($index)
SQL;

      $first = false;
    }

    $sql = str_replace('{prefix}', wc_db_prefix, $sql);
    $ok = $wiki->con->execute($sql);

    $check->addItem(
      $ok,
      tr('Table ').$table.' '.tr('successfully modified'),
      tr('Error while modifying table ').$table.' : '.$wiki->con->error
    );
  }

  // report
  echo $check->htmlReport();

  return $check->allOK();
}

?>
