<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004 Grzesiek, adapted by David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief this file reads a string from a session variable and generates an image from it.
 *
 * This feature is used in the registration process. It tries to stop basic bots from registering to the site.
 * a random text is generated from the calling page, it is stored in a session variable, then
 * an image is generated with some noise (so that OCR tool cannot easily guess it but human user can)
 * and served with a image/png content type.
 */

$file_path = __FILE__;
$path      = dirname(__FILE__).'/';
// let's use a constant instead of a variable so that hacker can't use it to inject something using a require
// on an unprotected directory
define('wc_inc_path', $path);
unset($path);

require wc_inc_path.'/inc/classes/util.class.php';
util::sessionWorkaround(wc_inc_path);

@ini_set('session.use_cookies','1');
@ini_set('session.use_only_cookies','1');
@ini_set('url_rewriter.tags','');
@ini_set('session.use_trans_sid','0');
@ini_set('display_errors','on');
@ini_set('default_charset',wc_charset);
@ini_set('mbstring.func_overload','7');
@ini_set('mbstring.internal_encoding',wc_charset);
@ini_set('mbstring.internal_encoding',wc_charset);
session_start();

require wc_inc_path.'/inc/lib/variables.lib.php';

if (!existVarSession('user_antibot'))
{
  exit;
}

$user_antibot = varSession('user_antibot');
$len = strlen($user_antibot);

$width = 9 * $len + 1;
$height = 18;

// create text
$img = ImageCreate($width, $height);
$bg_color   = ImageColorAllocate($img, 255, 255, 255);
ImageColorTransparent($img, $bg_color);
$text_color = ImageColorAllocate($img, 0, 0, 200);
ImageString($img, 9, 1, 0, $user_antibot, $text_color);

// create another image with ellipses
$img2 = ImageCreate($width, 2 * $height);
$bg2 = ImageColorAllocate($img2, 220, 220, 255);
$fg2 = ImageColorAllocate($img2, 190, 190, 235);

for ($i = 0; $i < rand(3,6); ++$i)
{
  ImageEllipse(
    $img2,
    rand($width / 8 , $width - $width / 8),
    rand($height / 3, $height - $height / 3),
    rand($width / 2,  $width * 2),
    rand($height / 2, $height * 2),
    $fg2
  );
}

// copy text image to ellipse image line by line
for ($i = 0; $i < $height; ++$i)
{
  ImageCopy($img2, $img, 0, $i * 2, 0, $i, $width, 1);
}

ImageDestroy($img);

header('Content-type: image/png');
ImagePng($img2);
?>
