<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief customize browser title when viewing site plan
 *
 * \return browser title
 */
function browserSitePlan()
{
  return wc_wiki_name.' - '.tr('Site plan');
}

// global variables so that it works with PHP4 :-(
$id2Index = array();
$result   = array();

function sitePlanSort($childId1, $childId2)
{
  global $id2Index;
  global $result;

  $i1         = $id2Index[$childId1];
  $r1         = $result[$i1];
  $title1     = $r1[1];
  $alt_title1 = $r1[2];
  if (empty($alt_title1))
  {
    $alt_title1 = $title1;
  }

  $i2         = $id2Index[$childId2];
  $r2         = $result[$i2];
  $title2     = $r2[1];
  $alt_title2 = $r2[2];
  if (empty($alt_title2))
  {
    $alt_title2 = $title2;
  }

  return strcasecmp($alt_title1, $alt_title2);
}

function recDisplayNodes(&$adjacency, &$translations, $id, $read_protected_ids)
{
  global $id2Index;
  global $result;

  $i         = $id2Index[$id];
  $r         = $result[$i];
  $title     = $r[1];
  $alt_title = $r[2];
  $lang      = $r[4];
  $desc      = $r[5];
  $flag      = $r[6];
  if (empty($alt_title))
  {
    $alt_title = $title;
  }

  $str = '<li>';
  if (!empty($adjacency[$id]))
  {
    //$str .= '<a href="#'.$id.'" class="expanded_node" name="'.$id.'"></a>';
    $str .= '<a href="#" class="expanded_node" id="node'.$id.'" name="node'.$id.'"></a>';
  }

  $flagTag = '<img src="'.createIconURL($flag).'" alt="'.$desc.'"/>';
  $str .= '<span>'.formatHtml($alt_title).'</span>';
  $str .= '<ul class="flag">';

  if (in_array($id, $read_protected_ids))
  {
    $display = $flagTag.'<img src="'.createIconURL('icons/lock.png').'" alt="'.tr('Read protected').'"/>';
  }
  else
  {
    $display = '<a href="'.createURL($title, array('lang'=>$lang)).'">'.$flagTag.'</a>';
  }

  $str .= '<li>'.$display.'</li>';

  if (!empty($translations[$id]))
  {
    $trans = $translations[$id];
    foreach ($trans as $transId)
    {
      $ti    = $id2Index[$transId];
      $tr    = $result[$ti];
      $tlang = $tr[4];
      $tdesc = $tr[5];
      $tflag = $tr[6];

      $tFlagTag = '<img src="'.createIconURL($tflag).'" alt="'.$tdesc.'"/>';

      if (in_array($transId, $read_protected_ids))
      {
        $tdisplay = $tFlagTag.'<img src="'.createIconURL('icons/lock.png').'" alt="'.tr('Read protected').'"/>';
      }
      else
      {
        $tdisplay = '<a href="'.createURL($title, array('lang'=>$tlang)).'">'.$tFlagTag.'</a>';
      }

      $str .= '<li>'.$tdisplay.'</li>';
    }
  }
  $str .= '</ul>';

  if (!empty($adjacency[$id]))
  {
    $childs = $adjacency[$id];
    usort($childs, 'sitePlanSort');

    $str .= '<ul>';
    foreach ($childs as $childId)
    {
      $str .= recDisplayNodes($adjacency, $translations, $childId, $read_protected_ids);
    }
    $str .=
    '</ul>';
  }

  $str .= '</li>';

  return $str;
}

function displaySitePlan()
{
  global $con;
  global $result;
  global $id2Index;
  global $user;

  $query = <<<SQL
      SELECT {prefix}content.id, {prefix}content.title, {prefix}content.alternate_title,
             {prefix}content_hierarchy.parent_content_fk,
             {prefix}lang.code, {prefix}lang.description, {prefix}lang.flag,
             {prefix}content_lang.related_content_fk
      FROM   {prefix}content
      JOIN   {prefix}content_lang on {prefix}content_lang.content_fk={prefix}content.id
      JOIN   {prefix}content_hierarchy on {prefix}content_hierarchy.content_fk={prefix}content_lang.related_content_fk
      JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $result = $con->select($query);

  $read_protected_ids = array();
  if (!$user->isModerator() && !$user->isAdmin())
  {
    $acl_read     = acl_read;
    $anonymous_id = user_anonymous_id;

    $subQuery1 = <<<SQL
      SELECT wc_content_group.content_fk
      FROM   wc_content_group
      JOIN   wc_user_group on wc_user_group.group_fk=wc_content_group.group_fk
      WHERE  (wc_user_group.user_fk='$user->id' OR wc_user_group.user_fk='$anonymous_id')
      AND    wc_content_group.acl_type='$acl_read'
SQL;

    $subQuery2 = <<<SQL
      SELECT wc_content_group.content_fk
      FROM   wc_content_group
      WHERE  (wc_content_group.user_fk='$user->id' OR wc_content_group.user_fk='$anonymous_id')
      AND    wc_content_group.acl_type='$acl_read'
SQL;

    $query = <<<SQL
      SELECT DISTINCT {prefix}content_group.content_fk
      FROM   {prefix}content_group
      WHERE  {prefix}content_group.acl_type='$acl_read'
      AND    {prefix}content_group.content_fk NOT IN ($subQuery1)
      AND    {prefix}content_group.content_fk NOT IN ($subQuery2)
SQL;

    $query = str_replace('{prefix}', wc_db_prefix, $query);
    $read_protected_result = $con->select($query);

    foreach ($read_protected_result as $r)
    {
      $read_protected_ids[] = $r[0];
    }
  }

  // create a list parent_id => list(child_id)
  $adjacency    = array();
  $id2Index     = array();
  $translations = array();
  $root_id = 0;
  $nbResult = sizeof($result);
  for ($i = 0; $i < $nbResult; ++$i)
  {
    $r   = $result[$i];
    $id  = $r[0];
    $p   = $r[3];
    $rel = $r[7];

    $id2Index[$id] = $i;

    // translations
    if ($id != $rel)
    {
      if (empty($translations[$rel]))
      {
        $translations[$rel] = array();
      }

      $translations[$rel][] = $id;
    }

    if ($id == $p)
    {
      $root_id = $id;
    }

    // hierarchy
    if ($id == $rel /*master node*/ && $id != $p /*not root node*/)
    {
      if (empty($adjacency[$p]))
      {
        $adjacency[$p] = array();
      }

      $adjacency[$p][] = $id;
    }
  }

  return
  '<div id="site_plan_box">'.
  '<ul>'.
  recDisplayNodes($adjacency, $translations, $root_id, $read_protected_ids).
  '</ul>'.
  '<div class="spacer">&nbsp;</div>'.
  '</div>';
}
?>
