<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

global $pluginManager;
// callback installation
$pluginManager->addValidatePluginCallback('spamSimpleFilterCallback');

function spamSimpleFilterSetup()
{
  global $mode;

  if ($mode == 'edit' || $mode == 'add' || $mode == 'comment')
  {
    global $pluginManager;
    $pluginManager->loadPlugin('spam_simple_filter');
  }
}

function wordMatch($words)
{
  if (empty($words))
  {
    return false;
  }

  global $wiki;

  $query = <<<SQL
    CREATE TEMPORARY TABLE TMP_SPAM_CANDIDATE_WORDS
    (
      word varchar(300) NOT NULL
    );
SQL;

  $wiki->con->execute($query);

  $first = true;
  $insert = 'INSERT INTO TMP_SPAM_CANDIDATE_WORDS (word) VALUES ';
  foreach ($words as $word)
  {
    $word = $wiki->con->escapeString($word);

    if (strlen($word) >= 3)
    {
      if (function_exists('mb_strtolower'))
      {
        $word = mb_strtolower($word);
      }
      else
      {
        $word = strtolower($word);
      }

      if (!$first)
      {
        $insert .= ',';
      }
      else
      {
        $first = false;
      }

      $insert .= "('".$word."')";
    }
  }

  $wiki->con->execute($insert);

  $query = <<<SQL
    SELECT DISTINCT TMP_SPAM_CANDIDATE_WORDS.word
    FROM TMP_SPAM_CANDIDATE_WORDS
    JOIN {prefix}spam_words
    WHERE LOWER(TMP_SPAM_CANDIDATE_WORDS.word) LIKE CONCAT('%',{prefix}spam_words.word,'%')
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);
  $result = $wiki->con->select($query);

  $query = 'DROP TABLE TMP_SPAM_CANDIDATE_WORDS;';
  $wiki->con->execute($query);

  $spam = array();
  foreach ($result as $r)
  {
    $spam[] = $r[0];
  }

  return $spam;
}

function get_url_density($text)
{
  $protocol_reg = '(http|https|ftp):\/\/';
  $login_reg    = '[\w\.]+:([\w,.\$\+]|%\w{2,2})+@';
  $domain_reg   = '\w+(\.\w+)+';
  $ip_reg       = '\d+\.\d+\.\d+\.\d+';
  $port_match   = ':\d+';
  $file_reg     = '[\w+\.]+(\/[\w\.]+)*';
  $arg_reg      = '([-+~;\w]|%\w{2,2})+(=([-+~;\w]|%\w{2,2})+)?';
  $args_reg     = $arg_reg.'(&'.$arg_reg.')*';
  $anchor_reg   = '\w+';
  $reg = '/('.$protocol_reg.')?('.$login_reg.')?('.$domain_reg.'|'.$ip_reg.')('.$port_match.')?(\/'.$file_reg.'?(\?'.$args_reg.')?(#'.$anchor_reg.')?)?/';

  if (preg_match_all($reg, $text, $matches) == 0)
  {
    return 0;
  }

  $len = 0;
  $all_urls = $matches[0];
  foreach ($all_urls as $url)
  {
    $len += strlen($url);
  }

  return $len / strlen($text) * 100;
}

function get_spam_distribution($uniqueSpam, $words_pos, $len)
{
  //E(X/Xmax) - E(X/Xmax)
  //1/N*sum(X/Xmax) - (1/N*sum(X/Xmax))
  //1/N*1/Xmax*sum(X) - 1/N*1/Xmax*(sum(X))
  //1/N*1/Xmax*(sum(X) - 1/N*(sum(X)))
  $sum_x_2   = 0;
  $sum_x     = 0;
  $N         = 0;
  foreach ($uniqueSpam as $spam)
  {
    if (array_key_exists($spam, $words_pos))
    {
      foreach($words_pos[$spam] as $pos)
      {
        $pos_norm = $pos / $len;
        $sum_x_2 += $pos_norm * $pos_norm;
        $sum_x   += $pos_norm;
        ++$N;
      }
    }
  }
  $variance = ($sum_x_2 - $sum_x * $sum_x / $N)/$N;
  return array($N, $variance);
}

function spamSimpleFilterCallback($where, $data, &$reason, $debug = false)
{
  global $user;
  if ($user->isModerator() || $user->isAdmin())
  {
    return true;
  }

  // analyse data
  $allWords  = array();
  $words_pos = array(); // one per data to analyse
  // $words_pos[0] for $data[0]
  // $words_pos[1] for $data[1]
  // ...
  // Then $words_pos keeps all offsets of all words
  // word => array(offset)
  $words_count = array(); // one per data to analyse
  $nb_words = array(); // one per data to analyse

  foreach ($data as $text)
  {
    $matches          = preg_split("/[\W]+/", $text, -1, PREG_SPLIT_OFFSET_CAPTURE);
    $words            = array();
    $current_word_pos = array();

    foreach ($matches as $match)
    {
      // merge words in all words
      $words[] = $match[0];

      // if new word in $current_word_pos, create a new entry
      if (!array_key_exists($match[0], $current_word_pos))
      {
        $current_word_pos[$match[0]] = array();
      }
      // append offset of word
      $current_word_pos[$match[0]][] = $match[1];
    }

    // and add it to $words_pos
    $words_pos[]   = $current_word_pos;
    $words_count[] = array_count_values($words);
    $nb_words[]    = count($words);

    $allWords      = array_merge($allWords, $words);
  }

  // retrieve all spams
  $uniqueWords = array_unique($allWords);
  $uniqueSpam = wordMatch($uniqueWords);
/*
  echo '<pre>';
  print_r($uniqueSpam);
  echo '</pre>';
*/
  // compute score (anonymous starts at 1)
  $spam_score = $user->isAnonymous() ? 1 : 0;
  for ($i = 0; $i < count($data); ++$i)
  {
    if ($debug) echo '<pre>spam_score='.$spam_score.'</pre>';
    // 1- density
    $density = get_url_density($data[$i]);
    if ($density >= 25)
    {
      $spam_score += floor($density / 25);
    }
    if ($debug) echo '<pre>density='.$density.', spam_score='.$spam_score.'</pre>';

    // 2- percent of spam word
    $spamCount = 0;
    foreach ($uniqueSpam as $spam)
    {
      if (array_key_exists($spam, $words_count[$i]))
      {
        $spamCount += $words_count[$i][$spam];
      }
    }
    $spam_percent = $spamCount / $nb_words[$i] * 100;
    if ($spam_percent > 0.5)
    {
      $spam_score += floor($spam_percent / 0.5);
    }
    if ($debug) echo '<pre>spam_percent='.$spam_percent.', spam_score='.$spam_score.'</pre>';

    if ($spam_percent > 0)
    {
      $N_distribution = get_spam_distribution($uniqueSpam, $words_pos[$i], strlen($data[$i]));
      $N            = $N_distribution[0];
      $distribution = $N_distribution[1];
      if ($N > 1 && $distribution < (0.35 * 0.35))
      {
        $spam_score += floor(2 * (0.35 * 0.35) / $distribution);
      }
      if ($debug) echo '<pre>distribution='.$distribution.', spam_score='.$spam_score.'</pre>';
    }
  }

  if ($debug) echo '<pre>spam_score='.$spam_score.'</pre>';

  if ($spam_score >= 10)
  {
    $reason = tr('Plugin').' spamSimpleFilter '.tr('rejected content').'score='.$spam_score.'('.implode(',', $uniqueSpam).')';
    return false;
  }
}
?>