<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

global $pluginManager;
$pluginManager->addContentBoxHandle('trackback_add_ping',       'addPing');
$pluginManager->addUserActionHandle('trackback_discover',       'onTrackbackDiscover');
$pluginManager->addUserActionHandle('trackback_do_ping',        'onTrackbackPing');
$pluginManager->addUserActionHandle('trackback_del',            'onTrackbackDelete');
$pluginManager->addBrowserTitlePageHandle('trackback_add_ping', 'browserAddTrackback');
$pluginManager->addPluginACL(acl_plugin_start, 'Trackback');

/**
 * \brief display an url to add a trackback from Wiclear to another site
 * \ingroup trackback_template_functions
 *
 * \return html blob
 */
function addTrackbackBox()
{
  global $user;
  global $node;
  if (empty($node) || !auth::canEdit($user, $node))
  {
    return;
  }

  global $pluginManager;
  $pluginManager->loadI18N('trackback');
  return '<a href="'.createURL($node->title, array('lang'=>$node->lang->code, 'mode'=>'trackback_add_ping')).'">'.tr('Add trackback').'</a>';
}

/**
 * \brief creates a trackback ping url
 * \ingroup trackback_template_functions
 *
 * \param node_id id of page
 *
 * \return url
 */
function trackbackPingUrl($node_id)
{
  global $app_path;
  return $app_path.'tools/trackback/trackback.php?id='.$node_id;
}

/**
 * \brief display trackbacks
 * \ingroup trackback_template_functions
 *
 * \return html blob
 */
function trackbackBox()
{
  global $node;
  global $mode;

  if (!empty($mode) || empty($node))
  {
    return '';
  }

  global $pluginManager;
  $pluginManager->loadPlugin('trackback');
  $trackbacks = getTrackbacks($node->id);

  global $user;
  if (auth::aclMatch(true, acl_plugin_start, $user, $node))
  {
    $url = createUrl($node->title, array('lang'=>$node->lang->code));

    $blob =
    '<p>'.tr('To make a trackback, use ping url').' '.trackbackPingUrl($node->id).'</p>'.
    '<p>'.tr("If your client support ping url auto discovery, you can also try it using directly this page's url").'</p>'.
    '<!---'.
    '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"'.
            'xmlns:dc="http://purl.org/dc/elements/1.1/"'.
            'xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">'.
    '<rdf:Description'.
        'rdf:about="'.$url.'"'.
        'dc:identifier="'.$url.'"'.
        'dc:title="'.$node->title.'"'.
        'trackback:ping="'.trackbackPingUrl($node->id).'" />'.
    '</rdf:RDF>'.
    '-->';
  }
  else
  {
    $blob = tr('Trackbacks are not allowed on this post');
  }

  $nbTrackbacks = sizeof($trackbacks);
  if ($nbTrackbacks == 0)
  {
    $blob = '<h1>'.tr('No trackbacks yet').'</h1>'.$blob;
    return $blob;
  }

  $blob = '<h1>'.tr('Trackbacks').'</h1>'.$blob;

  global $user;
  for ($i = 0; $i < $nbTrackbacks; ++$i)
  {
    $trackback = &$trackbacks[$i];
    $id = $trackback->id;

    if ($user->isModerator() || $user->isAdmin())
    {
      $blob .= '<div class="admin">';
      $blob .= '<a class="delete" href="'.createURL($node->title, array('mode'=>'trackback_del', 'id'=>$id, 'lang'=>$node->lang->code)).'">'.tr('Delete').'</a>';
      $blob .= '<a class="ban" href="'.createURL($node->title, array('mode'=>'ban_add', 'ip'=>$trackback->ip, 'lang'=>$node->lang->code, 'anchor'=>'tr'.$id)).'">'.tr('Ban').'</a>';
      $blob .= '</div>';
    }

    $blob .= displayTrackback($trackback->id, $trackback->url, $trackback->blog_name, $trackback->title, $trackback->excerpt, $trackback->timestamp);
  }

  $blob .= '<div class="spacer">&nbsp;</div>';

  return $blob;
}
?>