<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

require wc_inc_path.'/tools/trackback/trackback.class.php';
require wc_inc_path.'/inc/classes/HttpClient.class.php';

/**
 * \brief customize browser title when adding a trackback
 *
 * \return browser title
 */
function browserAddTrackback()
{
  return wc_wiki_name.' - '.tr('Add trackback');
}

/**
 * \brief get trackbacks attached to a page
 *
 * \param id id of page
 *
 * \return array<trackback>
 */
function getTrackbacks($id)
{
  global $wiki;

  $id  = $wiki->con->escapeString($id);

  $query =<<<SQL
    SELECT id, url, blog_name, title, excerpt, timestamp, ip
    FROM  {prefix}trackbacks
    WHERE content_fk='$id'
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);
  $result = $wiki->con->select($query);

  $trackbacks = array();
  foreach ($result as $r)
  {
    $trackbacks[] = new trackback($r[0], $r[1], $r[2], $r[3], $r[4], $r[5], $r[6]);
  }

  return $trackbacks;
}

/**
 * \brief add all successfull pinged urls
 *
 * \param pinged_urls array of url
 * \param id id of page
 */
function addTrackbackPingedUrl($pinged_urls, $id)
{
  global $wiki;

  $id  = $wiki->con->escapeString($id);

  $query =<<<SQL
    INSERT INTO {prefix}pings
    (ping_url, content_fk)
    VALUES
SQL;

  $first = true;
  foreach ($pinged_urls as $url)
  {
    if ($first == false)
    {
      $query .= ',';
    }

    $url = $wiki->con->escapeString($url);
    $query .= "('".$url."','".$id."')";
    $first = false;
  }

  $query = str_replace('{prefix}', $wiki->prefix, $query);
  return $wiki->con->execute($query);
}

/**
 * \brief get pinged urls already done on page
 *
 * \param id id of page
 *
 * \return array<url>
 */
function getPingedUrls($id)
{
  global $wiki;

  $id  = $wiki->con->escapeString($id);

  $query =<<<SQL
    SELECT ping_url
    FROM  {prefix}pings
    WHERE content_fk='$id'
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);
  $result = $wiki->con->select($query);

  $pings = array();
  foreach ($result as $r)
  {
    $pings[] = $r[0];
  }

  return $pings;
}

/**
 * \brief display one trackback
 *
 * \return html blob
 */
function displayTrackback($id, $url, $blog_name, $title, $excerpt, $timestamp)
{
  $excerpt = formatHtml($excerpt);
  $excerpt = str_replace("\n", '<br />', $excerpt);

  return
  '<a name="tr'.$id.'" id="tr'.$id.'"></a>'.
  '<div class="trackback">'.
  '<div class="trackback_meta">'.
  '<h1><a href="'.htmlentities($url, ENT_QUOTES).'">'.formatHtml($title).'</a></h1>'.
  '<p>'.formatDate($timestamp).', '.tr('on').' '.formatHtml($blog_name).'</p>'.
  '</div>'.
  //'<div class="spacer">&nbsp;</div>'.
  $excerpt.
  '</div>';
}

/**
 * \brief deletes a trackback
 *
 * \param id id of trackback to delete
 */
function trackbackDelete($id)
{
  global $wiki;

  $id  = $wiki->con->escapeString($id);

  $query =<<<SQL
    DELETE FROM  {prefix}trackbacks
    WHERE  id='$id'
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);
  return $wiki->con->execute($query);
}

/**
 * \brief called when a trackback needs to be deleted
 */
function onTrackbackDelete()
{
  global $user;
  if (!$user->isModerator() && !$user->isAdmin())
  {
    return;
  }

  if (empty($_GET['id']))
  {
    return;
  }
  $id = varGet('id');

  trackbackDelete($id);

  global $cacheLite;
  $cacheLite->wcClean();

  global $node;
  header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code, 'anchor'=>'trackbacks')));
}

/**
 * \brief display a form to add a trackback to another site, or a report form of auto discovered ping urls
 *
 * \return html blob
 */
function addPing()
{
  global $user;
  global $node;
  if (!auth::canEdit($user, $node))
  {
    return;
  }
  if ($user->isAnonymous() && auth::banned())
  {
    return;
  }

  // extract parameters from session
  $urls = '';
  if (existVarSession('trackback_ping_url'))
  {
    $urls = varSession('trackback_ping_url');
    unsetVarSession('trackback_ping_url');
  }
  $error = '';
  if (existVarSession('trackback_error'))
  {
    $error = varSession('trackback_error');
    unsetVarSession('trackback_error');
  }

  $auto_discover_url = createUrl($node->title, array('lang'=>$node->lang->code, 'mode'=>'trackback_discover'));
  $blob = '<a href="'.$auto_discover_url.'">'.tr('Auto discovery of ping url from wiki content').'</a>';

  $targetUrl = createUrl($node->title, array('lang'=>$node->lang->code, 'mode'=>'trackback_do_ping'));
  $form = new form($targetUrl, 'post', 'trackback');
  $form->addFieldSet();
  $form->addLegend(tr('Auto discovered ping urls'));

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Insert here urls to ping'), 'fping_url');
  $form->addTextArea('ping_url', 10, 10, $urls);
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addButton('ping', tr('Ping urls'), 'submit');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHidden('check', '1');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field"/>');

  $blob .= $form->toHtml();

  if (!empty($error))
  {
    $blob .= '<p>'.tr('Error').' : '.$error.'</p>';
  }

  $pinged_urls = getPingedUrls($node->id);
  if (count($pinged_urls) > 0)
  {
    $blob .= '<p>'.tr('Already pinged urls').'</p><ul>';
    foreach ($pinged_urls as $url)
    {
      $blob .= '<li>'.formatHtml($url).'</li>';
    }
    $blob .= '</ul>';
  }

  return $blob;
}

/**
 * \brief parses url, creates an instance of http client and returns
 *
 * \param url url to handle
 * \param path path on server (out parameter)
 *
 * \return array(error, HttpClient)
 */
function createHttpClientFromUrl($url, &$path)
{
  $http = '';
  $error = '';

  $my_url = $url;
  $url_components = parse_url($my_url);
  if (empty($url_components['scheme']) ||
      ($url_components['scheme'] != 'http' && $url_components['scheme'] != 'https') ||
      empty($url_components['host']) ||
      empty($url_components['path'])
     )
  {
    $error = tr('Could not parse url');
  }
  else
  {
    $host = $url_components['host'];
    $port = !empty($url_components['port']) ? $url_components['port'] : 80;
    $user = !empty($url_components['user']) ? $url_components['user'] : '';
    $pass = !empty($url_components['pass']) ? $url_components['pass'] : '';

    if (preg_match('|.*://[^/]*(/.*)|', $url, $path_match) == 1)
    {
      $path = $path_match[1];
    }
    else
    {
      // for a reason I ignore, the following does not always work, for example,
      // http://wiclear.free.fr/tools/trackback/trackback.php?id=46 is parsed
      // as path = /tools/trackback/trackback.php?id=46_
      // and the trailing '_' is not part as the original url, leading to bugs
      // so this is used only if preg_match does not work for now
      $path  = $url_components['path'];
      $path .= !empty($url_components['query']) ? '?'.$url_components['query'] : '';
      $path .= !empty($url_components['fragment']) ? '#'.$url_components['fragment'] : '';
    }

    $http = new HttpClient($host, $port);
    $http->timeout = 10;
    $http->setUserAgent('Wiclear HTTP Client - http://wiclear.free.fr/');
    $http->useGzip(false);
    $http->setPersistReferers(false);
    if ($user != '' || $pass != '')
    {
      $http->setAuthorization($user, $pass);
    }
    $http->charset = wc_charset;
  }

  return array($error, $http);
}

/**
 * \brief discover a ping url by parsing a web page
 */
function discoverPingUrlFrom($url)
{
  // extract host/port/path from url
  $error         = '';
  $trackbackPing = '';

  $path = '';
  $result = createHttpClientFromUrl($url, $path);
  $error = $result[0];
  if (empty($error))
  {
    $http = $result[1];

    if ($http->get($path))
    {
      if (preg_match_all('/<rdf:RDF.*?<rdf:Description(.*?)<\/rdf:RDF>/smi', $http->content, $rdf_all) > 0)
      {
        foreach ($rdf_all[1] as $rdf)
        {
          // extract dc:identifier and trackback:ping
          $dcIdentifier = '';
          if (preg_match('/dc:identifier="(.*?)"/smi', $rdf, $dcIdentifierMatch))
          {
            $dcIdentifier = $dcIdentifierMatch[1];
          }

          if ($dcIdentifier == $url)
          {
            if (preg_match('/trackback:ping="(.*?)"/smi', $rdf, $trackbackPingMatch))
            {
              $trackbackPing = $trackbackPingMatch[1];
            }

            break;
          }
        }
      }

      if (empty($trackbackPing))
      {
        $error = tr('Could not find a ping url');
      }
    }
    else
    {
      $error = tr('Could not get page from').' '.$url;
    }
  }

  return array($error, $trackbackPing);
}

/**
 * \brief called to discover ping urls directly from wiki page
 */
function onTrackbackDiscover()
{
  global $node;

  $ping_urls = array();

  $url_character_class = '\w\d\.\/\%\-\+\?\&\;\=\:\@';
  // url start with http://, and is composed with url_character_class
  $url_regexp = '/(http:\/\/['.$url_character_class.']+)/mi';
  if (preg_match_all($url_regexp, $node->content, $urls) > 0)
  {
    foreach ($urls[1] as $url)
    {
      $result = discoverPingUrlFrom($url);
      $error  = $result[0];
      if (empty($error))
      {
        $ping_urls[] = $result[1];
      }
    }
  }

  if (count($ping_urls) == 0)
  {
    setVarSession('trackback_error', tr('Could not extract a single ping url from entry'));
    session_write_close();
    header('Location: '.createHeaderURL($node->title, array('mode'=>'trackback_add_ping', 'lang'=>$node->lang->code)));
  }
  else
  {
    setVarSession('trackback_ping_url', implode("\n", $ping_urls));
    session_write_close();
    header('Location: '.createHeaderURL($node->title, array('mode'=>'trackback_add_ping', 'lang'=>$node->lang->code)));
  }
}

/**
 * \brief receives the url to ping and ping
 */
function onTrackbackPing()
{
  if (empty($_POST['check']))
  {
    return;
  }

  global $user;
  global $node;
  if (!auth::canEdit($user, $node))
  {
    return;
  }
  if ($user->isAnonymous() && auth::banned())
  {
    return;
  }

  unsetVarSession('trackback_ping_url');
  unsetVarSession('trackback_error');

  $ping_url = '';
  if (!empty($_POST['ping_url']))
  {
    $ping_url = varPost('ping_url');
  }

  $ping_urls = explode("\n", $ping_url);

  global $wikiRenderer;

  // we have our urls to ping in ping_urls, let's loop over them
  // and complete array errors and pinged_urls
  $errors      = array();
  $pinged_urls = array();
  foreach ($ping_urls as $ping_url)
  {
    $path = '';
    $result = createHttpClientFromUrl($ping_url, $path);
    $error  = $result[0];
    if (empty($error))
    {
      $http = $result[1];

      $entryUrl = createUrl($node->title, array('lang'=>$node->lang->code));
      $preview = $wikiRenderer->render($node->content);
      $preview = strip_tags($preview);

      if (utf8_helper::strlen($preview) > 255)
      {
        $preview = utf8_helper::substr($preview, 0, 252).'...';
      }

      $postData = array(
        'blog_name'=>wc_wiki_name,
        'url'=> $entryUrl,
        'title'=>$node->title,
        'excerpt'=>$preview
      );

      if (wc_charset == 'UTF-8')
      {
        $postData['utf8'] = '1';
      }

      if ($http->post($path, $postData))
      {
        $xmlResult = $http->getContent();

        $pattern = '|<response>.*<error>(.*)</error>.*(<message>(.*)</message>.*)?</response>|msU';

        if (!preg_match($pattern, $xmlResult, $matches))
        {
          if (utf8_helper::strlen($xmlResult) > 256)
          {
            $xmlResult = utf8_helper::substr($xmlResult, 0, 256);
          }
          $error = tr('Source is not a ping URL').' : '.formatHtml($xmlResult);
        }
        else
        {
          $ping_error = $matches[1];
          $ping_msg = (!empty($matches[2])) ? $matches[2] : '';

          if ($ping_error != '0')
          {
            $error = tr('Trackback error').' : '.$ping_msg;
          }
          else
          {
            $pinged_urls[] = $ping_url;
          }
        }
      }
      else
      {
        $error = tr('Could not connect to ping url').' '.$url;
      }
    }

    if (!empty($error))
    {
      $errors[] = $error;
    }
  }

  // add pinged urls to database, and report errors to user
  addTrackbackPingedUrl($pinged_urls, $node->id);
  setVarSession('trackback_error', implode("\n", $errors));

  if (sizeof($pinged_urls) > 0)
  {
    // notify users
    global $wiki;

    $message =
    tr('Ping have been added ').' '.tr('on page').' '.$node->title."\n".
    createHeaderURL($node->title, array('lang'=>$node->lang->code))."\n";

    $email_title = '['.wc_wiki_name.']'.tr('Ping added');

    $emails = $wiki->getEmails($node->id, array($user->id));
    foreach ($emails as $email)
    {
      $result = mail_helper::send(
        wc_mail, $email[0], wc_charset,
        $email_title,
        $message
      );
    }
  }

  session_write_close();
  header('Location: '.createHeaderURL($node->title, array('mode'=>'trackback_add_ping', 'lang'=>$node->lang->code)));
}
?>
