<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

error_reporting(E_ALL);

/**
 * \file
 * \brief receive trackbacks and register them with a page
 */

/**
 * \brief test if trackbacks are active (table installed by admin)
 */
function trackBackActive()
{
  global $con;

  $table_name = wc_db_prefix.'trackbacks';
  return $con->tableExists($table_name);
}

/**
 * \brief test if a similar trackback has already been registered for this node
 *
 * \param id id of page we want this trackback to be added
 * \param url url of trackback to add to page
 */

function trackBackExists($id, $url)
{
  global $con;

  $id  = $con->escapeString($id);
  $url = $con->escapeString($url);

  $query =<<<SQL
    SELECT {prefix}trackbacks.url
    FROM {prefix}trackbacks
    WHERE {prefix}trackbacks.url='$url'
    AND   {prefix}trackbacks.content_fk='$id';
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $result = $con->select1($query);

  return !empty($result);
}

/**
 * \brief add a trackback to a page
 *
 * \param id id of page we want this trackback to be added
 * \param url url of trackback
 * \param blog_name name of blog (or wiki)
 * \param title title of entry that track backs to our page
 * \param excerpt extract of entries
 * \param node_title title of page to add trackback to
 * \param node_lang_code lang code of page to add trackback to
 */
function trackbackAdd($id, $url, $blog_name, $title, $excerpt, $node_title, $node_lang_code)
{
  global $con;

  $id         = $con->escapeString($id);
  $url        = $con->escapeString($url);
  $blog_name  = $con->escapeString($blog_name);
  $title      = $con->escapeString($title);
  $excerpt    = $con->escapeString($excerpt);
  $timestamp  = time();
  $ips        = implode(',', auth::getIPs());

  $query =<<<SQL
    INSERT INTO {prefix}trackbacks
    SET content_fk='$id',
        url='$url',
        blog_name='$blog_name',
        title='$title',
        excerpt='$excerpt',
        timestamp='$timestamp',
        ip='$ips'
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  $sqlResult = $con->execute($query);

  // cache
  require wc_inc_path.'/inc/classes/PEAR/Cache_Lite.class.php';
  require wc_inc_path.'/inc/classes/WiclearCache.class.php';

  $cacheLite = new WiclearCache(user_anonymous_id, '', $node_title, $node_lang_code);
  $cacheLite->wcClean();

  return $sqlResult;
}

/**
 * \brief returns content info from an id
 *
 * \return array(title, lang_code)
 */
function contentInfo($id)
{
  global $con;

  $id  = $con->escapeString($id);

  $query =<<<SQL
    SELECT {prefix}content.title, {prefix}lang.code
    FROM {prefix}content
    JOIN {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
    WHERE {prefix}content.id='$id'
SQL;

  $query = str_replace('{prefix}', wc_db_prefix, $query);
  return $con->select1($query);
}

/**
 * \brief extract all subdomains of an host
 *
 * \param hosts list of hosts to complete
 * \param host host to extract subdomain from
 *
 * \return new list of sub domains extracted
 */
function collectSubDomains(&$hosts, $host)
{
  $new_hosts = array();

  if (!in_array($host, $hosts))
  {
    $hosts[] = $new_hosts[] = $host;
  }

  $dot_pos = strpos($host, '.');
  while ($dot_pos !== false)
  {
    $subdomain = substr($host, $dot_pos + 1);
    $next_dot_pos = strpos($host, '.', $dot_pos + 1);
    if ($next_dot_pos !== false)
    {
      if (!in_array($subdomain, $hosts))
      {
        $hosts[] = $new_hosts[] = $subdomain;
      }
    }
    $dot_pos = $next_dot_pos;
  }

  return $new_hosts;
}

/**
 * \brief get more ips from a list of hosts
 *
 * \param $ips the list of ips to complete
 * \param $hosts list of hosts to extract ips from
 *
 * \return new added ips
 */
function completeIpsFromHosts(&$ips, $hosts)
{
  $new_ips = array();

  foreach ($hosts as $host)
  {
    $host_ips = gethostbynamel($host);
    if ($host_ips !== false)
    {
      foreach ($host_ips as $ip)
      {
        if (!in_array($ip, $ips))
        {
          $ips[] = $new_ips[] = $ip;
        }
      }
    }
  }

  return $new_ips;
}

/**
 * \brief get more hosts from a list of ips
 *
 * \param $hosts the list of $hosts to complete
 * \param $ips list of $ips to resolve hosts from
 *
 * \return new added hosts
 */
function completeHostsFromIps(&$hosts, $ips)
{
  $new_hosts = array();

  foreach ($ips as $ip)
  {
    $host = gethostbyaddr($ip);
    if ($host != $ip && !in_array($host, $hosts))
    {
      $new_hosts = collectSubDomains($hosts, $host);
    }
  }

  return $new_hosts;
}

function display($str, $array)
{
  $text = $str.' : ';
  foreach ($array as $v)
  {
    $text .= $v.',';
  }
  global $errors;
  $errors[] = $text;
}

/**
 * \brief checks poster of trackback uses same ip as url being posted
 */
function checkIP($url)
{
  global $errors;

  $url_components = parse_url($url);
  if (!isset($url_components['host']))
  {
    $errors[] = 'Invalid url.';
    $host     = '';
  }
  else
  {
    $host = $url_components['host'];
  }

  // collect al subdomains 
  // somesubdomain.somedomain.com => [somesubdomain.somedomain.com, somedomain.com]
  $site_hosts = array();
  collectSubDomains($site_hosts, $host);
  $site_ips = array();

  // collect all known ips of client
  $poster_ips = auth::getIPs();
  $poster_hosts = array();

  $legitimate = false;

  $new_site_hosts = $site_hosts; // begin iteration
  $new_poster_ips = $poster_ips; // begin iteration
  while (!empty($new_site_hosts) || !empty($new_poster_ips))
  {
    // get more ips from known hosts
    $new_site_ips     = completeIpsFromHosts($site_ips, $new_site_hosts);
    // and vice versa
    $new_site_hosts   = completeHostsFromIps($site_hosts, $new_site_ips);

    // get more hosts from ips
    $new_poster_hosts = completeHostsFromIps($poster_hosts, $new_poster_ips);
    // and vice versa
    $new_poster_ips   = completeIpsFromHosts($poster_ips, $new_poster_hosts);

/*
    display("site_ips", $site_ips);
    display("site_hosts", $site_hosts);
    display("poster_ips", $poster_ips);
    display("poster_hosts", $poster_hosts);
*/

    // any intersection ?
    $inter_ips   = array_intersect($site_ips, $poster_ips);
    $inter_hosts = array_intersect($site_hosts, $poster_hosts);

/*
    display("inter_ips", $inter_ips);
    display("inter_hosts", $inter_hosts);
*/

    if (count($inter_ips) != 0 || count($inter_hosts) != 0)
    {
      // one ip of poster match one ip of site : legitimate trackback
      // or one resolved host of poster match one resolved host of site : legitimate trackback
      $legitimate = true;
      break;
    }
  }

  if (!$legitimate)
  {
    // ip of url is not related to ip that post this trackback. Spam anyone ?
    $errors[] = "You appear not to post this trackback from the url you're referring.";
  }
}

function checkContent($title, $excerpt)
{
  global $pluginManager;

  $data = array($title, $excerpt);
  $reason = '';

  if ($pluginManager->validatePluginCallback('on_trackback', $data, $reason) === false)
  {
    global $errors;
    $errors[] = $reason;
  }
}

// main begin here
$path      = dirname(__FILE__).'/../../';
// let's use a constant instead of a variable so that hacker can't use it to inject something using a require
// on an unprotected directory
define('wc_inc_path', $path);
unset($path);

require wc_inc_path.'/inc/lib/constants.lib.php';

// conf
require wc_inc_path.'/inc/classes/WiclearIni.conf.php';
require wc_inc_path.'/inc/classes/ini.class.php';
$ini = new ini(wc_inc_path.'/conf/config.ini');
$ini->importConstants();

@ini_set('display_errors','on');
@ini_set('default_charset',wc_charset);
@ini_set('mbstring.func_overload','7');
@ini_set('mbstring.internal_encoding',wc_charset);
@ini_set('mbstring.internal_encoding',wc_charset);

// url
require wc_inc_path.'/inc/lib/url.lib.php';
$app_path = createWebAppPath();

require wc_inc_path.'/inc/classes/chrono.class.php';

// db
require wc_inc_path.'/inc/classes/db_connection.class.php';
require wc_inc_path.'/inc/classes/mysql.class.php';
$con = new mysql(wc_db_host, wc_db_login, wc_db_password, wc_db_database);

// wiki
require wc_inc_path.'/inc/classes/wiki.class.php';
$wiki = new wiki($con, wc_db_prefix);

// variables
require wc_inc_path.'/inc/lib/variables.lib.php';

// auth
require wc_inc_path.'/inc/classes/auth.class.php';

// helper
require wc_inc_path.'/inc/classes/utf8_helper.class.php';
require wc_inc_path.'/inc/classes/mail_helper.class.php';

// translation
require wc_inc_path.'/inc/classes/translation.class.php';

// user
require wc_inc_path.'/inc/classes/user.class.php';
$user = $wiki->anonymousUser();

// plugin manager
require wc_inc_path.'inc/classes/plugin_manager.class.php';
$pluginManager = new PluginManager(wc_inc_path);
$pluginManager->importPlugins();

// list of errors
$errors = array();

if (auth::banned())
{
  // IP was banned
  $errors[] = "You're banned";
}

// extract variables from the url
$id = 0;
if (isset($_GET['id']))
{
  $id = varGet('id');
}
else
{
  // id is mandatory
  $errors[] = 'Missing id of page in get arguments.';
}

$contentInfo = contentInfo($id);
if ($contentInfo === false)
{
  // content id '$id' does not exist
  $errors[] = 'No page for id '.$id.'.';
}
else
{
  $node_title     = $contentInfo[0];
  $node_lang_code = $contentInfo[1];
}

if ($wiki->existAclOnIdAndType($id, acl_plugin_start))
{
  // acls
  $errors[] = 'Trackbacks are not allowed on this post';
}

// extract variables from the form
$blog_name = '';
if (isset($_POST['blog_name']))
{
  $blog_name = varPost('blog_name');
}
$url       = '';
if (isset($_POST['url']))
{
  $url = varPost('url');
}
else
{
  // url is mandatory
  $errors[] = 'Missing url in trackback arguments.';
}

checkIP($url);

$title     = '';
if (isset($_POST['title']))
{
  $title = varPost('title');
}
$excerpt   = '';
if (isset($_POST['excerpt']))
{
  $excerpt = varPost('excerpt');
}

checkContent($title, $excerpt);

$trackbackActive = trackBackActive();
if ($trackbackActive == false)
{
  $errors[] = 'Trackbacks are not active yet.';
}

// let's verify if we don't have already this trackback
if ($trackbackActive && trackBackExists($id, $url))
{
  $errors[] = 'Trackback already registered.';
}

//header('Content-Type: text/xml');
echo '<?xml version="1.0" encoding="'.wc_charset.'"?>'."\n";
if (count($errors) > 0)
{
  // errors were found
/* debug
foreach ($errors as $error)
{
  echo '<pre>'.$error.'</pre>';
}*/
?>
<response>
  <error>1</error>
  <message><?php echo implode("\n\t\t",$errors) ?></message>
</response><?php
}
else
{
  // valid trackback, try to complete missing informations
  if (empty($blog_name) && preg_match('/^(http:\/\/)?([^\/]+)/i', $url, $matches) == 1)
  {
    $blog_name = $matches[2];
  }
/* test
echo '<pre>id='.$id.'</pre>';
echo '<pre>blog_name='.$blog_name.'</pre>';
echo '<pre>url='.$url.'</pre>';
echo '<pre>title='.$title.'</pre>';
echo '<pre>excerpt='.$excerpt.'</pre>';
echo '<pre>trackbackActive='.$trackbackActive.'</pre>';
*/

  $content_type_charset = 'ISO-8859-1';
  $charset_decoded = false;
  //test
  //$_SERVER["CONTENT_TYPE"] = "Content-Type: application/x-www-form-urlencoded; charset=utf-8";
  if (isset($_SERVER['CONTENT_TYPE']))
  {
    $content_type = $_SERVER['CONTENT_TYPE'];
    if (preg_match('/^.+charset=(.+)/i', $content_type, $matches) == 1)
    {
      $content_type_charset = strtoupper($matches[1]);
      $charset_decoded = true;
    }
  }

  if (!$charset_decoded && isset($_POST['utf8']))
  {
    // dotclear compatibility hack
    $utf8 = varPost('utf8');
    if ($utf8 == 1)
    {
      $content_type_charset = 'UTF-8';
      $charset_decoded = true;
    }
  }

  // strip all tags as we want valid xhtml and we're only interested in content
  // this also avoids having spam linking back to spam sites
  // this also avoids the substr() call below to accidently cut the string in the middle of a tag
  $excerpt = strip_tags($excerpt);

  if (utf8_helper::strlen($excerpt) > 255)
  {
    $excerpt = utf8_helper::substr($excerpt, 0, 252).'...';
  }

  if (!$charset_decoded && utf8_helper::is_utf8($excerpt))
  {
    $content_type_charset = 'UTF-8';
    $charset_decoded = true;
  }

  if (function_exists('utf8_encode') && wc_charset == 'UTF-8' && $content_type_charset != 'UTF-8')
  {
    // if we're configured in UTF-8 and excerpt is not in UTF-8
    // then encode it to UTF-8
    $title     = utf8_encode($title);
    $blog_name = utf8_encode($blog_name);
    $excerpt   = utf8_encode($excerpt);
  }
  else
  if (function_exists('utf8_decode') && wc_charset != 'UTF-8' && $content_type_charset == 'UTF-8')
  {
    // if we're not configured in UTF-8 and excerpt is in UTF-8
    // then decode it from UTF-8
    $title     = utf8_decode($title);
    $blog_name = utf8_decode($blog_name);
    $excerpt   = utf8_decode($excerpt);
  }

  // add trackback in database
  trackbackAdd($id, $url, $blog_name, $title, $excerpt, $node_title, $node_lang_code);

  // notify users
  $message =
  'A trackback has been added  on page '.$node_title."\n".
  createHeaderURL($node_title, array('lang'=>$node_lang_code))."\n";

  $email_title = '['.wc_wiki_name.']Trackback added';

  $emails = $wiki->getEmails($id);
  foreach ($emails as $email)
  {
    $result = mail_helper::send(
      wc_mail, $email[0], wc_charset,
      $email_title,
      $message
    );
  }

  // no errors, generate xml response
?>
<response>
  <error>0</error>
</response><?php
}
?>