<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief customize browser title when displaying content by archive
 *
 * \return browser title
 */
function browserArchive()
{
  if (!isset($_GET['archive']))
  {
    return;
  }
  $category = varGet('archive');

  return wc_wiki_name.' - '.tr('Display by archive').' : '.$archive;
}

function createWhereClause($firstTimestamp, $lastTimestamp)
{
  global $user;

  $whereClause = '{prefix}content.timestamp BETWEEN '.$firstTimestamp.' AND '.$lastTimestamp;
  if (!$user->isModerator() && !$user->isAdmin())
  {
    $acl_read = acl_read;

    // create a subquery that returns all nodes containing a read acl
    $subQuery1 = <<<SQL
      SELECT wc_content_group.content_fk
      FROM wc_content_group
      WHERE wc_content_group.acl_type='$acl_read'
SQL;

    // create a subquery that returns all nodes where user has explicit read access
    $userId = $user->id;
    $subQuery2 = <<<SQL
      SELECT wc_content_group.content_fk
      FROM   wc_content_group
      JOIN   wc_user_group on wc_user_group.group_fk=wc_content_group.group_fk
      WHERE  wc_user_group.user_fk='$userId'
      AND    wc_content_group.acl_type='$acl_read'
SQL;

    $whereClause = '({prefix}content.id NOT IN ('.$subQuery1.') OR {prefix}content.id IN ('.$subQuery2.')) AND '.$whereClause;
  }

  return $whereClause;
}

/**
 * \brief retrieve list of last changements in month (like recentChanges)
 *
 * \param month month to look at
 * \param year year to look at
 * \return array containing results in row with title, date, code, first name, last name, content
 */
function recentChangesByMonth($month, $year)
{
  global $wiki;
  global $user;

  $month = $wiki->con->escapeString($month);
  $year  = $wiki->con->escapeString($year);

  $firstTimestamp = mktime(0, 0, 0, $month, 1, $year);
  $lastTimestamp  = mktime(23, 59, 59, $month, date_helper::daysInMonth($month, $year) - 1, $year);

  $whereClause = createWhereClause($firstTimestamp, $lastTimestamp);

  $query =<<<SQL
  SELECT   {prefix}content.title, {prefix}content.alternate_title, {prefix}content.timestamp, {prefix}lang.code,
           {prefix}user.firstname, {prefix}user.lastname,
           {prefix}content.content
  FROM     {prefix}content
  JOIN     {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
  JOIN     {prefix}user on {prefix}user.id={prefix}content.user_fk
  WHERE    $whereClause
  ORDER BY {prefix}content.timestamp DESC
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);

  return $wiki->con->select($query);
}

/**
 * \brief retrieve list of last changements in year (like recentChanges)
 *
 * \param year year to look at
 * \return array containing results in row with title, date, code, first name, last name, content
 */
function recentChangesByYear($year)
{
  global $wiki;

  $year  = $wiki->con->escapeString($year);

  $firstTimestamp = mktime(0, 0, 0, 1, 1, $year);
  $lastTimestamp  = mktime(23, 59, 59, 12, 31, $year);

  $whereClause = createWhereClause($firstTimestamp, $lastTimestamp);

  $query =<<<SQL
  SELECT   {prefix}content.title, {prefix}content.alternate_title, {prefix}content.timestamp, {prefix}lang.code,
           {prefix}user.firstname, {prefix}user.lastname,
           {prefix}content.content
  FROM     {prefix}content
  JOIN     {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
  JOIN     {prefix}user on {prefix}user.id={prefix}content.user_fk
  WHERE    $whereClause
  ORDER BY {prefix}content.timestamp DESC
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);

  return $wiki->con->select($query);
}

/**
 * \brief display all entries by archive
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function displayBlogEntriesByArchive()
{
  if (isset($_GET['month']))
  {
    $month = varGet('month');
    $date  = getdate();
    $year = $date['year'];
    $recentChanges = recentChangesByMonth($month, $year);
    if (empty($recentChanges))
    {
      $blob = '<h1>'.tr('No entries in month').' '.formatMonth($month - 1).'</h1>';
    }
    else
    {
      $blob = '<h1>'.tr('Entries in month').' '.formatMonth($month - 1).'</h1>';
    }
  }

  if (isset($_GET['year']))
  {
    $year = varGet('year');
    $recentChanges = recentChangesByYear($year);
    if (empty($recentChanges))
    {
      $blob = '<h1>'.tr('No entries in year').' '.$year.'</h1>';
    }
    else
    {
      $blob = '<h1>'.tr('Entries in year').' '.$year.'</h1>';
    }
  }

  $blob .= displayEntries($recentChanges);

  return $blob;
}
?>