<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#

/**
 * \brief customize browser title when displaying content with the calendar
 *
 * \return browser title
 */
function browserCalendar()
{
  if (!isset($_GET['day']) || !isset($_GET['month']) || !isset($_GET['year']))
  {
    return;
  }
  $day   = varGet('day');
  $month = varGet('month');
  $year  = varGet('year');

  $date = mktime(0, 0, 0, $month, $day, $year);

  return wc_wiki_name.' - '.tr('Display with calendar').' : '.formatDate($date);
}

/**
 * \brief retrieve list of date of blog in month
 *
 * \param month month to look at
 * \param year year to look at
 * \return array<day>
 */
function blogEntriesInMonth($month, $year)
{
  global $wiki;

  $month = $wiki->con->escapeString($month);
  $year  = $wiki->con->escapeString($year);

  $firstTimestamp = mktime(0, 0, 0, $month, 1, $year);
  $lastTimestamp  = mktime(23, 59, 59, $month, date_helper::daysInMonth($month - 1, $year) - 1, $year);

  $query =<<<SQL
  SELECT   {prefix}content.timestamp
  FROM     {prefix}content
  WHERE    {prefix}content.timestamp BETWEEN $firstTimestamp AND $lastTimestamp
  ORDER BY {prefix}content.timestamp ASC
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);

  $dates = $wiki->con->select($query);
  $result = array();
  $nbDates = sizeof($dates);
  for ($i = 0; $i < $nbDates; ++$i)
  {
    $date = getdate($dates[$i][0]);
    $day = $date['mday'];
    if (sizeof($result) == 0 || $day != $lastDay)
    {
      $result[] = $day;
      $lastDay  = $day;
    }
  }

  return $result;
}

/**
 * \brief retrieve list of last changements in day (like recentChanges)
 *
 * \param day day to look at
 * \param month month to look at
 * \param year year to look at
 * \return array containing results in row with title, date, code, first name, last name, content
 */
function recentChangesByDay($day, $month, $year)
{
  global $wiki;

  $day   = $wiki->con->escapeString($day);
  $month = $wiki->con->escapeString($month);
  $year  = $wiki->con->escapeString($year);

  $firstTimestamp = mktime(0, 0, 0, $month, $day, $year);
  $lastTimestamp  = mktime(23, 59, 59, $month, $day, $year);

  $query =<<<SQL
  SELECT   {prefix}content.title, {prefix}content.alternate_title, {prefix}content.timestamp, {prefix}lang.code,
           {prefix}user.firstname, {prefix}user.lastname,
           {prefix}content.content
  FROM     {prefix}content
  JOIN     {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
  JOIN     {prefix}user on {prefix}user.id={prefix}content.user_fk
  WHERE    {prefix}content.timestamp BETWEEN $firstTimestamp AND $lastTimestamp
  ORDER BY {prefix}content.timestamp DESC
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);

  return $wiki->con->select($query);
}

/**
 * \brief display entries at specified day
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function displayBlogEntriesByCalendar()
{
  if (!isset($_GET['month']) || !isset($_GET['year']))
  {
    return;
  }

  $month = varGet('month');
  $year  = varGet('year');

  if (isset($_GET['day']))
  {
    $day   = varGet('day');

    $recentChanges = recentChangesByDay($day, $month, $year);
    $dateStr = formatDate(mktime(0, 0, 0, $month, $day, $year));
    if (empty($recentChanges))
    {
      $blob = '<h1>'.tr('No entries in day').' '.$dateStr.'</h1>';
    }
    else
    {
      $blob = '<h1>'.tr('Entries in day').' '.$dateStr.'</h1>';
    }
  }
  else
  {
    // display entries in month
    $recentChanges = recentChangesByMonth($month, $year);
    if (empty($recentChanges))
    {
      $blob = '<h1>'.tr('No entries in month').' '.formatMonth($month - 1).'</h1>';
    }
    else
    {
      $blob = '<h1>'.tr('Entries in month').' '.formatMonth($month - 1).'</h1>';
    }
  }

  $blob .= displayEntries($recentChanges);

  return $blob;
}

?>
