<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief customize browser title when displaying content by category
 *
 * \return browser title
 */
function browserCategory()
{
  if (!isset($_GET['category']))
  {
    return;
  }
  $category = varGet('category');

  return wc_wiki_name.' - '.tr('Display by category').' : '.$category;
}

/**
 * \brief remove current category of current page
 */
function removeCurrentCategory()
{
  global $wiki;
  global $node;

  $node_id = $node->id;

  $query =<<<SQL
    DELETE {prefix}metadata_content
    FROM {prefix}metadata_content
    INNER JOIN {prefix}metadata
    WHERE {prefix}metadata_content.metadata_fk={prefix}metadata.id
    AND   {prefix}metadata.name="category"
    AND   {prefix}metadata_content.content_fk='$node_id'
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);

  $wiki->con->execute($query);
}

/**
 * \brief handles category change from wikiBlogCategorySelect() method
 */
function onChangeCategory()
{
  if (!isset($_POST['check']) || !isset($_POST['category_change']) || !isset($_POST['category']))
  {
    return;
  }

  global $node;
  global $wiki;
  $category_id = varPost('category');

  $metadatas = $wiki->getMetaDatas();
  removeCurrentCategory();
  $metadata_id = 0;
  foreach ($metadatas as $metadata)
  {
    if ($metadata->name == 'category')
    {
      $metadata_id = $metadata->id;
      $wiki->addContentMetadataClosedValue($node->id, $metadata_id, $category_id);
      break;
    }
  }

  header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code)));
}

/**
 * \brief retrieve list of last changements (like recentChanges) feature by category
 *
 * \param category category name
 * \return array containing results in row with title, date, code, first name, last name, content
 */
function recentChangesByCategory($category)
{
  global $wiki;
  global $user;

  $category = $wiki->con->escapeString($category);

  $whereClause = 
  '{prefix}metadata.name="category" AND {prefix}metadata_value.value='."'".$category."'";

  if (!$user->isModerator() && !$user->isAdmin())
  {
    $acl_read = acl_read;

    // create a subquery that returns all nodes containing a read acl
    $subQuery1 = <<<SQL
      SELECT wc_content_group.content_fk
      FROM wc_content_group
      WHERE wc_content_group.acl_type='$acl_read'
SQL;

    // create a subquery that returns all nodes where user has explicit read access
    $userId = $user->id;
    $subQuery2 = <<<SQL
      SELECT wc_content_group.content_fk
      FROM   wc_content_group
      JOIN   wc_user_group on wc_user_group.group_fk=wc_content_group.group_fk
      WHERE  wc_user_group.user_fk='$userId'
      AND    wc_content_group.acl_type='$acl_read'
SQL;

    $whereClause = '({prefix}content.id NOT IN ('.$subQuery1.') OR {prefix}content.id IN ('.$subQuery2.')) AND '.$whereClause;
  }

  $query =<<<SQL
  SELECT {prefix}content.title, {prefix}content.alternate_title, {prefix}content.timestamp, {prefix}lang.code,
         {prefix}user.firstname, {prefix}user.lastname,
         {prefix}content.content
  FROM   {prefix}content
  JOIN   {prefix}lang on {prefix}content.lang_fk={prefix}lang.id
  JOIN   {prefix}user on {prefix}content.user_fk={prefix}user.id
  JOIN   {prefix}metadata_content on {prefix}content.id={prefix}metadata_content.content_fk
  JOIN   {prefix}metadata_value on {prefix}metadata_content.metadata_value_fk={prefix}metadata_value.id
  JOIN   {prefix}metadata on {prefix}metadata_value.metadata_fk={prefix}metadata.id
  WHERE  $whereClause
  ORDER BY timestamp DESC
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);

  return $wiki->con->select($query);
}

/**
 * \brief display all entries by category
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function displayBlogEntriesByCategory()
{
  if (!isset($_GET['category']))
  {
    return;
  }
  $category = varGet('category');

  $recentChanges = recentChangesByCategory($category);
  $blob = displayEntries($recentChanges);

  if (empty($blob))
  {
    $blob = tr('No entries in category').' '.$category;
  }
  else
  {
    $blob = '<h1>'.tr('Entries in category').' '.$category.'</h1>'.$blob;
  }

  return $blob;
}

/**
 * \brief get the list of authorized values for closed metadata named "category"
 *
 * \return map<id, value>
 */
function getCategories()
{
  global $wiki;

  $query =<<<SQL
    SELECT {prefix}metadata_value.id, {prefix}metadata_value.value
    FROM {prefix}metadata_value, {prefix}metadata
    WHERE {prefix}metadata.name='category'
    AND   {prefix}metadata_value.metadata_fk={prefix}metadata.id
SQL;

  $query = str_replace('{prefix}', $wiki->prefix, $query);
  $sqlResult = $wiki->con->select($query);

  $result = array();
  foreach ($sqlResult as $r)
  {
    $result[$r[0]] = $r[1];
  }

  return $result;
}
?>
