<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

global $pluginManager;
// archive
$pluginManager->addContentBoxHandle('archive',        'displayBlogEntriesByArchive');
$pluginManager->addBrowserTitlePageHandle('archive',  'browserArchive');
// calendar
$pluginManager->addContentBoxHandle('calendar',        'displayBlogEntriesByCalendar');
$pluginManager->addBrowserTitlePageHandle('calendar',  'browserCalendar');
// category
$pluginManager->addContentBoxHandle('category',        'displayBlogEntriesByCategory');
$pluginManager->addUserActionHandle('category_change', 'onChangeCategory');
$pluginManager->addBrowserTitlePageHandle('category',  'browserCategory');

/**
 * \brief activate wikiblog plugin, must be included in user's theme (prepend.inc.php is a good place)
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogSetup()
{
  global $mainPageHandler;
  global $mainPage;
  global $mode;
  global $contentBoxHandle;
  global $browserTitlePageHandle;

  $mainPageHandler = '';
  $contentBoxHandle['default']       = 'displayBlogEntries';
  $browserTitlePageHandle['default'] = 'browserBlog';

  if ($mainPage == true && empty($mode))
  {
    // hack to disable some entries
    $mode = 'default';
  }
  if ($mode == 'category' || $mode == 'archive')
  {
    $mainPage = true;
  }
}

/**
 * \brief customize browser title when displaying blog
 *
 * \return browser title
 */
function browserBlog()
{
  global $title;

  if (isset($_GET['title']))
  {
    return browserContentDisplay();
  }
  else
  {
    return wc_wiki_name;
  }
}

/**
 * \brief display wiki menu
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogWikiMenu()
{
  global $mainPage;

  if ($mainPage == true)
  {
    $addContentLink = addContentLink();

    if (!empty($addContentLink))
    {
      return '<ul><li>'.$addContentLink.'</li></ul>';
    }
  }
  else
  {
    return wikiMenu();
  }
}

/**
 * \brief display a tool box
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogTools()
{
  global $app_path;
  global $mainPage;

  $blob =
  '<h1>'.tr('Tools').'</h1>'.
  '<ul>'.
  '<li><a href="'.$app_path.'">Home</a></li>';

  if ($mainPage == false)
  {
    $whatLinksHere = whatLinksHereBox();
    if (!empty($whatLinksHere))
    {
      $blob .= '<li>'.$whatLinksHere.'</li>';
    }

    $attachmentsBox = attachmentsBox();
    if (!empty($attachmentsBox))
    {
      $blob .= '<li>'.$attachmentsBox.'</li>';
    }
  }

  $blob .= '<li>'.browseAllAttachmentsBox().'</li>';

  // if plugin trackback is installed
  if (function_exists('addTrackbackBox'))
  {
    $addTrackbackBox = addTrackbackBox();
    if (!empty($addTrackbackBox))
    {
      $blob .= '<li>'.$addTrackbackBox.'</li>';
    }
  }

  $blob .=
  '<li>'.sitePlanBox().'</li>'.
  '</ul>';

  return $blob;
}

/**
 * \brief display feeds
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogFeeds()
{
  global $mainPage;

  $blob =
  '<h1>'.tr('Syndication').'</h1>'.
  '<ul>'.
  '<li><a href="'.feedRecentChangesUrl().'">'.tr('Rss thread').'</a></li>'.
  '<li><a href="'.feedRecentCommentsUrl().'">'.tr('Rss comments').'</a></li>';

  if ($mainPage == false)
  {
    $blob .=
    '<li><a href="'.feedLocalChangesUrl().'">'.tr('Local Rss history').'</a></li>'.
    '<li><a href="'.feedLocalCommentsUrl().'">'.tr('Local Rss comments').'</a></li>';
  }

  $blob .= '</ul>';

  return $blob;
}

/**
 * \brief display parent content box
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogParentContentBox()
{
  global $mainPage;

  if ($mainPage == false)
  {
    return parentContentBox();
  }
}

/**
 * \brief display sub content box
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogSubContentBox()
{
  global $mainPage;

  if ($mainPage == false)
  {
    return '<h1>'.tr('Sub nodes').'</h1>'.subContentBox();
  }
}

/**
 * \brief display comment areas
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogCommentArea()
{
  global $mainPage;

  $blob = '';

  if ($mainPage == false)
  {
    $blob =
    '<div id="comment_link">'.commentLink().'</div>'.
    '<div id="comments">'.commentBox().'</div>';
  }

  return $blob;
}

/**
 * \brief display trackback areas
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogTrackbackArea()
{
  global $mainPage;

  $blob = '';

  if ($mainPage == false && function_exists('trackbackBox'))
  {
    $blob = '<div id="trackbacks">'.trackbackBox().'</div>';
  }

  return $blob;
}

/**
 * \brief display an html list containing all archive available
 * \ingroup wikiblog_template_functions
 *
 * \return html unordered list
 */
function wikiblogArchiveBox()
{
  $date  = getdate();
  $month = $date['mon'] - 1;
  $year  = $date['year'];

  $blob = '<ul>';
  for ($i = $month; $i >= 0; --$i)
  {
    $blob .= '<li><a href="'.createUrl('', array('mode'=>'archive', 'month'=>($i+1))).'">'.formatMonth($i).'</a></li>';
  }
  $blob .= '<li><a href="'.createUrl('', array('mode'=>'archive', 'year'=>$year - 1)).'">'.($year - 1).'</a></li>';
  $blob .= '<li><a href="'.createUrl('', array('mode'=>'archive', 'year'=>$year - 2)).'">'.($year - 2).'</a></li>';

  $blob .= '</ul>';

  return $blob;
}

/**
 * \brief display a calendar
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogCalendarBox()
{
  global $pluginManager;
  $pluginManager->loadPlugin('wiki_blog');

  $date  = getdate();

  $month = $date['mon'] - 1;
  $currentMonth = $month;
  $year  = $date['year'];
  $currentYear = $year;

  global $mode;
  if ($mode == 'calendar' && isset($_GET['month']))
  {
    $month = varGet('month') - 1;
  }
  if ($mode == 'calendar' && isset($_GET['year']))
  {
    $year = varGet('year');
  }

  // previous month url in calendar
  $previousMonth = $month - 1;
  $previousYear  = $year;
  if ($previousMonth < 0)
  {
    $previousMonth = 11;
    $previousYear  = $year - 1;
  }
  $previousMonthUrl = createUrl('', array('mode'=>'calendar', 'month'=>$previousMonth+1, 'year'=>$previousYear));
  $blob = '<table><caption><a href="'.$previousMonthUrl.'">&lt;&lt;</a>';

  // current month/year
  $blob .= formatMonth($month).' '.$year;

  // eventually add next month url in calendar 
  if ($month != $currentMonth)
  {
    $nextMonth = $month + 1;
    $nextYear  = $year;
    if ($nextMonth > 11)
    {
      $nextMonth = 0;
      $nextYear  = $year + 1;
    }
    $nextMonthUrl = createUrl('', array('mode'=>'calendar', 'month'=>$nextMonth+1, 'year'=>$nextYear));
    $blob .= '<a href="'.$nextMonthUrl.'">&gt;&gt;</a>';
  }

  // now display the calendar
  $blob .= '</caption><thead><tr>';

  for ($i = 0; $i < 7; ++$i)
  {
    $blob .= '<th>'.formatSmallDay($i).'</th>';
  }
  $blob .= '</tr></thead><tbody><tr>';

  // now iterates over all days in month, finding the column to put it in
  $current_date = getdate(mktime(0, 0, 0, $month + 1, 1, $year));
  // getdate returns wday with 0=sun, 6=sat
  // and we want 0=mon, 6=sun => we add 6 and use a modulo to have this result
  $dayInWeek = ($current_date['wday'] + 6) % 7;

  //echo '<pre>original dayInWeek='.$current_date["wday"].'</pre>';
  //echo '<pre>dayInWeek='.$dayInWeek."=".formatDay($dayInWeek).'</pre>';

  // padd the right side of the week with empty cells
  for ($j = 0; $j < $dayInWeek; ++$j)
  {
    $blob .= '<td></td>';
  }

  $nbDaysInMonth = date_helper::daysInMonth($month, $year);
  $blogDates = blogEntriesInMonth($month + 1, $year);
  $blogDatesIt = 0;

  for ($i = 1; $i <= $nbDaysInMonth; ++$i)
  {
    $cellContent = $i;

    if ($blogDatesIt < count($blogDates))
    {
      if ($blogDates[$blogDatesIt] == $i)
      {
        // this day we have a blog entry : make a link
        $dayUrl = createUrl('', array('mode'=>'calendar', 'day'=>$i, 'month'=>$month+1, 'year'=>$year));
        $cellContent = '<a href="'.$dayUrl.'">'.$i.'</a>';
        ++$blogDatesIt;
      }
    }

    $blob .= '<td>'.$cellContent.'</td>';

    ++$dayInWeek;
    if ($dayInWeek == 7)
    {
      $dayInWeek = 0;
      if ($i < $nbDaysInMonth)
      {
        // start a new line
        $blob .= '</tr><tr>';
      }
    }
  }

  // padd the left side of the week with empty cells
  if ($dayInWeek != 0)
  {
    for (; $dayInWeek < 7; ++$dayInWeek)
    {
      $blob .= '<td></td>';
    }
  }

  $blob .= '</tr></tbody></table>';

  return $blob;
}

/**
 * \brief display a box to select a category for current page
 * \ingroup wikiblog_template_functions
 *
 * \return html blob
 */
function wikiblogCategorySelectBox()
{
  global $mainPage;
  global $mode;

  $blob = '';
  if ($mainPage == false && empty($mode))
  {
    global $pluginManager;
    $pluginManager->loadI18N('wiki_blog');

    $blob =
    '<div id="category_select">'.
    '<h1>'.tr('Current Category').'</h1>'.
    wikiBlogCategorySelect().
    '</div>';
  }

  return $blob;
}

/**
 * \brief display a form that enables to change current category of a particular page
 * \ingroup wikiblog_template_functions
 *
 * The form is displayed if the user has the rights to change it.
 * Otherwise, the current category is displayed
 *
 * \return html form
 */
function wikiBlogCategorySelect()
{
  global $node;
  global $wiki;
  global $user;

  if (empty($node))
  {
    return;
  }

  global $pluginManager;
  $pluginManager->loadPlugin('wiki_blog');

  // array[array[id, name, value_id, open_value, closed_value_id]]
  $nodeMetadatas = $wiki->getMetaDataInfoByContentId($node->id);
  $mapIdToCategory = getCategories();

  $currentCategory = 0;
  foreach ($nodeMetadatas as $metadata)
  {
    if ($metadata[1] == 'category')
    {
      $currentCategory = $metadata[4];
    }
  }

  $options = array();
  $options['--'] = 0;
  $categoryName = '--';
  foreach ($mapIdToCategory as $id=>$category)
  {
    $options[$category] = $id;
    if ($currentCategory == $id)
    {
      $categoryName = $category;
    }
  }

  if (auth::canEdit($user, $node))
  {
    $targetUrl = createUrl($node->title, array('lang'=>$node->lang->code, 'mode'=>'category_change'));
    $form = new form($targetUrl, 'post');
    $form->addDropDown($options, $currentCategory, 'category');
    $form->addButton('category_change', tr('Change'));
    $form->addHidden('check', 1);

    return $form->toHtml();
  }
  else
  {
    return $categoryName;
  }
}

/**
 * \brief display an html list containing all categories available
 * \ingroup wikiblog_template_functions
 *
 * \return html unordered list
 */
function wikiblogCategoryBox()
{
  global $pluginManager;
  $pluginManager->loadPlugin('wiki_blog');

  $mapIdToCategory = getCategories();

  $blob = '';
  if (!empty($mapIdToCategory))
  {
    $blob = '<ul>';
    foreach ($mapIdToCategory as $id=>$category)
    {
      $blob .= '<li><a href="'.createUrl('', array('mode'=>'category', 'category'=>$category)).'">'.$category.'</a></li>';
    }
    $blob .= '</ul>';
  }

  return $blob;
}
?>