<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

require wc_inc_path.'tools/wiki_blog/category.lib.php';
require wc_inc_path.'tools/wiki_blog/archive.lib.php';
require wc_inc_path.'tools/wiki_blog/calendar.lib.php';

/**
 * \brief display all entries to create a summary
 *
 * \param $recentChanges array the list of recent changes
 *
 * \return html blob
 */
function displayEntries($recentChanges)
{
  global $wikiRenderer;
  $wikiRenderer->config->showImage = false;

  $currentYear = 0;
  $currentMonth = 0;
  $currentDay = 0;

  $blob = '';
  foreach ($recentChanges as $change)
  {
    $title      = $change[0];
    $alt_title  = $change[1];
    $date       = $change[2];
    $lang       = $change[3];
    $first_name = $change[4];
    $last_name  = $change[5];
    $content    = $change[6];

    $year  = date('y', $date);
    $month = date('m', $date);
    $day   = date('d', $date);

    if ($day != $currentDay || $month != $currentMonth || $year != $currentYear)
    {
      $currentDay   = $day;
      $currentMonth = $month;
      $currentYear  = $year;

      $entriesDate = mktime(0, 0, 0, $month, $day, $year);
      $blob .= '<hr/><h1 class="date">'.formatDate($entriesDate).'</h1>';
    }

    $lang_object = new lang(0, $lang, '', '');

    $node = new node(
      0, $title, $alt_title, $content, '', major_value, '', $date, $lang_object,
      0, 0, 0, 0,
      0, '', $first_name, $last_name, ''
    );
    $node->langs[0] = $lang_object;
    $blob .= $node->toHtml(false, false);
    $url = createUrl($node->title, array('lang'=>$node->lang->code));
    $blob .= '<a class="content_link" href="'.$url.'">'.tr('See comments...').'</a>';
  }
  return $blob;
}

/**
 * \brief display all entries to create a summary, or a particular page
 *
 * \return html blob
 */
function displayBlogEntries()
{
  global $wiki;
  global $title;
  global $user;

  if (isset($_GET['title']))
  {
    return contentDisplay();
  }
  else
  {
    //array[title, date, lang, author, content]
    $recentChanges = $wiki->recentChanges(0, 20, true, true, $user->id);

    return displayEntries($recentChanges);
  }
}
?>
