<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004 Grzesiek, adapted by David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

error_reporting(E_ALL);

@ini_set('session.use_cookies','1');
@ini_set('session.use_only_cookies','1');
@ini_set('url_rewriter.tags','');
@ini_set('session.use_trans_sid','0');
@ini_set('display_errors','on');
@ini_set('mbstring.func_overload','7');

$file_path = __FILE__;
$path      = dirname(__FILE__).'/';
// let's use a constant instead of a variable so that hacker can't use it to inject something using a require
// on an unprotected directory
define('wc_inc_path', $path);
unset($path);

require wc_inc_path.'/inc/lib/constants.lib.php';

// conf
require wc_inc_path.'/inc/classes/WiclearIni.conf.php';
require wc_inc_path.'/inc/classes/ini.class.php';
$ini = new ini(wc_inc_path.'/conf/config.ini');
$ini->importConstants();

require wc_inc_path.'/inc/lib/variables.lib.php';

require wc_inc_path.'/inc/lib/url.lib.php';

/**
 * \brief create a full path from a file and a mode
 *
 * \param file file to load (relative to wc_icons or images)
 * \param mode "icon" or "image"
 * \brief resolved full path. If the file is an image, it is simply resolved, else, if the mime type is known, the file is replaced with an icon
 */
function resolveFileModeToImage($file, $mode)
{
  if ($mode == 'icon')
  {
    $file = createIconPath($file);
  }
  else
  {
    $file = createImagePath($file);
  }

  if (file_exists($file) == false)
  {
    // file does not exist, use broken icon instead
    $file = createIconPath(wc_icon_broken);
  }
  $extensions = explode('.', $file);
  $extension = strtoupper(array_pop($extensions));

  // find an icon matching the extension except if it's an image
  // we will generate a thumb directly if that's the case
  $handled_extension = array(
    'GIF'  => '*',
    'PNG'  => '*',
    'JPG'  => '*',
    'JPEG' => '*',
    'PDF'  => wc_icon_pdf,
    'MP3'  => wc_icon_music,
    'OGG'  => wc_icon_music,
    'MPEG' => wc_icon_video,
    'MPG'  => wc_icon_video
  );

  $icon = wc_icon_unknown;
  if (isset($handled_extension[$extension]))
  {
    $icon = $handled_extension[$extension];
  }

  if ($icon != '*')
  {
    // generate a thumb from the icon instead of the file
    $file = createIconPath($icon);
  }

  return $file;
}

/**
 * \brief load an image and scale it according to width/height
 *
 * \param file the image to load
 * \param width the width to give to the image
 * \param height the height to give to the image
 * \param scaled image. Ratio is kept
 */
function &renderImage($file, $width, $height)
{
  $extensions = explode('.', $file);
  $extension = strtoupper(array_pop($extensions));

  // load image
  if ($extension == 'GIF')
  {
    $img = ImageCreateFromGif($file);
  }
  else
  if ($extension == 'PNG')
  {
    $img = ImageCreateFromPng($file);
  }
  else
  if ($extension == 'JPG' || $extension == 'JPEG')
  {
    $img = ImageCreateFromJpeg($file);
  }

  // compute size
  $ow = ImageSX($img);
  $oh = ImageSY($img);
  if ($width == -1)
  {
    $maxw = $ow;
  }
  else
  {
    $maxw = $width;
  }
  if ($height == -1)
  {
    $maxh = $oh;
  }
  else
  {
    $maxh = $height;
  }

  $new_h = $oh;
  $new_w = $ow;

  if (($oh > $maxh || $ow > $maxw) || ($oh < $maxh && $ow < $maxw))
  {
    // shrink or grow to maxh/maxws
    $new_h = ($oh > $ow) ? $maxh : $oh * ($maxw / $ow);
    $new_w = $new_h / $oh * $ow;
  }

  // create dst image
  $dst_img = ImageCreateTrueColor($new_w, $new_h);
  // resize and copy image
  ImageCopyResized($dst_img, $img, 0, 0, 0, 0, $new_w, $new_h, $ow, $oh);

  return $dst_img;
}

// get args from url
if (!isset($_GET['thumb']))
{
  return;
}

$file = varGet('thumb');
$id = $file;

$mode = '';
if (isset($_GET['mode']))
{
  $mode = varGet('mode');
  $id .= '-'.$mode;
}
$width = -1;
if (isset($_GET['width']))
{
  $width = varGet('width');
  $id .= '-'.$width;
}
$height = -1;
if (isset($_GET['height']))
{
  $height = varGet('height');
  $id .= '-'.$height;
}

// cache
require wc_inc_path.'/inc/classes/PEAR/Cache_Lite.class.php';

$cacheDir = wc_inc_path.wc_cache_path.'/';

if (!file_exists($cacheDir))
{
  @mkdir($cacheDir);
}

 // Set a few options
$options = array(
  'cacheDir' => $cacheDir,
  'lifeTime' => 60*60*24
);
$cacheLite = new Cache_Lite($options);

if (wc_activate_cache == "true" && ($data = $cacheLite->get($id)) !== false)
{
  // cache hit
}
else
{
  $file = resolveFileModeToImage($file, $mode);
  $img  = &renderImage($file, $width, $height);

  ob_start();
  ImagePng($img);
  $data = ob_get_contents();
  ob_end_clean();
  $cacheLite->save($data, $id);
}

header('Content-type: image/png');
echo $data;
?>
